/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <ctype.h>                                     // to declare tolower()
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

// struct to describe a for loop
struct ForControl {
  Value lvalue;
  long begin;
  long increment;
  long end;
  ForControl() : lvalue(), begin(0), increment(1), end(0) {}
  ForControl(const Value &lv,
      const Value &b, const Value &i,
      const Value &e)
  : lvalue(lv),
    begin(b.getLong()),
    increment(i.getLong()),
    end(e.getLong())
  {
    // Nothing to do
  }
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146346
#include "dxi.h"
#endif

#ifndef DXI_H_1031146346
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 544, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

ForControl dxi_pcb_struct::forControl(const Value &lv, const Value &b, const Value &i, const Value &e) {
  if (mode == executeMode) return ForControl(lv, b, i, e);
  return ForControl();
}

void dxi_pcb_struct::forLoop(ForControl &fc) {
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  if (fc.increment == 0) reportError("Loop increment is zero");
  // Calculate number of times through the loop
  int count = (fc.end + fc.increment - fc.begin)/fc.increment;
  int index = fc.begin;
  // Set mode for parsing loop
  loopPCB.mode = (mode == executeMode && count > 0) ? executeMode : breakMode;
  if (loopPCB.mode == executeMode) fc.lvalue = index;
  while (count--) {
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    index += fc.increment;
    if (loopPCB.mode == executeMode) fc.lvalue = index;
    if (loopPCB.mode == breakMode) return;
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  return pcb.returnValue;
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 200, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return Value();
}

static inline void ag_rp_5(PCB_DECL) {
/* Line 207, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 210, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 213, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 216, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 239, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 241, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 245, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 259, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.simpleLoop("repeatLoop");
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 260, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_14(PCB_DECL, ForControl &fc) {
/* Line 261, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.forLoop(fc);
}

static inline void ag_rp_15(PCB_DECL, Value &x) {
/* Line 265, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_16(PCB_DECL) {
/* Line 273, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static void ag_rp_17(PCB_DECL, Value &x) {
/* Line 284, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static void ag_rp_18(PCB_DECL, Value &x) {
/* Line 288, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
if (PCB.mode != executeMode || x.isTrue()) PCB.mode = breakMode;
}

static inline ForControl ag_rp_19(PCB_DECL, Value &lv, Value &b, Value &i, Value &e) {
/* Line 292, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return PCB.forControl(lv, b, i, e);
}

static inline Value ag_rp_20(PCB_DECL) {
/* Line 295, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(1);
}

static inline Value ag_rp_21(PCB_DECL, Value &x) {
/* Line 296, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x;
}

static void ag_rp_22(PCB_DECL, Value &x) {
/* Line 301, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_23(PCB_DECL) {
/* Line 305, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_24(PCB_DECL, Value &lv, Value &x) {
/* Line 308, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  VALUE(lv = x);
}

static inline void ag_rp_25(PCB_DECL, AgString &n) {
/* Line 315, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_26(PCB_DECL, AgString &n) {
/* Line 316, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_27(PCB_DECL, Value &x) {
/* Line 320, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_28(PCB_DECL, Value &x) {
/* Line 321, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  PCB.print(x);
}

static inline Value ag_rp_29(PCB_DECL) {
/* Line 324, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return Value();
}

static inline Value ag_rp_30(PCB_DECL, Value &x, Value &y) {
/* Line 330, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_31(PCB_DECL, Value &x, Value &y) {
/* Line 331, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_32(PCB_DECL, Value &x, Value &y) {
/* Line 332, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_33(PCB_DECL, Value &x, Value &y) {
/* Line 333, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_34(PCB_DECL, Value &x, Value &y) {
/* Line 334, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_35(PCB_DECL, Value &x, Value &y) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_36(PCB_DECL, Value &x, Value &y) {
/* Line 339, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x + y);
}

static inline Value ag_rp_37(PCB_DECL, Value &x, Value &y) {
/* Line 340, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x - y);
}

static inline Value ag_rp_38(PCB_DECL, Value &x, Value &y) {
/* Line 341, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x | y);
}

static inline Value ag_rp_39(PCB_DECL, Value &x, Value &y) {
/* Line 342, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x ^ y);
}

static inline Value ag_rp_40(PCB_DECL, Value &x, Value &y) {
/* Line 346, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x * y);
}

static inline Value ag_rp_41(PCB_DECL, Value &x, Value &y) {
/* Line 347, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x.rdiv(y));
}

static inline Value ag_rp_42(PCB_DECL, Value &x, Value &y) {
/* Line 348, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x.idiv(y));
}

static inline Value ag_rp_43(PCB_DECL, Value &x, Value &y) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x % y);
}

static inline Value ag_rp_44(PCB_DECL, Value &x, Value &y) {
/* Line 350, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x & y);
}

static inline Value ag_rp_45(PCB_DECL, Value &x, Value &y) {
/* Line 351, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x << y);
}

static inline Value ag_rp_46(PCB_DECL, Value &x, Value &y) {
/* Line 352, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x >> y);
}

static inline Value ag_rp_47(PCB_DECL, Value &x) {
/* Line 356, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_48(PCB_DECL, Value &x) {
/* Line 357, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_49(PCB_DECL, Value &x) {
/* Line 358, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_50(PCB_DECL, Value &x, Value &y) {
/* Line 366, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_51(PCB_DECL, Value &x) {
/* Line 374, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x;
}

static inline Value ag_rp_52(PCB_DECL, Value &lv) {
/* Line 376, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_53(PCB_DECL, Value &x) {
/* Line 378, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_54(PCB_DECL, Value &x) {
/* Line 379, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_55(PCB_DECL, AgString &n) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_56(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 385, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(callFunction(n, args));
}

static inline AgStack<Value> ag_rp_57(PCB_DECL) {
/* Line 388, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_58(PCB_DECL, Value &x) {
/* Line 392, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_59(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 393, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_60(PCB_DECL, long x) {
/* Line 396, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_61(PCB_DECL, double x) {
/* Line 397, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_62(PCB_DECL, int x) {
/* Line 399, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_63(PCB_DECL, int c) {
/* Line 418, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_64(PCB_DECL, AgString &ns, int c) {
/* Line 419, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_65(PCB_DECL, double x, long e) {
/* Line 424, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_66(PCB_DECL, double x, long e) {
/* Line 425, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_67(PCB_DECL, double i, double f) {
/* Line 428, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return i+f;
}

static inline double ag_rp_68(PCB_DECL, double i) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return i;
}

static inline double ag_rp_69(PCB_DECL, double f) {
/* Line 430, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return f;
}

static inline double ag_rp_70(PCB_DECL, long x) {
/* Line 433, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x;
}

static inline double ag_rp_71(PCB_DECL, long x) {
/* Line 434, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x;
}

static inline double ag_rp_72(PCB_DECL, long x) {
/* Line 435, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_73(PCB_DECL, long x) {
/* Line 438, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return (long)x;
}

static inline long ag_rp_74(PCB_DECL, long x) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_75(PCB_DECL, int d) {
/* Line 442, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_76(PCB_DECL, int d, double f) {
/* Line 443, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_77(PCB_DECL, int d) {
/* Line 451, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return d-'0';
}

static inline long ag_rp_78(PCB_DECL, long x, int d) {
/* Line 452, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_79(PCB_DECL, int d) {
/* Line 455, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return d-'0';
}

static inline long ag_rp_80(PCB_DECL, long x, int d) {
/* Line 456, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_81(PCB_DECL, long x, int d) {
/* Line 459, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_82(PCB_DECL, long x, int d) {
/* Line 460, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_83(PCB_DECL) {
/* Line 463, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 0;
}

static inline long ag_rp_84(PCB_DECL, long n, int d) {
/* Line 464, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_85(PCB_DECL) {
/* Line 467, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 0;
}

static inline long ag_rp_86(PCB_DECL, long n, int d) {
/* Line 468, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_87(PCB_DECL, int x) {
/* Line 471, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return x - '0';
}

static inline int ag_rp_88(PCB_DECL, int d) {
/* Line 472, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_89(PCB_DECL, Value &s, Value &e) {
/* Line 478, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_90(PCB_DECL, AgString &b) {
/* Line 481, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_91(PCB_DECL) {
/* Line 484, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_92(PCB_DECL, AgString &s, int c) {
/* Line 485, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_93(PCB_DECL) {
/* Line 497, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\'';
}

static inline int ag_rp_94(PCB_DECL) {
/* Line 498, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '"';
}

static inline int ag_rp_95(PCB_DECL) {
/* Line 499, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\?';
}

static inline int ag_rp_96(PCB_DECL) {
/* Line 500, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\\';
}

static inline int ag_rp_97(PCB_DECL) {
/* Line 501, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\a';
}

static inline int ag_rp_98(PCB_DECL) {
/* Line 502, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\b';
}

static inline int ag_rp_99(PCB_DECL) {
/* Line 503, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\f';
}

static inline int ag_rp_100(PCB_DECL) {
/* Line 504, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\n';
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 505, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\r';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 506, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\t';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 507, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return '\v';
}

static inline int ag_rp_104(PCB_DECL, int d) {
/* Line 513, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return d-'0';
}

static inline int ag_rp_105(PCB_DECL, int n, int d) {
/* Line 516, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_106(PCB_DECL, int n, int d) {
/* Line 519, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_107(PCB_DECL, int d) {
/* Line 522, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return d;
}

static inline int ag_rp_108(PCB_DECL, int n, int d) {
/* Line 523, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_109(PCB_DECL, int d) {
/* Line 526, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return d-'0';
}

static inline int ag_rp_110(PCB_DECL, int d) {
/* Line 527, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_111(PCB_DECL, int c) {
/* Line 536, W:/agdev/agex/packages/xidek/pll/base/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0,
  0, 0, 0, 0, 0, 4, 5, 0, 0, 0, 0, 0, 5, 0, 0, 0, 5, 5, 0, 0, 0, 0, 0,
  5, 0, 0, 0, 5, 0, 5, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 6, 6, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 5,
  0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<ForControl >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  0,  0,  0,  0,  6,  7,  8,  0,  9,
   10, 11,  0,  0,  0, 12,  0,  0,  0, 13,  0,  0, 14, 15, 16, 17, 18, 19,
   20, 21,  0, 22, 23, 24, 25, 26,  0,  0, 27, 28, 29, 30, 31, 32,  0, 33,
   34, 35, 36,  0, 37, 38, 39, 40, 41, 42, 43,  0, 44, 45, 46,  0, 47, 48,
    0, 49,  0, 50, 51, 52, 53, 54,  0, 55, 56, 57, 58,  0, 59,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0, 60, 61,  0, 62, 63, 64, 65, 66, 67, 68,
   69,  0,  0, 70, 71, 72, 73,  0,  0,  0, 74, 75, 76, 77, 78, 79, 80, 81,
   82, 83, 84, 85,  0, 86, 87, 88, 89,  0,  0,  0,  0,  0, 90, 91, 92, 93,
   94, 95, 96, 97, 98, 99,100,  0,  0,  0,101,102,103,104,105,106,107,108
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,160,  0, 20,  0, 21,  0, 22,  0, 23,  0,165,  0,166,  0,167,
  0,168,  0,169,  0,170,  0,172,  0,174,  0,173,  0,175,  0,177,
  0,176,  0,179,  0,181,  0,190,  0,191,  0,194,  0,195,  0,196,
  0,197,  0,198,  0,199,  0,203,  0,204,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 67, 84,255, 47, 82, 83, 87,255, 42,255, 42, 47,255, 61,
   62,255, 69, 89,255, 85,255, 73, 79, 85,255, 76, 78,255, 76,255, 84,255,
   65,255, 69,255, 80, 84,255, 69,255, 76, 82,255, 67, 72, 84,255, 72, 79,
  255, 76,255, 69,255, 76,255, 73,255, 72,255, 42, 47, 48, 58, 60, 62, 65,
   66, 68, 69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85, 87, 88,255, 67,
   84,255, 82, 83, 87,255, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 70,
   73, 80, 82, 87,255, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 70, 73,
   80, 82, 85, 87,255, 42, 47,255, 47, 48, 78,255, 80, 84,255, 69,255, 66,
   68, 70, 73, 80, 82, 87,255, 80, 84,255, 69,255, 66, 68, 70, 73, 80, 82,
   85, 87,255, 48, 78,255, 42, 47,255, 61, 62,255, 42, 47, 58, 60, 62,255,
   42, 47,255, 47,255, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 69, 70,
   73, 80, 82, 87,255, 42, 47,255, 76, 78,255, 80, 84,255, 69,255, 47, 66,
   68, 69, 70, 73, 80, 82, 85, 87,255, 80, 84,255, 69,255, 66, 68, 69, 70,
   73, 80, 82, 87,255, 58,255, 68,255, 34, 39, 63, 65, 66, 70, 78, 82, 84,
   86, 88, 92,255, 92,255, 42, 47,255, 61, 62,255, 73, 79,255, 76, 82,255,
   72,255, 72, 79,255, 42, 47, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255,
   42, 47,255, 61, 62,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60, 62,
   77, 79, 83, 84, 88,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72,
   79,255, 42, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 47,
   48, 68, 76, 78,255, 48, 68, 76, 78,255, 61, 62,255, 73, 79,255, 76, 82,
  255, 72,255, 72, 79,255, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 61,
   62,255, 72, 79,255, 60, 62, 66, 68, 79, 84, 88,255, 84,255, 76, 78,255,
   80, 84,255, 69,255, 66, 68, 69, 70, 73, 80, 82, 85, 87,255, 42, 47,255,
   61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 48,255, 72, 79,255, 66, 68, 79,
   84, 88,255, 66, 84,255, 84,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,2,7,4,3,4,0,0,4,0,0,4,7,5,4,7,4,7,6,7,4,7,7,4,7,4,6,
  4,2,4,2,4,2,7,4,2,4,5,5,4,7,2,7,4,7,5,4,7,4,6,4,2,4,2,4,2,4,3,2,3,3,2,
  3,7,2,2,2,7,7,7,7,7,7,7,2,2,2,7,2,7,4,7,7,4,7,2,7,4,0,0,4,7,7,4,2,4,2,
  7,7,7,7,7,2,7,4,0,0,4,7,7,4,2,4,2,7,7,7,7,7,2,7,7,4,0,0,4,2,3,7,4,7,7,
  4,2,4,7,7,7,7,7,2,7,4,7,7,4,2,4,7,7,7,7,7,2,7,7,4,3,7,4,0,0,4,0,0,4,3,
  2,3,2,3,4,0,0,4,2,4,0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,2,7,4,0,0,4,7,7,4,7,
  7,4,2,4,2,7,7,2,7,7,7,2,7,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,4,3,4,7,4,0,0,
  0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,2,3,7,
  7,2,7,7,2,2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,7,4,0,0,4,
  7,5,4,5,5,4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,7,4,3,7,7,
  7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,7,5,4,2,3,
  7,7,7,2,7,4,7,4,7,7,4,7,7,4,2,4,7,7,2,7,7,7,2,7,7,4,0,0,4,0,0,4,7,5,4,
  5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,7,7,
  4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 91, 96,  0,  2,  8,  0,  0,  6,  0,  4,  0, 95,  0, 91, 96,  0,183,
  187,  0, 30, 22,  0, 56,  0, 42,  0, 34,  0, 28, 32,  0,  6,  0, 12,  0,
    0,  0,  0,  0,  0, 10,  0,  0,  0, 48, 50,  0,  2,  0,  8,  0, 26, 20,
    0,  4,  0, 16,  0,  0,  0,  0,  0,  0,  0,200,  0,121,171,  0,185, 46,
    0,  0,  0, 18, 24, 54, 44, 52, 38, 36,  0,  0,  0, 14,  0, 40,  0,  2,
    8,  0,  6,  0,  4,  0, 91, 96,  0, 12, 10,  0,  0,  0,  0, 30, 34, 18,
   24, 36,  0, 16,  0, 91, 96,  0, 12, 10,  0,  0,  0,  0, 30, 34, 18, 24,
   36,  0, 14, 16,  0, 91, 96,  0,  0,121, 52,  0, 12, 10,  0,  0,  0, 30,
   34, 18, 24, 36,  0, 16,  0, 12, 10,  0,  0,  0, 30, 34, 18, 24, 36,  0,
   14, 16,  0,121, 52,  0, 91, 96,  0,183,187,  0,200,  0,171,  0,185,  0,
   91, 96,  0,  0,  0, 91, 96,  0, 12, 10,  0,  0,  0,  0, 30, 34, 32, 18,
   24, 36,  0, 16,  0, 91, 96,  0, 28, 32,  0, 12, 10,  0,  0,  0,  0, 30,
   34,  0, 18, 24, 36,  0, 14, 16,  0, 12, 10,  0,  0,  0, 30, 34, 32, 18,
   24, 36,  0, 16,  0,171,  0,  0,  0,133,132,134,136,137,138,139,140,141,
  142,147,135,  0,  0,  0, 91, 96,  0,183,187,  0, 42,  0,  0, 48, 50,  0,
    0,  0, 26, 20,  0,200,  0,  0,185, 46, 22,  0, 44, 38,  0,  0, 40,  0,
   91, 96,  0,183,187,  0, 48, 50,  0,  0,  0, 26, 20,  0,200,  0,  0,185,
   44, 38,  0,  0, 40,  0,183,187,  0, 42,  0,  0, 48, 50,  0,  0,  0, 26,
   20,  0,200,  0,185, 46, 22,  0, 44, 38,  0,  0, 40,  0, 91, 96,  0,  0,
  121, 56, 54, 52,  0,121, 56, 54, 52,  0,183,187,  0, 42,  0,  0, 48, 50,
    0,  0,  0, 26, 20,  0,  0,185, 46, 22,  0, 44, 38,  0,  0, 40,  0,183,
  187,  0, 26, 20,  0,  0,185, 22,  0, 38,  0, 40,  0, 26,  0, 28, 32,  0,
   12, 10,  0,  0,  0, 30, 34,  0, 18, 24, 36,  0, 14, 16,  0, 91, 96,  0,
  183,187,  0, 42,  0,  0, 48, 50,  0,  0,  0, 26, 20,  0,200,  0,121,  0,
  185, 46, 22,  0, 44, 38,  0,  0, 40,  0,121,  0, 26, 20,  0, 22,  0, 38,
    0, 40,  0, 22, 20,  0, 20,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 10, 15,  0,  1,  0,  4, 23,  0, 32,  0,  0,  0,  0,  0,
    0,  0, 45,  0,  0, 51,  0, 49, 23, 55,  0, 58, 61,  0, 85,  0, 32,  0,
   34,  0, 36,  0, 38, 89,  0, 40,  0,  0,  0,  0, 93, 45, 98,  0,106,  0,
    0,114,  0, 55,  0, 57,  0, 59,  0, 61,  0, 34, 14, 36, 38, 17, 40, 42,
   20, 25, 29, 63, 66, 68, 72, 75, 78, 80, 43, 48, 52,109, 63,118,  0,131,
  136,  0,121, 89,144,  0,  0,  0,  0,172,176,  0, 99,  0, 96,153,158,162,
  165,167,102,180,  0,  0,  0,  0,204,208,  0,116,  0,113,185,190,194,197,
  199,119,212,217,  0,  0,  0,  0,131,222,224,  0,246,250,  0,138,  0,227,
  232,236,239,241,141,254,  0,278,282,  0,151,  0,259,264,268,271,273,154,
  286,291,  0,296,298,  0,  0,  0,  0,  0,  0,  0,301,168,303,171,305,  0,
    0,  0,  0,180,  0,  0,  0,  0,329,333,  0,188,  0,185,307,312,316,319,
  322,324,191,337,  0,  0,  0,  0,351,354,  0,366,370,  0,209,  0,203,342,
  347,206,356,359,361,212,374,379,  0,406,410,  0,225,  0,384,389,393,396,
  399,401,228,414,  0,419,  0,421,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,243,  0,  0,  0,  0,  0,  0,  0,432,  0,  0,  0,  0,  0,
  267,  0,439,  0,  0,423,258,261,425,427,430,264,434,437,270,272,442,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,294,  0,454,  0,  0,445,288,291,447,
  449,452,297,299,457,  0,  0,  0,  0,469,  0,  0,  0,  0,  0,318,  0,476,
    0,  0,460,312,462,464,467,315,471,474,321,323,479,  0,  0,  0,  0,338,
  482,484,490,494,  0,497,499,505,509,  0,  0,  0,  0,519,  0,  0,  0,  0,
    0,358,  0,526,  0,  0,352,512,514,517,355,521,524,361,363,529,  0,  0,
    0,  0,540,  0,  0,377,532,534,536,538,380,543,  0,546,  0,559,562,  0,
  574,578,  0,396,  0,550,555,393,564,567,569,399,582,587,  0,  0,  0,  0,
    0,  0,  0,603,  0,  0,  0,  0,  0,420,  0,610,  0,  0,592,411,594,414,
  596,598,601,417,605,608,423,425,613,  0,616,  0,624,  0,  0,618,620,622,
  444,627,  0,630,632,  0,634,  0
};

static const unsigned short ag_key_index[] = {
    7,  0, 12, 65, 92,  0,  0, 12, 12,104,121,134,104,143,156,165,143,174,
  183,134,183,193,214,214,165,  0,134,230,214,239,  0,134,  0,  0,165,  0,
  241,165,143,156,256,  0,  0,256,275,  0,275,302,275,275,275,275,275,326,
  326,341,347,134,134,134,326,165,165,165,366,383,241,143,230,134,165,239,
  134,165,  0,  0,214,391,401,134,165,256,275,  0,  0,256,256,256,  0,275,
    0,  0,165,183,  0,183,  0,  0,134,165,134,165,134,165,134,165,134,165,
  134,165,134,165,134,165,134,165,134,165,165,165,134,165,134,165,134,165,
  134,165,134,165,134,165,143,214,165,104,104,143,  0,  0,  0,  0,  0,428,
  442,442,366,366,366,366,447,447,447,447,447,447,453,275,275,165,134,165,
  456,134,165
};

static const unsigned char ag_key_ends[] = {
69,80,69,65,84,76,79,79,80,0, 82,73,80,84,0, 
65,84,69,77,69,78,84,0, 72,73,76,69,76,79,79,80,0, 47,0, 42,0, 
88,0, 61,0, 61,0, 78,68,0, 71,73,78,0, 86,0, 66,76,69,0, 
77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 79,78,71,0, 79,68,0, 
79,84,0, 82,0, 82,73,78,84,0, 79,79,80,0, 85,82,78,0, 
82,73,80,84,0, 65,84,69,77,69,78,84,0, 69,78,0, 78,84,73,76,0, 
79,79,80,0, 79,82,0, 69,80,69,65,84,76,79,79,80,0, 82,73,80,84,0, 
65,84,69,77,69,78,84,0, 72,73,76,69,76,79,79,80,0, 69,71,73,78,0, 
85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 88,0, 79,84,0, 69,71,73,78,0, 85,77,80,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 88,0, 79,84,0, 42,0, 61,0, 61,0, 69,71,73,78,0, 
85,77,80,0, 78,68,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 
68,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 61,0, 79,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 
78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 
89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 69,71,73,78,0, 
85,77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 78,84,73,76,0, 72,73,76,69,0, 42,0, 
88,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 88,0, 89,0, 79,0, 82,0, 69,78,0, 79,82,0, 89,0, 
79,0, 79,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   13,153,153,153,153,153,153,153,153,152,100,152,152,152,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,152,153,124,153,
  153,153,153,149,202,201,192,188,180,189,107,193,119,154,154,154,154,154,
  154,154,118,118,153,162,182,186,184,153,153,155,155,155,155,104,155,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,153,146,153,153,156,153,155,155,155,155,104,155,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
152,100,96,91,23,22,21,20,0,1,157,158,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,149,146,124,
  119,118,107,104,100,0,98,99,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,149,146,124,
  119,118,107,104,100,95,0,93,94,
152,100,96,91,0,1,
23,22,21,20,0,7,8,9,10,11,159,161,163,164,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,149,146,124,
  119,118,107,104,0,
100,0,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,149,146,124,
  119,118,107,104,100,0,
95,0,
181,179,177,174,169,168,166,165,162,156,155,152,104,100,96,91,0,1,157,158,
181,179,177,174,169,168,167,166,165,162,156,155,152,104,100,96,91,0,1,157,
  158,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
181,179,177,174,169,168,166,165,162,156,155,152,104,100,96,91,13,0,1,157,
  158,
181,179,177,174,169,168,166,165,162,156,155,104,0,4,16,19,24,25,26,28,29,30,
  31,32,33,34,36,37,38,40,41,42,44,49,52,54,56,178,
181,179,177,174,169,168,167,166,165,162,156,155,104,0,12,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,14,43,
  44,57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,
  206,207,208,
181,179,177,174,169,168,166,165,162,156,155,104,13,0,12,
202,201,200,193,192,189,188,187,186,185,184,183,182,180,171,162,156,155,154,
  152,119,118,104,100,96,91,0,1,157,158,
156,155,152,104,100,96,91,0,1,157,158,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
156,155,152,104,100,96,91,0,1,157,158,
181,179,177,176,174,169,168,166,165,162,156,155,152,104,100,96,91,0,1,157,
  158,
181,179,177,176,175,174,169,168,167,166,165,162,156,155,152,104,100,96,91,
  13,0,1,157,158,
181,179,177,176,175,174,169,168,167,166,165,162,156,155,152,104,100,96,91,
  13,0,1,157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
156,155,104,0,4,178,
202,199,189,188,162,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,
  1,157,158,
181,179,177,176,174,169,168,166,165,162,156,155,104,0,12,
181,179,177,176,175,174,169,168,167,166,165,162,156,155,152,104,100,96,91,
  13,0,1,157,158,
171,0,45,
156,155,104,0,4,44,178,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
180,162,0,19,55,
180,162,0,19,55,
202,199,189,188,162,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,35,
  43,44,57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,
  205,206,207,208,
162,0,19,
160,0,15,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
181,179,177,174,169,168,166,165,162,156,155,104,0,4,16,19,24,25,26,28,29,30,
  31,32,33,34,36,37,38,40,41,42,44,49,52,54,56,178,
181,179,177,174,169,168,167,166,165,162,156,155,104,0,4,16,17,19,24,25,26,
  28,29,30,31,32,33,34,36,37,38,39,40,41,42,44,49,52,54,56,178,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,149,147,146,
  142,141,140,139,138,137,136,135,134,133,132,124,119,118,107,104,0,125,
154,119,118,0,114,
154,119,118,0,108,114,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,147,146,142,
  141,140,139,138,137,136,135,134,133,132,124,119,118,107,104,0,128,129,
  130,131,143,144,145,150,
201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,
  180,173,172,170,162,160,152,124,100,96,91,0,1,157,158,
107,104,0,
104,0,
155,154,119,118,104,0,122,
201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,
  180,173,172,170,162,160,154,152,119,118,107,104,100,96,91,0,
201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,
  180,173,172,170,162,160,154,152,119,118,107,104,100,96,91,0,114,
201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,
  180,173,172,170,162,160,152,100,96,91,0,1,157,158,
201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,
  180,173,172,170,162,160,152,100,96,91,0,1,157,158,
201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,
  180,173,172,170,162,160,152,100,96,91,0,1,157,158,
202,0,81,
124,0,5,208,
204,203,202,201,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,
  96,91,0,1,157,158,
204,203,202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,
  43,44,57,64,65,66,69,77,78,79,81,83,84,85,86,89,103,106,109,110,111,115,
  178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
200,0,80,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
198,197,196,195,194,193,192,0,70,71,72,73,74,75,76,
191,190,189,188,187,186,185,184,183,182,0,58,59,60,61,62,63,65,66,67,68,
160,0,15,
181,179,177,174,169,168,166,165,162,156,155,104,13,0,4,16,19,24,25,26,28,29,
  30,31,32,33,34,36,37,38,40,41,42,44,49,52,54,56,178,
181,179,177,176,174,169,168,166,165,162,156,155,104,0,4,16,19,24,25,26,28,
  29,30,31,32,33,34,36,37,38,40,41,42,44,49,52,53,54,56,178,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
171,0,45,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
156,155,104,0,4,178,
162,0,19,
181,179,177,176,175,174,169,168,167,166,165,162,156,155,152,104,100,96,91,
  13,0,1,157,158,
173,0,50,
175,0,27,51,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,18,43,
  44,57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,
  206,207,208,
202,201,193,192,189,188,186,184,182,180,162,156,155,154,153,152,149,147,146,
  142,141,140,139,138,137,136,135,134,133,132,124,119,118,107,104,0,126,
  128,129,130,131,143,144,145,
154,119,118,0,108,114,
154,119,0,
155,154,119,118,104,0,148,
154,119,0,
154,119,0,
155,154,119,118,104,0,148,
149,0,
154,119,118,0,108,114,
189,188,154,119,118,0,105,112,
189,188,154,119,118,0,105,112,
202,201,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,
  44,57,64,65,66,69,77,78,79,81,83,84,87,88,89,103,106,109,110,111,115,
  178,205,206,207,208,
201,152,100,96,91,0,1,157,158,
201,0,82,
201,152,100,96,91,0,1,157,158,
201,0,82,
201,0,82,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,65,
  66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,64,
  65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,
  208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,64,
  65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,
  208,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,64,
  65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,
  208,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,64,
  65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,207,
  208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,57,
  64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,57,
  64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,57,
  64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,57,
  64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,57,
  64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,57,
  64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
181,179,177,174,169,168,166,165,162,156,155,104,0,4,16,19,24,25,26,28,29,30,
  31,32,33,34,36,37,38,40,41,42,44,49,52,54,56,178,
181,179,177,176,175,174,169,168,167,166,165,162,156,155,152,104,100,96,91,
  13,0,1,157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
181,179,177,174,169,168,166,165,162,156,155,152,104,100,96,91,0,1,157,158,
181,179,177,174,169,168,166,165,162,156,155,152,104,100,96,91,0,1,157,158,
181,179,177,174,169,168,166,165,162,156,155,104,0,4,19,24,25,26,28,29,30,31,
  32,33,34,36,37,38,40,41,42,44,49,52,54,56,178,
162,0,19,
154,119,118,0,113,
154,119,118,0,113,
180,0,55,
201,0,82,
202,201,200,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,
  182,180,173,172,170,162,160,156,155,154,152,149,124,121,119,118,107,104,
  100,96,91,0,1,157,158,
202,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,79,81,83,84,89,
  103,106,109,110,111,115,178,205,206,207,208,
202,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,44,79,81,83,84,89,
  103,106,109,110,111,115,178,205,206,207,208,
198,197,196,195,194,193,192,0,70,71,72,73,74,75,76,
198,197,196,195,194,193,192,0,70,71,72,73,74,75,76,
198,197,196,195,194,193,192,0,70,71,72,73,74,75,76,
198,197,196,195,194,193,192,0,70,71,72,73,74,75,76,
191,190,189,188,0,65,66,67,68,
191,190,189,188,0,65,66,67,68,
191,190,189,188,0,65,66,67,68,
191,190,189,188,0,65,66,67,68,
191,190,189,188,0,65,66,67,68,
191,190,189,188,0,65,66,67,68,
172,170,0,46,48,
154,119,118,0,114,
154,119,118,0,114,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,
170,0,47,
202,199,189,188,156,155,154,152,149,124,121,119,118,107,104,100,96,91,0,1,
  157,158,
202,199,189,188,156,155,154,149,124,121,119,118,107,104,0,2,3,4,5,6,43,44,
  57,64,65,66,69,77,78,79,81,83,84,89,103,106,109,110,111,115,178,205,206,
  207,208,

};


static unsigned const char ag_astt[3681] = {
  1,1,1,1,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,2,
  2,2,2,7,1,1,1,1,0,1,1,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,
  3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,5,5,5,5,5,5,5,5,
  5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,
  1,3,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,2,1,2,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,
  1,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,8,8,8,8,8,8,8,8,8,
  8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,
  3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,
  1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,
  5,7,1,1,3,1,7,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,7,3,1,1,1,7,3,1,1,1,1,1,4,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,3,1,7,2,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,7,2,2,1,2,1,1,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,3,1,3,3,3,1,3,2,2,1,1,3,1,1,1,2,2,2,1,1,1,1,1,1,
  1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,
  2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,1,1,5,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,
  2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,
  7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,1,2,7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,7,1,1,1,
  1,1,1,1,1,1,1,2,2,2,2,7,2,3,3,3,3,1,3,2,2,1,1,3,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,3,3,3,3,1,3,2,2,1,1,3,1,1,1,2,2,1,1,1,1,1,3,
  1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,
  1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,1,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,
  2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,
  2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,7,2,1,5,1,2,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,
  2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,
  7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,4,1,1,1,2,2,2,1,1,2,2,2,1,
  2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,
  1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,
  7,2,2,1,1,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,2,1,2,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,
  1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,
  1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,2,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,7,2,2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,1,1,2,2,2,1,2,
  7,2,2,1,1,2,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,
  1,1,2,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,
  1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,
  4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,4,7,1,1,10,10,10,4,2,10,10,
  10,4,2,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,7,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,
  1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
275,275,1,2,277,275,
8,7,6,5,4,16,15,14,13,0,12,11,10,9,
94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,96,
97,6,
89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,89,91,
92,8,
276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,9,3,3,283,
276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,10,3,3,282,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,11,3,3,280,
276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,276,12,3,3,278,
19,20,21,31,18,22,23,26,28,98,98,98,13,77,4,4,4,4,38,4,17,18,36,35,4,34,33,
  32,17,18,30,30,29,37,27,25,24,17,
39,39,39,39,39,39,39,39,39,39,39,39,39,14,39,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,15,83,84,53,54,86,66,30,73,65,
  64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
67,67,67,67,67,67,67,67,67,67,67,67,67,16,67,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,99,99,99,3,
  99,99,99,3,1,2,17,3,3,297,
276,276,3,276,3,1,2,18,3,3,288,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,19,3,3,300,
276,276,3,276,3,1,2,20,3,3,298,
276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,21,3,3,296,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,276,22,
  3,3,287,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,276,23,
  3,3,285,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,24,83,84,53,54,86,41,73,65,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
98,98,98,25,39,17,
276,276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,26,3,3,
  284,
68,68,68,68,68,68,68,68,68,68,68,68,68,27,68,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,276,28,
  3,3,281,
69,29,70,
98,98,98,30,77,71,17,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,31,3,3,293,
72,28,32,25,73,
72,28,33,24,74,
55,57,58,59,43,98,98,118,43,40,126,124,118,42,98,34,83,84,53,54,86,75,75,73,
  65,64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
28,35,20,
76,36,19,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,37,83,84,53,54,86,77,73,65,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
19,20,21,31,18,22,23,26,28,98,98,98,38,77,13,78,13,78,38,78,17,18,36,35,78,
  34,33,32,17,18,30,30,29,37,27,25,24,17,
19,20,21,31,18,22,79,23,26,28,98,98,98,39,77,10,80,10,10,10,38,10,17,18,36,
  35,10,34,33,32,17,27,18,30,30,29,37,27,25,24,17,
133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
  133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,40,81,
123,123,123,107,123,
82,82,82,42,105,82,
88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,84,83,150,149,148,147,146,
  145,144,143,142,141,140,88,88,88,88,88,43,88,88,88,87,86,85,88,88,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,
  276,276,276,276,276,276,3,276,3,1,2,44,3,3,327,
89,90,45,
91,100,
129,128,128,128,129,117,127,
116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,
  116,116,116,116,116,116,125,116,125,122,108,108,116,116,116,48,
115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,
  115,115,115,115,115,115,119,115,119,119,106,106,115,115,115,49,119,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,
  276,276,276,276,276,276,3,3,1,2,50,3,3,326,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,
  276,276,276,276,276,276,3,3,1,2,51,3,3,325,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,
  276,276,276,276,276,276,3,3,1,2,52,3,3,324,
55,77,92,
40,85,131,44,
276,276,276,276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,
  55,3,3,321,
93,95,55,57,58,59,98,98,118,43,40,126,124,118,42,98,56,83,84,53,54,86,97,73,
  65,64,63,62,64,64,61,60,56,60,60,96,94,54,46,45,49,41,48,47,17,52,51,50,
  44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,57,3,3,318,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,58,3,3,308,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,59,3,3,307,
98,69,99,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,61,83,84,53,54,86,73,63,62,68,
  68,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,62,83,84,53,54,86,73,63,62,67,
  67,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,63,83,84,53,54,86,73,63,62,66,
  66,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
100,102,104,106,108,110,112,52,113,111,109,107,105,103,101,
114,116,58,59,120,122,124,126,128,130,45,131,129,127,125,123,121,119,118,
  117,115,
76,66,132,
19,20,21,31,18,22,23,26,28,98,98,98,1,67,77,10,10,10,10,38,10,17,18,36,35,
  10,34,33,32,17,18,30,30,29,37,27,25,24,17,
19,20,21,133,31,18,22,23,26,28,98,98,98,68,77,10,10,10,10,38,10,17,18,36,35,
  10,34,33,32,17,18,30,30,29,37,27,38,25,24,17,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,69,3,3,290,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,70,83,84,53,54,86,37,73,65,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
69,71,134,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,72,3,3,299,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,73,83,84,53,54,86,42,73,65,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
98,98,98,74,40,17,
28,75,23,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,276,76,
  3,3,279,
135,77,35,
136,12,137,36,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,79,3,3,286,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,80,83,84,53,54,86,138,31,73,
  65,64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,84,83,
  150,149,148,147,146,145,144,143,142,141,140,132,134,134,134,134,81,134,
  134,134,134,87,86,85,134,
82,82,82,113,114,82,
154,154,83,
160,159,159,159,160,84,157,
156,156,152,
155,155,151,
160,159,159,159,160,139,158,
161,88,
82,82,82,104,103,82,
139,140,140,140,140,90,102,140,
139,140,140,140,140,91,101,140,
55,79,57,58,59,98,98,118,43,40,126,124,118,42,98,92,83,84,53,54,86,81,73,65,
  64,63,62,64,64,61,60,56,60,60,142,141,54,46,45,49,41,48,47,17,52,51,50,
  44,
276,3,3,1,2,93,3,3,323,
143,94,144,
276,3,3,1,2,95,3,3,322,
143,96,145,
143,97,71,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,98,3,3,319,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,99,83,84,53,54,86,73,63,62,70,
  70,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,100,3,3,317,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,101,83,84,53,54,86,73,63,62,
  64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,102,3,3,316,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,103,83,84,53,54,86,73,63,62,
  63,63,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,104,3,3,315,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,105,83,84,53,54,86,73,63,62,
  62,62,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,106,3,3,314,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,107,83,84,53,54,86,73,63,62,
  61,61,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,108,3,3,313,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,109,83,84,53,54,86,73,63,62,
  60,60,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,110,3,3,312,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,111,83,84,53,54,86,73,63,62,
  59,59,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,112,3,3,311,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,113,83,84,53,54,86,73,63,62,
  58,58,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,114,3,3,310,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,115,83,84,53,54,86,73,146,63,
  62,146,146,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,116,3,3,309,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,117,83,84,53,54,86,73,147,63,
  62,147,147,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,118,83,84,53,54,86,73,148,63,
  62,148,148,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,119,83,84,53,54,86,73,149,63,
  62,149,149,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,120,3,3,306,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,121,83,84,53,54,86,73,150,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,122,3,3,305,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,123,83,84,53,54,86,73,151,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,124,3,3,304,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,125,83,84,53,54,86,73,152,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,126,3,3,303,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,127,83,84,53,54,86,73,153,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,128,3,3,302,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,129,83,84,53,54,86,73,154,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,130,3,3,301,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,131,83,84,53,54,86,73,155,64,
  63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
19,20,21,31,18,22,23,26,28,98,98,98,132,77,2,2,2,2,38,2,17,18,36,35,2,34,33,
  32,17,18,30,30,29,37,27,25,24,17,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,276,133,
  3,3,295,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,134,83,84,53,54,86,156,73,65,
  64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,135,3,3,292,
276,276,276,276,276,276,276,276,276,276,276,3,276,3,1,2,136,3,3,294,
19,20,21,31,18,22,23,26,28,98,98,98,137,77,15,14,15,38,15,17,18,36,35,15,34,
  33,32,17,18,30,30,29,37,27,25,24,17,
28,138,3,
120,120,120,139,157,
120,120,120,140,158,
72,80,159,
143,142,78,
276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,276,
  276,276,276,276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,
  1,2,143,3,3,320,
55,98,98,118,43,40,126,124,118,42,98,144,83,84,53,54,86,73,76,56,76,76,54,
  46,45,49,41,48,47,17,52,51,50,44,
55,98,98,118,43,40,126,124,118,42,98,145,83,84,53,54,86,73,75,56,75,75,54,
  46,45,49,41,48,47,17,52,51,50,44,
100,102,104,106,108,110,112,56,113,111,109,107,105,103,101,
100,102,104,106,108,110,112,55,113,111,109,107,105,103,101,
100,102,104,106,108,110,112,54,113,111,109,107,105,103,101,
100,102,104,106,108,110,112,53,113,111,109,107,105,103,101,
114,116,58,59,51,119,118,117,115,
114,116,58,59,50,119,118,117,115,
114,116,58,59,49,119,118,117,115,
114,116,58,59,48,119,118,117,115,
114,116,58,59,47,119,118,117,115,
114,116,58,59,46,119,118,117,115,
160,33,156,162,161,
121,121,121,112,121,
121,121,121,111,121,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,159,83,84,53,54,86,82,73,65,
  64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,160,3,3,291,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,161,83,84,53,54,86,34,73,65,
  64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,
163,162,164,
276,276,276,276,276,276,276,3,276,276,276,276,276,276,276,3,1,2,163,3,3,289,
55,57,58,59,98,98,118,43,40,126,124,118,42,98,164,83,84,53,54,86,32,73,65,
  64,63,62,64,64,61,60,56,60,60,54,46,45,49,41,48,47,17,52,51,50,44,

};


static const unsigned short ag_sbt[] = {
     0,  12,  39,  67,  73,  87, 111, 113, 138, 140, 160, 181, 203, 224,
   262, 277, 323, 338, 368, 379, 401, 412, 433, 457, 481, 526, 532, 555,
   570, 594, 597, 604, 626, 631, 636, 683, 686, 689, 734, 772, 813, 850,
   855, 861, 904, 938, 941, 943, 950, 985,1021,1054,1087,1120,1123,1127,
  1152,1201,1223,1245,1267,1270,1312,1354,1396,1411,1432,1435,1474,1514,
  1536,1581,1584,1606,1651,1657,1660,1684,1687,1691,1713,1759,1803,1809,
  1812,1819,1822,1825,1832,1834,1840,1848,1856,1904,1913,1916,1925,1928,
  1931,1953,1995,2017,2059,2081,2123,2145,2187,2209,2251,2273,2315,2337,
  2379,2401,2443,2465,2508,2530,2573,2616,2659,2681,2725,2747,2791,2813,
  2857,2879,2923,2945,2989,3011,3055,3093,3117,3162,3182,3202,3239,3242,
  3247,3252,3255,3258,3302,3336,3370,3385,3400,3415,3430,3439,3448,3457,
  3466,3475,3484,3489,3494,3499,3544,3566,3611,3614,3636,3681
};


static const unsigned short ag_sbe[] = {
     8,  36,  64,  71,  77, 110, 112, 137, 139, 156, 177, 199, 220, 236,
   275, 291, 336, 364, 375, 397, 408, 429, 453, 477, 495, 529, 551, 568,
   590, 595, 600, 622, 628, 633, 651, 684, 687, 703, 746, 785, 848, 853,
   858, 895, 934, 940, 942, 948, 984,1019,1050,1083,1116,1121,1124,1148,
  1168,1219,1241,1263,1268,1284,1326,1368,1403,1421,1433,1448,1487,1532,
  1550,1582,1602,1620,1654,1658,1680,1685,1688,1709,1727,1794,1806,1811,
  1817,1821,1824,1830,1833,1837,1845,1853,1871,1909,1914,1921,1926,1929,
  1949,1967,2013,2031,2077,2095,2141,2159,2205,2223,2269,2287,2333,2351,
  2397,2415,2461,2479,2526,2544,2587,2630,2677,2695,2743,2761,2809,2827,
  2875,2893,2941,2959,3007,3025,3067,3113,3131,3178,3198,3214,3240,3245,
  3250,3253,3256,3298,3313,3347,3377,3392,3407,3422,3434,3443,3452,3461,
  3470,3479,3486,3492,3497,3513,3562,3580,3612,3632,3650,3681
};


static const unsigned char ag_fl[] = {
  2,3,4,5,2,1,1,1,1,0,2,1,1,2,4,4,1,1,1,2,2,1,1,3,2,2,1,1,1,1,1,1,7,0,2,
  3,1,3,3,2,3,2,3,0,1,1,3,3,3,3,3,3,1,3,3,3,3,1,3,3,3,3,3,3,3,1,2,2,2,1,
  3,3,1,1,1,4,4,1,4,0,1,1,3,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,
  2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0, 11, 11, 11, 11,159,161,163,164, 12, 12, 16, 16, 24, 24, 25, 25, 28,
   28, 28, 28, 28, 28, 28, 28, 28, 29, 17, 30, 41, 14, 18, 31, 46, 46, 26,
   27, 32, 33, 36, 36, 37, 37, 35, 35, 43, 43, 43, 43, 43, 43, 43, 57, 57,
   57, 57, 57, 64, 64, 64, 64, 64, 64, 64, 64, 69, 69, 69, 69, 77, 77, 79,
   79, 79, 79, 79, 79, 44, 84, 87, 87, 88, 88, 83, 83, 83, 83,  1, 93, 93,
   94, 94,  1, 98, 98, 99, 99,  1,178,178,206,206,206,103,103,103,106,106,
  106,112,112,105,105,108,108,205,205,205,109,109,113,113,110,110,111,111,
  115,115,122,122, 89, 89,208,125,125,126,126,128,128,128,129,129,129,129,
  129,129,129,129,129,129,129,130,130,130,143,144,145,131,131,148,148,207,
  150,150,114, 90, 90, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92,
   92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 97, 97, 97, 97, 97, 97, 97,
   97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97,101,101,
  101,102,102,102,102,102,102,116,116,117,117,117,120,120,123,123,127,127,
  127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,
  127,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
  151,151,151,151,157,157,158,158,  7, 15,  8, 19,  9, 10, 34, 38, 39, 40,
   42, 47, 45, 48, 50, 49, 51, 53, 52,  4, 54, 55, 56, 58, 59, 60, 61, 62,
   63, 65, 66, 67, 68, 70, 71, 72, 73, 74, 75, 76, 78, 80, 82, 81, 85, 86,
    2,  3,  6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: ag_rp_5(PCB_POINTER); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER, VW(0, AG_WRAP_7 *)); 
            VWD(0, AG_WRAP_7 *); break;
    case 12: ag_rp_15(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 13: ag_rp_16(PCB_POINTER); break;
    case 14: ag_rp_17(PCB_POINTER, VW(0, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); break;
    case 15: ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); break;
    case 16: VNO AG_WRAP_7(ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(4, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); VWD(3, AG_WRAP_5 *); VWD(4, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 17: VNO AG_WRAP_5(ag_rp_20(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_5(ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 19: ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 20: ag_rp_23(PCB_POINTER); break;
    case 21: ag_rp_24(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); VWD(2, AG_WRAP_5 *); break;
    case 22: ag_rp_25(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 23: ag_rp_26(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 24: ag_rp_27(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 25: ag_rp_28(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 26: VNO AG_WRAP_5(ag_rp_29(PCB_POINTER)); break;
    case 27: VRO(AG_WRAP_5 *, ag_rp_30(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 28: VRO(AG_WRAP_5 *, ag_rp_31(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 29: VRO(AG_WRAP_5 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 30: VRO(AG_WRAP_5 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 31: VRO(AG_WRAP_5 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 32: VRO(AG_WRAP_5 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 33: VRO(AG_WRAP_5 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 34: VRO(AG_WRAP_5 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 35: VRO(AG_WRAP_5 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 36: VRO(AG_WRAP_5 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 37: VRO(AG_WRAP_5 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 38: VRO(AG_WRAP_5 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 39: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: VRO(AG_WRAP_5 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 43: VRO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 44: VNO AG_WRAP_5(ag_rp_47(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 45: VNO AG_WRAP_5(ag_rp_48(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 46: VNO AG_WRAP_5(ag_rp_49(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 47: VRO(AG_WRAP_5 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 48: VNO AG_WRAP_5(ag_rp_51(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 49: VRO(AG_WRAP_5 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 50: VNO AG_WRAP_5(ag_rp_53(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 51: VNO AG_WRAP_5(ag_rp_54(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 52: VRO(AG_WRAP_4 *, ag_rp_55(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 53: VRO(AG_WRAP_4 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 54: VNO AG_WRAP_6(ag_rp_57(PCB_POINTER)); break;
    case 55: VRO(AG_WRAP_5 *, ag_rp_58(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 56: VRO(AG_WRAP_6 *, ag_rp_59(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 57: VNO AG_WRAP_5(ag_rp_60(PCB_POINTER, V(0,(long *)))); break;
    case 58: VNO AG_WRAP_5(ag_rp_61(PCB_POINTER, V(0,(double *)))); break;
    case 59: VNO AG_WRAP_5(ag_rp_62(PCB_POINTER, V(0,(int *)))); break;
    case 60: VNO AG_WRAP_4(ag_rp_63(PCB_POINTER, V(0,(int *)))); break;
    case 61: VRO(AG_WRAP_4 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 62: V(0,(double *)) = ag_rp_65(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 63: V(0,(double *)) = ag_rp_66(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 64: V(0,(double *)) = ag_rp_67(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 65: V(0,(double *)) = ag_rp_68(PCB_POINTER, V(0,(double *))); break;
    case 66: V(0,(double *)) = ag_rp_69(PCB_POINTER, V(1,(double *))); break;
    case 67: V(0,(double *)) = ag_rp_70(PCB_POINTER, V(0,(long *))); break;
    case 68: V(0,(double *)) = ag_rp_71(PCB_POINTER, V(0,(long *))); break;
    case 69: V(0,(double *)) = ag_rp_72(PCB_POINTER, V(0,(long *))); break;
    case 70: V(0,(long *)) = ag_rp_73(PCB_POINTER, V(1,(long *))); break;
    case 71: V(0,(long *)) = ag_rp_74(PCB_POINTER, V(1,(long *))); break;
    case 72: V(0,(double *)) = ag_rp_75(PCB_POINTER, V(0,(int *))); break;
    case 73: V(0,(double *)) = ag_rp_76(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 74: V(0,(long *)) = ag_rp_77(PCB_POINTER, V(0,(int *))); break;
    case 75: V(0,(long *)) = ag_rp_78(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 76: V(0,(long *)) = ag_rp_79(PCB_POINTER, V(0,(int *))); break;
    case 77: V(0,(long *)) = ag_rp_80(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 78: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 79: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 80: V(0,(long *)) = ag_rp_83(PCB_POINTER); break;
    case 81: V(0,(long *)) = ag_rp_84(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 82: V(0,(long *)) = ag_rp_85(PCB_POINTER); break;
    case 83: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 84: V(0,(int *)) = ag_rp_87(PCB_POINTER, V(0,(int *))); break;
    case 85: V(0,(int *)) = ag_rp_88(PCB_POINTER, V(0,(int *))); break;
    case 86: VRO(AG_WRAP_5 *, ag_rp_89(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 87: VNO AG_WRAP_5(ag_rp_90(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 88: VNO AG_WRAP_4(ag_rp_91(PCB_POINTER)); break;
    case 89: VRO(AG_WRAP_4 *, ag_rp_92(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 90: V(0,(int *)) = ag_rp_93(PCB_POINTER); break;
    case 91: V(0,(int *)) = ag_rp_94(PCB_POINTER); break;
    case 92: V(0,(int *)) = ag_rp_95(PCB_POINTER); break;
    case 93: V(0,(int *)) = ag_rp_96(PCB_POINTER); break;
    case 94: V(0,(int *)) = ag_rp_97(PCB_POINTER); break;
    case 95: V(0,(int *)) = ag_rp_98(PCB_POINTER); break;
    case 96: V(0,(int *)) = ag_rp_99(PCB_POINTER); break;
    case 97: V(0,(int *)) = ag_rp_100(PCB_POINTER); break;
    case 98: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 99: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 100: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 101: V(0,(int *)) = ag_rp_104(PCB_POINTER, V(1,(int *))); break;
    case 102: V(0,(int *)) = ag_rp_105(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 103: V(0,(int *)) = ag_rp_106(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 104: V(0,(int *)) = ag_rp_107(PCB_POINTER, V(1,(int *))); break;
    case 105: V(0,(int *)) = ag_rp_108(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 106: V(0,(int *)) = ag_rp_109(PCB_POINTER, V(0,(int *))); break;
    case 107: V(0,(int *)) = ag_rp_110(PCB_POINTER, V(0,(int *))); break;
    case 108: V(0,(int *)) = ag_rp_111(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[209] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "REPEAT LOOP",
  "STATEMENT",
  "syntax selection",
  "statement list",
  "eof",
  "loop condition",
  "\"DO\"",
  "statement",
  "UNTIL",
  "until condition",
  "';'",
  "\"SCRIPT\"",
  "\"WHILELOOP\"",
  "\"REPEATLOOP\"",
  "\"STATEMENT\"",
  "open statement",
  "closed statement",
  "if condition",
  "ELSE",
  "simple statement",
  "REPEAT",
  "WHILE",
  "for initialization",
  "assignment statement",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "FOR",
  "\"FOR\"",
  "expression",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "\"BY\"",
  "\"IF\"",
  "\"THEN\"",
  "\"ELSE\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "simple expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'='",
  "\"<>\"",
  "term",
  "'+'",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "unary expression",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "factor",
  "\"NOT\"",
  "primary",
  "\"**\"",
  "'('",
  "')'",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "\"DO\"",
  "WHILE LOOP",
  "';'",
  "REPEAT LOOP",
  "STATEMENT",
  "\"RETURN\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "\"THEN\"",
  "\"IF\"",
  "\"ELSE\"",
  "\"END\"",
  "\"BEGIN\"",
  "name",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'='",
  "\"<>\"",
  "'+'",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "\"**\"",
  "')'",
  "'('",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


