/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.

 Modified to support arrays and subscripts.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In cll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146192
#include "dxi.h"
#endif

#ifndef DXI_H_1031146192
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 591, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    loopActive(0),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

void dxi_pcb_struct::forLoop
    (const FileLocation &condLoc, const FileLocation &incLoc)
{
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    if (loopPCB.mode == executeMode) {
      loopPCB.parseSyntax("forField", condLoc);    // Evaluate condition field
      Value result = dxi_value(&loopPCB);
      if (result.isDefined() && result.isFalse()) loopPCB.mode = breakMode;
    }
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // Mode could have been set to continueMode
    loopPCB.parseSyntax("forField", incLoc);
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

void dxi_pcb_struct::interruptLoop(Mode desiredMode) {
  if (mode != executeMode) return;
  if (loopActive == 0) reportError("No loop active");
  mode = desiredMode;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

Value dxi_pcb_struct::powm(Value &lvalue, const Value &exp) const {
  if (mode != executeMode) return Value();
  Value base = lvalue;
  lvalue = pow(base.deref(), exp);
  return lvalue.deref();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  return pcb.returnValue;
}

#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 185, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return Value();
}

static inline Value ag_rp_5(PCB_DECL, Value &x) {
/* Line 189, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x;
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 193, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 196, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 199, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 202, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 205, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 228, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 230, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 234, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_14(PCB_DECL) {
/* Line 248, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.simpleLoop("doLoop");
}

static inline void ag_rp_15(PCB_DECL) {
/* Line 249, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_16(PCB_DECL, FileLocation c, FileLocation i) {
/* Line 250, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.forLoop(c, i);
}

static inline void ag_rp_17(PCB_DECL) {
/* Line 254, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.interruptLoop(breakMode);
}

static inline void ag_rp_18(PCB_DECL) {
/* Line 255, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.interruptLoop(continueMode);
}

static inline void ag_rp_19(PCB_DECL, Value &x) {
/* Line 256, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_20(PCB_DECL, AgString &n, Value &x) {
/* Line 260, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  VALUE(PCB.dataset.value(n) = Value(x, 0));
}

static inline void ag_rp_21(PCB_DECL) {
/* Line 272, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static void ag_rp_22(PCB_DECL, Value &x) {
/* Line 277, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static inline void ag_rp_23(PCB_DECL) {
/* Line 280, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.mode = scanMode;
}

static inline FileLocation ag_rp_24(PCB_DECL) {
/* Line 283, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return CONTEXT;
}

static void ag_rp_25(PCB_DECL, Value &x) {
/* Line 288, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_26(PCB_DECL) {
/* Line 292, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_27(PCB_DECL, AgString &n) {
/* Line 299, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_28(PCB_DECL, AgString &n) {
/* Line 300, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_29(PCB_DECL, Value &x) {
/* Line 304, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_30(PCB_DECL, Value &x) {
/* Line 305, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  PCB.print(x);
}

static inline Value ag_rp_31(PCB_DECL) {
/* Line 308, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return Value();
}

static inline Value ag_rp_32(PCB_DECL, Value &x) {
/* Line 314, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x;
}

static inline Value ag_rp_33(PCB_DECL, Value &lv, Value &x) {
/* Line 318, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv = x);
}

static inline Value ag_rp_34(PCB_DECL, Value &lv, Value &x) {
/* Line 319, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv += x);
}

static inline Value ag_rp_35(PCB_DECL, Value &lv, Value &x) {
/* Line 320, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv -= x);
}

static inline Value ag_rp_36(PCB_DECL, Value &lv, Value &x) {
/* Line 321, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv *= x);
}

static inline Value ag_rp_37(PCB_DECL, Value &lv, Value &x) {
/* Line 322, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv /= x);
}

static inline Value ag_rp_38(PCB_DECL, Value &lv, Value &x) {
/* Line 323, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv %= x);
}

static inline Value ag_rp_39(PCB_DECL, Value &lv, Value &x) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv |= x);
}

static inline Value ag_rp_40(PCB_DECL, Value &lv, Value &x) {
/* Line 325, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv ^= x);
}

static inline Value ag_rp_41(PCB_DECL, Value &lv, Value &x) {
/* Line 326, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv &= x);
}

static inline Value ag_rp_42(PCB_DECL, Value &lv, Value &x) {
/* Line 327, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv <<= x);
}

static inline Value ag_rp_43(PCB_DECL, Value &lv, Value &x) {
/* Line 328, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv >>= x);
}

static inline Value ag_rp_44(PCB_DECL, Value &lv, Value &x) {
/* Line 329, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return PCB.powm(lv,x);
}

static void ag_rp_45(PCB_DECL, Value &c) {
/* Line 335, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
if (PCB.mode == executeMode && c.isFalse())  PCB.mode = elseMode;
}

static void ag_rp_46(PCB_DECL, Value &c, Value &x) {
/* Line 337, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
if (!PCB.changeMode(elseMode, executeMode))
         PCB.changeMode(executeMode, blockMode);
}

static inline Value ag_rp_47(PCB_DECL, Value &c, Value &x, Value &y) {
/* Line 340, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return PCB.mode = CONTEXT.mode, VALUE(c.isTrue() ? x : y);
}

static void ag_rp_48(PCB_DECL, Value &x) {
/* Line 346, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
if (PCB.mode == executeMode && x.isTrue()) PCB.mode = blockMode;
}

static inline Value ag_rp_49(PCB_DECL, Value &x, Value &y) {
/* Line 347, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? x : y);
}

static void ag_rp_50(PCB_DECL, Value &x) {
/* Line 352, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = blockMode;
}

static inline Value ag_rp_51(PCB_DECL, Value &x, Value &y) {
/* Line 353, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? y : x);
}

static inline Value ag_rp_52(PCB_DECL, Value &x, Value &y) {
/* Line 358, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x |= y);
}

static inline Value ag_rp_53(PCB_DECL, Value &x, Value &y) {
/* Line 362, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x ^= y);
}

static inline Value ag_rp_54(PCB_DECL, Value &x, Value &y) {
/* Line 366, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x &= y);
}

static inline Value ag_rp_55(PCB_DECL, Value &x, Value &y) {
/* Line 370, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_56(PCB_DECL, Value &x, Value &y) {
/* Line 371, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_57(PCB_DECL, Value &x, Value &y) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_58(PCB_DECL, Value &x, Value &y) {
/* Line 376, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_59(PCB_DECL, Value &x, Value &y) {
/* Line 377, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_60(PCB_DECL, Value &x, Value &y) {
/* Line 378, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_61(PCB_DECL, Value &x, Value &y) {
/* Line 382, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x <<= y);
}

static inline Value ag_rp_62(PCB_DECL, Value &x, Value &y) {
/* Line 383, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x >>= y);
}

static inline Value ag_rp_63(PCB_DECL, Value &x, Value &y) {
/* Line 387, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x += y);
}

static inline Value ag_rp_64(PCB_DECL, Value &x, Value &y) {
/* Line 388, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x -= y);
}

static inline Value ag_rp_65(PCB_DECL, Value &x, Value &y) {
/* Line 392, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x *= y);
}

static inline Value ag_rp_66(PCB_DECL, Value &x, Value &y) {
/* Line 393, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x /= y);
}

static inline Value ag_rp_67(PCB_DECL, Value &x, Value &y) {
/* Line 394, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x %= y);
}

static inline Value ag_rp_68(PCB_DECL, Value &x) {
/* Line 398, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_69(PCB_DECL, Value &x) {
/* Line 399, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_70(PCB_DECL, Value &x) {
/* Line 400, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_71(PCB_DECL, Value &x) {
/* Line 401, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(~x);
}

static inline Value ag_rp_72(PCB_DECL, Value &x, Value &y) {
/* Line 409, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_73(PCB_DECL, Value &x) {
/* Line 417, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x;
}

static inline Value ag_rp_74(PCB_DECL, Value &lv) {
/* Line 419, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_75(PCB_DECL, Value &lv) {
/* Line 420, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(++lv);
}

static inline Value ag_rp_76(PCB_DECL, Value &lv) {
/* Line 421, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(--lv);
}

static inline Value ag_rp_77(PCB_DECL, Value &lv) {
/* Line 422, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv++);
}

static inline Value ag_rp_78(PCB_DECL, Value &lv) {
/* Line 423, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv--);
}

static inline Value ag_rp_79(PCB_DECL, Value &x) {
/* Line 425, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_80(PCB_DECL, Value &x) {
/* Line 426, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_81(PCB_DECL, AgString &n) {
/* Line 429, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_82(PCB_DECL, Value &lv, Value &x) {
/* Line 430, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(lv[x]);
}

static inline Value ag_rp_83(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 433, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(callFunction(n, args));
}

static inline AgStack<Value> ag_rp_84(PCB_DECL) {
/* Line 436, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_85(PCB_DECL, Value &x) {
/* Line 440, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_86(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 441, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_87(PCB_DECL, long x) {
/* Line 444, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_88(PCB_DECL, double x) {
/* Line 445, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_89(PCB_DECL, int x) {
/* Line 447, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_90(PCB_DECL, int c) {
/* Line 465, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_91(PCB_DECL, AgString &ns, int c) {
/* Line 466, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return ns.append(c);
}

static inline double ag_rp_92(PCB_DECL, double x, long e) {
/* Line 471, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_93(PCB_DECL, double x, long e) {
/* Line 472, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_94(PCB_DECL, double i, double f) {
/* Line 475, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return i+f;
}

static inline double ag_rp_95(PCB_DECL, double i) {
/* Line 476, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return i;
}

static inline double ag_rp_96(PCB_DECL, double f) {
/* Line 477, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return f;
}

static inline double ag_rp_97(PCB_DECL, long x) {
/* Line 480, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x;
}

static inline double ag_rp_98(PCB_DECL, long x) {
/* Line 481, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x;
}

static inline double ag_rp_99(PCB_DECL, long x) {
/* Line 482, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_100(PCB_DECL, long x) {
/* Line 485, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return (long)x;
}

static inline long ag_rp_101(PCB_DECL, long x) {
/* Line 486, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_102(PCB_DECL, int d) {
/* Line 489, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_103(PCB_DECL, int d, double f) {
/* Line 490, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_104(PCB_DECL, int d) {
/* Line 498, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return d-'0';
}

static inline long ag_rp_105(PCB_DECL, long x, int d) {
/* Line 499, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_106(PCB_DECL, int d) {
/* Line 502, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return d-'0';
}

static inline long ag_rp_107(PCB_DECL, long x, int d) {
/* Line 503, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_108(PCB_DECL, long x, int d) {
/* Line 506, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_109(PCB_DECL, long x, int d) {
/* Line 507, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_110(PCB_DECL) {
/* Line 510, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 0;
}

static inline long ag_rp_111(PCB_DECL, long n, int d) {
/* Line 511, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_112(PCB_DECL) {
/* Line 514, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 0;
}

static inline long ag_rp_113(PCB_DECL, long n, int d) {
/* Line 515, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_114(PCB_DECL, int x) {
/* Line 518, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return x - '0';
}

static inline int ag_rp_115(PCB_DECL, int d) {
/* Line 519, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_116(PCB_DECL, Value &s, Value &e) {
/* Line 525, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_117(PCB_DECL, AgString &b) {
/* Line 528, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_118(PCB_DECL) {
/* Line 531, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_119(PCB_DECL, AgString &s, int c) {
/* Line 532, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_120(PCB_DECL) {
/* Line 544, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\'';
}

static inline int ag_rp_121(PCB_DECL) {
/* Line 545, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '"';
}

static inline int ag_rp_122(PCB_DECL) {
/* Line 546, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\?';
}

static inline int ag_rp_123(PCB_DECL) {
/* Line 547, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\\';
}

static inline int ag_rp_124(PCB_DECL) {
/* Line 548, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\a';
}

static inline int ag_rp_125(PCB_DECL) {
/* Line 549, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\b';
}

static inline int ag_rp_126(PCB_DECL) {
/* Line 550, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\f';
}

static inline int ag_rp_127(PCB_DECL) {
/* Line 551, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\n';
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 552, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\r';
}

static inline int ag_rp_129(PCB_DECL) {
/* Line 553, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\t';
}

static inline int ag_rp_130(PCB_DECL) {
/* Line 554, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return '\v';
}

static inline int ag_rp_131(PCB_DECL, int d) {
/* Line 560, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return d-'0';
}

static inline int ag_rp_132(PCB_DECL, int n, int d) {
/* Line 563, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_133(PCB_DECL, int n, int d) {
/* Line 566, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_134(PCB_DECL, int d) {
/* Line 569, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return d;
}

static inline int ag_rp_135(PCB_DECL, int n, int d) {
/* Line 570, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_136(PCB_DECL, int d) {
/* Line 573, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return d-'0';
}

static inline int ag_rp_137(PCB_DECL, int d) {
/* Line 574, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_138(PCB_DECL, int c) {
/* Line 583, W:/agdev/agex/packages/xidek/cll/array/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0,
  5, 5, 5, 5, 5, 5, 5, 5, 4, 5, 4, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 4,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 5, 0, 0, 5, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 5,
  5, 0, 0, 0, 0, 5, 0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
  0, 6, 6, 0, 0, 5, 0, 5, 0, 0, 5, 0, 0, 0, 0, 5, 0, 5, 5, 0, 5, 0, 0,
  0, 0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  6,  7,  0,  0,  0,  0,  8,  9, 10,
    0, 11, 12, 13, 14,  0,  0, 15, 16, 17,  0,  0, 18,  0,  0,  0, 19, 20,
   21,  0, 14, 22, 23, 24,  0, 25, 26, 27, 28, 29,  0,  0, 30,  0, 31, 32,
   33, 34, 35, 36, 37, 38, 39, 40, 41, 42,  0, 43, 44, 45,  0, 46, 47,  0,
   48, 49,  0, 50,  0, 51,  0, 52,  0, 53, 54,  0, 55, 56, 57, 58,  0, 59,
   60,  0, 61, 62,  0, 63, 64, 65,  0, 66, 67, 68, 69,  0, 70, 71,  0, 72,
   73, 74, 75, 76,  0, 77, 78, 79, 80, 81, 82,  0, 83, 84, 85, 86,  0, 87,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 88, 89,  0, 90, 91, 92, 93,
   94, 95, 96, 97,  0,  0, 98, 99,100,101,  0,  0,  0,102,103,104,105,106,
  107,108,109,  0,  0,110,111,112,113,  0,114,115,116,117,  0,  0,  0,  0,
    0,118,119,120,121,122,123,124,125,126,127,128,  0,  0,  0,129,130,131,
  132,133,134,135,136
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0, 20,  0, 21,  0, 22,  0, 23,  0, 24,  0,201,  0,202,  0,203,
  0,207,  0,208,  0,209,  0,210,  0,212,  0,213,  0,216,  0,218,
  0,256,  0,257,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 99,116,255, 47,100,102,115,119,255, 42,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 88,120,255,
   61,255, 60, 61,255, 61,255, 61, 62,255, 76,117,255,111,117,255, 70,255,
  114,255,111,255, 99,116,255, 76,255,101,255,108,255,105,255,104,255, 61,
  124,255, 33, 37, 38, 42, 43, 45, 47, 48, 60, 61, 62, 94, 97, 98, 99,100,
  101,102,105,108,112,114,115,119,124,255, 99,116,255,100,102,115,119,255,
   42, 47,255, 88,120,255, 43, 45, 47, 48,255, 42, 47,255, 88,120,255,111,
  117,255, 43, 45, 47, 48, 97, 98, 99,100,102,105,112,114,119,255, 42, 47,
  255, 47,255, 88,120,255, 43, 45, 48,255, 88,120,255,111,117,255, 43, 45,
   48, 97, 98, 99,100,102,105,112,114,119,255, 34, 39, 63, 92, 97, 98,102,
  110,114,116,118,120,255, 92,255, 42, 47,255, 60, 61,255, 61, 62,255, 33,
   38, 42, 47, 60, 61, 62,124,255, 60, 61,255, 61, 62,255, 33, 38, 42, 60,
   61, 62,124,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,100,108,255, 88,
  120,255, 43, 45, 48,100,108,255, 60, 61,255, 61, 62,255, 33, 38, 60, 61,
   62,124,255, 33, 38, 60, 61, 62,124,255, 33, 38, 61,124,255, 38,124,255,
   38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255,
   61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43,
   45, 47, 60, 61, 62, 94,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,
  255, 45, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33,
   37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255,124,255, 42, 47,255, 88,
  120,255,111,117,255, 43, 45, 47, 48, 97, 98, 99,100,101,102,105,112,114,
  119,255,119,255, 60, 61,255, 61, 62,255, 33, 38, 42, 43, 45, 60, 61, 62,
  124,255, 42, 47,255, 88,120,255, 60, 61,255, 61, 62,255,111,117,255, 33,
   38, 42, 43, 45, 47, 48, 60, 61, 62, 97, 98, 99,100,101,102,105,112,114,
  119,124,255, 88,120,255,111,117,255, 43, 45, 48, 97, 98, 99,100,101,102,
  105,112,114,119,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,7,2,7,4,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,0,
  4,0,4,1,0,4,0,4,0,1,4,7,7,4,6,7,4,7,4,6,4,2,4,7,7,4,7,4,6,4,2,4,2,4,2,
  4,0,0,4,3,3,2,2,2,2,2,2,2,3,2,3,7,7,7,2,7,2,7,7,7,7,2,2,2,4,7,7,4,7,7,
  2,7,4,0,0,4,0,0,4,3,3,2,2,4,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,7,2,7,7,7,7,
  7,4,0,0,4,2,4,0,0,4,3,3,2,4,0,0,4,5,7,4,3,3,2,7,7,7,2,7,7,7,7,7,4,0,0,
  0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,0,4,0,
  0,4,3,3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,3,3,2,7,7,4,0,0,4,
  0,0,4,3,3,2,3,2,3,4,3,3,3,3,3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,0,0,
  4,0,0,4,0,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,0,
  0,4,0,4,1,0,4,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,
  2,3,2,4,3,4,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,7,2,7,7,7,7,7,7,4,7,4,0,0,4,
  0,0,4,3,3,3,3,3,2,3,2,3,4,0,0,4,0,0,4,0,0,4,0,0,4,5,7,4,3,3,3,3,3,2,2,
  2,3,2,7,7,7,2,7,7,7,7,7,7,3,4,0,0,4,5,7,4,3,3,2,7,7,7,2,7,7,7,7,7,7,4
};

static const unsigned short ag_key_parm[] = {
    0,124,129,  0,  0,  6,  0,  0,  4,  8,  0,  2,  0,128,  0,234,227,  0,
  230,  0,253,222,  0,254,220,  0,255,221,  0,124,129,223,  0,155,154,  0,
  228,  0,244,241,  0,229,  0,243,245,  0,  4, 34,  0, 18, 28,  0,  8,  0,
   22,  0,  0,  0,  0,  6,  0,  2,  0, 20,  0,  0,  0,  0,  0,  0,  0,225,
  233,  0,239,224,  0,  0,  0,  0,  0,  0,  0,238,  0,226, 16, 10, 12,  0,
   26,  0, 24, 32, 30, 14,  0,  0,  0,  0,  0,  6,  0,  4,  8,  0,  2,  0,
  124,129,  0,155,154,  0,254,255,  0,  0,  0,124,129,  0,155,154,  0, 18,
   28,  0,254,255,  0,  0, 16, 10, 12,  0, 22, 24, 30, 14, 20,  0,124,129,
    0,  0,  0,155,154,  0,254,255,  0,  0,155,154,  0, 18, 28,  0,254,255,
    0, 16, 10, 12,  0, 22, 24, 30, 14, 20,  0,168,167,169,170,171,172,173,
  174,175,176,177,182,  0,  0,  0,124,129,  0,244,241,  0,243,245,  0,239,
  234,253,  0,  0,238,  0,233,  0,244,241,  0,243,245,  0,239,234,253,  0,
  238,  0,233,  0,124,129,  0,155,154,  0,254,255,  0,  0, 34, 32,  0,155,
  154,  0,254,255,  0, 34, 32,  0,244,241,  0,243,245,  0,239,234,  0,238,
    0,233,  0,239,234,241,238,243,233,  0,239,234,238,233,  0,234,233,  0,
  234,227,  0,230,  0,253,222,  0,254,220,  0,255,221,  0,124,129,223,  0,
  228,  0,244,241,  0,229,  0,243,245,  0,225,233,  0,239,224,  0,  0,  0,
    0,  0,  0,238,  0,226,  0,  0,234,227,  0,230,  0,253,222,  0,254,220,
    0,255,221,  0,228,  0,244,241,  0,229,  0,243,245,  0,225,233,  0,239,
  224,  0,  0,  0,  0,223,  0,238,  0,226,  0,  0,233,  0,124,129,  0,155,
  154,  0, 18, 28,  0,254,255,  0,  0, 16, 10, 12,  0, 26, 22, 24, 30, 14,
   20,  0, 20,  0,244,241,  0,243,245,  0,239,234,253,254,255,  0,238,  0,
  233,  0,124,129,  0,155,154,  0,244,241,  0,243,245,  0, 18, 28,  0,239,
  234,253,254,255,  0,  0,  0,238,  0, 16, 10, 12,  0, 26, 22, 24, 30, 14,
   20,233,  0,155,154,  0, 18, 28,  0,254,255,  0, 16, 10, 12,  0, 26, 22,
   24, 30, 14, 20,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 14, 19,  0,  1,  0,  6,  4, 27,  0, 36,  0,  0,  0,  0,
    0,  0, 18,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0, 36,  0,  0,  0,  0,  0, 41,  0, 64, 68,  0, 46, 72,  0, 79,  0,
   52,  0, 54,  0,101,106,  0,114,  0, 61,  0, 63,  0, 65,  0, 67,  0,  0,
    0,  0, 38, 40, 15, 20, 23, 26, 29, 33, 38, 42, 43, 44, 46, 51, 56, 49,
   75, 56, 84, 86, 90, 95, 58, 69, 71,  0,132,137,  0,118,124,100,145,  0,
    0,  0,  0,  0,  0,  0,154,156,108,111,  0,  0,  0,  0,  0,  0,  0,  0,
  180,  0,158,160,119,122,162,167,172,125,183,186,188,193,199,  0,  0,  0,
    0,142,  0,  0,  0,  0,204,206,147,  0,  0,  0,  0,  0,230,  0,208,210,
  154,212,217,222,157,233,236,238,243,249,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,173,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,254,
  256,258,188,191,260,194,262,  0,  0,  0,  0,  0,  0,  0,264,266,268,206,
  270,209,272,  0,  0,  0,  0,  0,  0,  0,274,276,220,223,278,284,  0,  0,
    0,  0,288,290,233,292,298,  0,  0,  0,  0,  0,  0,  0,302,304,242,306,
  245,308,  0,310,312,314,316,318,320,  0,322,324,326,328,  0,330,332,  0,
    0,  0,  0,  0,  0,273,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,288,  0,  0,  0,  0,  0,293,  0,  0,  0,  0,334,336,270,275,278,
  281,284,290,338,295,340,298,  0,  0,  0,  0,  0,  0,317,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,328,  0,  0,  0,  0,  0,333,  0,  0,  0,  0,342,
  344,314,319,322,325,346,330,348,335,350,338,  0,352,  0,  0,  0,  0,  0,
    0,  0,  0,376,  0,354,356,356,359,358,363,368,362,379,383,386,388,393,
  399,  0,404,  0,  0,  0,  0,  0,  0,  0,409,411,413,415,417,382,419,385,
  421,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,453,  0,423,
  425,427,429,431,398,401,404,433,407,435,440,445,410,456,460,463,465,470,
  476,481,  0,  0,  0,  0,  0,505,  0,483,485,435,487,492,497,438,508,512,
  515,517,522,528,  0
};

static const unsigned short ag_key_index[] = {
    7,  0, 13, 74,103,  0,  0, 13, 13,114,128,128,145,128,150,160,160,  0,
  160,186,  0,  0,186,197,  0,197,197,197,197,197,197,197,197,197,212,226,
    0,  0,236,114,114,114,114,212,150,150,150,150,248,248,248,255,262,267,
  267,267,301,267,341,354,341,  0,  0,145,114,145,128,365,365,145,150,  0,
  114,145,145,160,365,  0,145,  0,  0,  0,150,  0,  0,  0,150,  0,160,380,
  150,160,160,186,197,  0,  0,186,186,186,  0,197,  0,  0,212,212,145,  0,
  145,  0,  0,114,150,388,114,150,114,150,114,150,150,150,114,150,114,150,
  114,150,114,150,114,150,114,150,114,150,114,150,114,150,114,150,114,150,
  114,150,150,114,150,114,150,114,150,114,150,114,150,114,150,114,150,114,
  150,114,150,114,150,114,150,114,150,114,150,114,150,114,150,114,150,413,
  160,150,  0,150,  0,  0,  0,  0,150,441,  0,  0,  0,  0,150,150,248,248,
  248,248,248,248,248,248,255,255,262,267,267,150,  0,  0,150,150,  0,365,
    0,150,150,  0,128,160,197,197,267,150,267,  0,301,  0,  0,114,150,  0,
  150
};

static const unsigned char ag_key_ends[] = {
111,76,111,111,112,0, 111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
47,0, 61,0, 61,0, 61,0, 61,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 111,111,112,0, 98,108,101,0, 
109,112,0, 108,115,101,0, 105,101,108,100,0, 102,0, 111,110,103,0, 
114,105,110,116,0, 101,116,117,114,110,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 111,111,112,0, 111,76,111,111,112,0, 
111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
43,0, 45,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 43,0, 
45,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 61,0, 
38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 42,0, 61,0, 124,0, 
43,0, 45,0, 111,117,98,108,101,0, 111,110,103,0, 43,0, 45,0, 
111,117,98,108,101,0, 111,110,103,0, 61,0, 38,0, 61,0, 124,0, 
61,0, 38,0, 61,0, 61,0, 61,0, 124,0, 61,0, 38,0, 61,0, 124,0, 
38,0, 124,0, 61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 
61,0, 124,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 114,114,97,121,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 124,0, 43,0, 45,0, 
114,114,97,121,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned short ag_tcv[] = {
   14,188,188,188,188,188,188,188,188,187,133,187,187,187,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,187,251,159,188,
  188,250,237,184,211,199,248,246,217,247,140,249,152,189,189,189,189,189,
  189,189,151,151,231,200,240,219,242,232,188,190,190,190,190,137,190,191,
  191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,
  191,205,181,204,236,191,188,190,190,190,190,137,190,191,191,191,191,191,
  191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,215,235,214,
  252,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned short ag_tstt[] = {
187,133,129,124,24,23,22,21,20,0,1,192,193,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,184,181,159,152,151,140,137,133,0,
  131,132,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,184,181,159,152,151,140,137,133,128,
  0,126,127,
187,133,129,124,0,1,
24,23,22,21,20,0,7,8,9,10,11,12,194,195,196,197,198,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,184,181,159,152,151,140,137,0,
133,0,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,184,181,159,152,151,140,137,133,0,
128,0,
255,254,252,251,247,246,211,200,199,191,190,189,187,184,159,155,154,152,151,
  140,137,133,129,124,0,1,192,193,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,0,1,192,193,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,0,1,192,193,
211,187,133,129,124,0,1,192,193,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,14,0,1,192,
  193,
255,254,252,251,247,246,211,200,199,191,190,189,184,159,155,154,152,151,140,
  137,0,2,3,4,5,6,18,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,
  104,105,109,110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,
  206,258,259,260,261,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,56,58,59,60,61,
  74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,56,58,59,60,61,
  74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,122,136,139,142,143,144,148,156,206,258,259,260,261,
211,0,15,50,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,14,0,13,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,184,182,181,177,176,175,174,173,172,
  171,170,169,168,167,159,152,151,140,137,0,160,
189,152,151,0,147,
189,152,151,0,141,147,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,182,181,177,176,175,174,173,172,171,
  170,169,168,167,159,152,151,140,137,0,163,164,165,166,178,179,180,185,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,187,159,133,129,124,0,1,192,193,
140,137,0,
137,0,
190,189,152,151,137,0,157,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,189,187,152,151,140,137,133,129,124,0,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,189,187,152,151,140,137,133,129,124,0,147,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,187,133,129,124,0,1,192,193,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,187,133,129,124,0,1,192,193,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,187,133,129,124,0,1,192,193,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,191,190,187,137,133,129,124,0,1,192,193,
253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,217,204,200,199,191,190,187,137,133,129,124,0,1,192,193,
159,0,5,261,
257,256,255,254,252,251,247,246,211,200,199,191,190,189,187,184,159,155,154,
  152,151,140,137,133,129,124,0,1,192,193,
191,190,137,0,4,61,206,
191,190,137,0,4,61,206,
257,256,255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,
  137,0,2,3,4,5,6,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,118,119,122,136,139,142,143,144,148,
  156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
253,0,113,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
250,249,248,0,106,107,108,
247,246,0,103,104,
245,244,0,100,101,
243,242,241,240,0,95,96,97,98,
239,238,0,92,93,
237,0,90,
236,0,88,
235,0,86,
255,254,253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,
  234,233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,217,211,
  205,204,200,199,191,190,189,187,152,151,137,133,129,124,0,1,192,193,
234,0,83,
211,0,50,
233,232,0,75,80,
255,254,230,229,228,227,226,225,224,223,222,221,220,219,205,0,44,62,63,64,
  65,66,67,68,69,70,71,72,73,115,116,
217,0,57,
200,199,0,34,35,
211,187,133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
191,190,187,137,133,129,124,0,1,192,193,
255,254,252,251,247,246,218,216,215,214,212,211,210,209,208,207,203,202,201,
  200,191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,0,1,192,
  193,
255,254,252,251,247,246,218,216,215,214,213,212,211,210,209,208,207,203,202,
  201,200,191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,14,
  0,1,192,193,
255,254,252,251,247,246,218,216,215,214,213,212,211,210,209,208,207,203,202,
  201,200,191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,14,
  0,1,192,193,
191,190,187,137,133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
191,190,137,0,4,206,
255,254,252,251,247,246,211,200,191,190,189,187,184,159,155,154,152,151,140,
  137,133,129,124,0,1,192,193,
200,187,133,129,124,0,1,192,193,
200,187,133,129,124,0,1,192,193,
255,254,252,251,247,246,218,216,215,214,212,211,210,209,208,207,203,202,201,
  200,191,190,189,184,159,155,154,152,151,140,137,0,13,
255,254,252,251,247,246,218,216,215,214,213,212,211,210,209,208,207,203,202,
  201,200,199,191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,
  14,0,1,192,193,
211,0,50,
211,187,133,129,124,0,1,192,193,
191,190,137,0,4,206,
217,200,0,34,57,
217,200,0,34,57,
255,254,252,251,247,246,211,200,191,190,189,184,159,155,154,152,151,140,137,
  0,2,3,4,5,6,18,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,
  258,259,260,261,
200,0,34,
200,0,34,
217,200,0,34,57,
255,254,252,251,247,246,211,200,191,190,189,184,159,155,154,152,151,140,137,
  0,2,3,4,5,6,33,36,50,51,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,
  258,259,260,261,
211,0,50,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,56,58,59,60,61,
  74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,122,136,139,142,143,144,148,156,206,258,259,260,261,
209,0,17,47,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,
  260,261,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,56,58,59,60,61,
  74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,14,0,2,3,4,5,6,16,25,26,27,
  29,30,31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,56,58,59,60,
  61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
252,251,250,249,248,247,246,242,240,237,236,235,232,231,219,217,215,214,211,
  205,204,200,199,191,190,189,188,187,184,182,181,177,176,175,174,173,172,
  171,170,169,168,167,159,152,151,140,137,0,161,163,164,165,166,178,179,
  180,
189,152,151,0,141,147,
189,152,0,
190,189,152,151,137,0,183,
189,152,0,
189,152,0,
190,189,152,151,137,0,183,
184,0,
189,152,151,0,141,147,
247,246,189,152,151,0,138,145,
247,246,189,152,151,0,138,145,
205,0,44,
205,0,44,
199,187,133,129,124,0,1,192,193,
199,0,35,
199,187,133,129,124,0,1,192,193,
199,0,35,
217,199,0,35,57,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,205,0,44,115,116,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,103,104,105,109,110,111,112,114,115,116,117,122,136,139,
  142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,102,103,104,105,109,110,111,112,114,115,116,117,122,136,
  139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,102,103,104,105,109,110,111,112,114,115,116,117,122,136,
  139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,99,102,103,104,105,109,110,111,112,114,115,116,117,122,
  136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,99,102,103,104,105,109,110,111,112,114,115,116,117,122,
  136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,122,
  136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,122,
  136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,122,
  136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,122,
  136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,91,94,99,102,103,104,105,109,110,111,112,114,115,116,117,
  122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,91,94,99,102,103,104,105,109,110,111,112,114,115,116,117,
  122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,
  84,
255,254,252,251,247,246,211,199,191,190,189,184,159,155,154,152,151,140,137,
  0,2,3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,120,121,122,136,139,142,143,144,148,156,206,
  258,259,260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,
  81,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,
  76,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,
  260,261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,
  236,235,234,233,232,231,218,217,216,215,214,213,212,211,210,209,208,207,
  204,203,202,201,200,199,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,14,0,1,192,193,
255,254,252,251,247,246,218,216,215,214,212,211,210,209,208,207,203,202,201,
  200,191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,16,25,26,27,
  29,30,31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,55,56,58,59,
  60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,
  115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,200,191,190,189,184,159,155,154,152,151,140,137,
  0,2,3,4,5,6,18,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,
  258,259,260,261,
205,0,44,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
191,190,137,0,4,206,
200,0,34,
217,0,57,
200,0,34,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,
  260,261,
213,0,28,53,
211,0,15,50,
217,199,0,35,57,
189,152,151,0,146,
189,152,151,0,146,
255,254,211,191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,50,61,
  112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,211,191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,50,61,
  112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
250,249,248,0,106,107,108,
250,249,248,0,106,107,108,
247,246,0,103,104,
247,246,0,103,104,
245,244,0,100,101,
245,244,0,100,101,
245,244,0,100,101,
245,244,0,100,101,
243,242,241,240,0,95,96,97,98,
243,242,241,240,0,95,96,97,98,
239,238,0,92,93,
237,0,90,
236,0,88,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,
  115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
217,0,57,
199,0,35,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,61,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,
  114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,
  260,261,
217,204,0,45,57,
255,254,252,251,247,246,218,216,215,214,213,212,211,210,209,208,207,203,202,
  201,200,191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,14,
  0,1,192,193,
200,0,34,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,36,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,
  260,261,
255,254,252,251,247,246,211,199,191,190,189,184,159,155,154,152,151,140,137,
  0,2,3,4,5,6,33,36,50,51,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,
  258,259,260,261,
217,199,0,35,57,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,187,184,159,155,154,152,151,140,137,133,129,124,0,1,192,193,
255,254,252,251,247,246,218,216,215,212,211,210,209,208,207,203,202,201,200,
  191,190,189,184,159,155,154,152,151,140,137,0,2,3,4,5,6,25,26,27,29,30,
  31,32,34,36,37,38,39,40,41,42,43,46,47,48,49,50,52,54,56,58,59,60,61,74,
  79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,117,
  122,136,139,142,143,144,148,156,206,258,259,260,261,
189,152,151,0,147,
189,152,151,0,147,
235,0,86,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,59,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,
  261,
234,0,83,
231,217,0,57,77,
255,254,253,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,
  234,233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,217,205,
  204,200,199,187,133,129,124,0,1,192,193,
217,204,0,45,57,
199,0,35,
255,254,252,251,247,246,211,191,190,189,187,184,159,155,154,152,151,140,137,
  133,129,124,0,1,192,193,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,
  78,
200,0,34,
255,254,252,251,247,246,211,191,190,189,184,159,155,154,152,151,140,137,0,2,
  3,4,5,6,50,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,
  112,114,115,116,117,122,136,139,142,143,144,148,156,206,258,259,260,261,

};


static unsigned const char ag_astt[6692] = {
  1,1,1,1,8,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,2,2,2,2,2,7,1,1,1,1,1,0,1,
  1,1,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,5,3,7,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,5,7,1,1,3,1,1,1,1,1,1,1,4,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,
  7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,
  2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,5,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,1,1,
  1,1,5,1,1,1,1,5,1,1,1,1,1,1,5,1,1,1,1,1,1,5,1,1,1,5,1,1,5,1,1,5,1,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,5,1,1,4,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,5,1,1,1,7,2,2,5,1,1,1,1,7,1,
  1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,
  1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,
  1,1,1,5,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,5,1,1,1,5,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,
  3,2,2,2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,
  2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,1,7,2,1,7,2,1,1,7,3,1,1,1,1,1,1,1,1,5,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,1,7,1,2,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,2,2,
  2,1,2,1,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,1,2,2,7,2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,
  2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,
  7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,4,1,1,4,1,5,1,1,1,1,7,1,1,
  3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,2,2,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,
  7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,
  7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,1,1,1,4,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,3,
  3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,2,2,1,1,1,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,7,2,1,5,1,1,7,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,1,7,2,1,1,1,7,2,1,2,2,2,7,1,2,2,2,7,1,1,1,
  1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,
  1,1,1,4,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,4,1,
  1,4,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,1,7,2,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,2,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,5,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,7,
  2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,4,2,10,10,
  10,4,2,1,4,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,4,1,1,1,7,
  1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,1,1,1,7,1,1,3,1,1,7,1,1,1,7,2,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,7,2,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,
  6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,
  7,8,
366,366,1,2,368,366,
10,9,8,7,6,4,18,17,16,15,14,0,13,12,11,10,9,
133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,
  133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,133,135,
136,6,
128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
  128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,130,
131,8,
367,367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,
  367,367,3,1,2,9,3,3,373,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,10,3,3,372,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,11,3,3,371,
367,3,3,1,2,12,3,3,370,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,367,13,3,3,369,
32,33,39,40,41,42,35,47,47,137,137,157,22,19,167,167,163,157,21,137,14,122,
  123,58,34,125,62,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
32,33,39,40,41,42,64,65,66,78,35,63,67,68,69,72,73,74,76,137,137,157,22,19,
  167,167,163,157,21,137,15,122,123,58,34,125,4,4,4,88,4,19,20,86,4,85,4,
  84,83,82,81,80,79,19,20,77,77,38,87,75,71,70,85,85,60,59,57,55,54,53,52,
  51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,
  56,31,30,29,23,
32,33,39,40,41,42,64,65,66,78,35,63,67,68,69,72,73,74,76,137,137,157,22,19,
  167,167,163,157,21,137,16,122,123,58,34,125,89,89,89,88,89,19,20,86,89,
  85,89,84,83,82,81,80,79,19,20,77,77,38,87,75,71,70,85,85,60,59,57,55,54,
  53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
35,17,91,90,
92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,92,
  92,92,92,92,92,92,18,92,
174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,19,93,
162,162,162,146,162,
94,94,94,21,144,94,
100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,
  100,100,100,100,100,100,100,100,100,96,95,191,190,189,188,187,186,185,
  184,183,182,181,100,100,100,100,100,22,100,100,100,99,98,97,100,100,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,367,3,1,2,23,3,3,436,
101,102,24,
103,139,
170,169,169,169,170,156,168,
155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,164,155,164,161,147,147,155,155,155,27,
154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,154,
  154,154,154,154,154,154,158,154,158,158,145,145,154,154,154,28,158,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,3,1,2,29,3,3,435,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,3,1,2,30,3,3,434,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,3,1,2,31,3,3,433,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,3,367,3,1,2,32,3,3,430,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,3,367,3,1,2,33,3,3,429,
19,124,172,23,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,
  367,367,367,367,3,1,2,35,3,3,386,
137,137,137,36,115,104,56,
137,137,137,37,115,105,56,
106,108,32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,38,
  122,123,58,34,125,110,38,110,110,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,109,107,34,25,24,28,20,27,26,167,56,31,30,29,
  23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,39,3,3,427,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,40,3,3,426,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,41,3,3,422,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,42,3,3,421,
111,103,112,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,44,122,123,58,
  34,125,38,113,47,46,102,102,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,45,122,123,58,
  34,125,38,113,47,46,101,101,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,46,122,123,58,
  34,125,38,113,47,46,100,100,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,47,122,123,58,
  34,125,38,113,47,46,99,99,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,
  56,31,30,29,23,
114,116,118,91,119,117,115,
41,42,88,121,120,
122,124,83,125,123,
126,128,130,132,80,133,131,129,127,
134,136,78,137,135,
138,76,139,
140,74,141,
142,71,143,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,138,138,138,3,138,138,138,3,1,2,56,3,3,381,
144,68,145,
35,115,146,
147,149,64,150,148,
32,33,153,155,157,159,161,163,165,167,169,171,173,175,151,107,152,176,174,
  172,170,168,166,164,162,160,158,156,154,110,111,
177,48,178,
76,179,62,5,5,
367,3,3,1,2,63,3,3,385,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,64,3,3,393,
367,367,3,367,3,1,2,65,3,3,391,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,66,3,3,390,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,367,67,3,3,
  384,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,367,68,3,3,
  383,
367,367,3,367,3,1,2,69,3,3,382,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,70,122,123,58,
  34,125,38,45,45,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
137,137,137,71,43,56,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,
  367,3,1,2,72,3,3,378,
367,3,3,1,2,73,3,3,377,
367,3,3,1,2,74,3,3,376,
180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,
  180,180,180,180,180,180,180,180,180,180,180,180,75,180,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,367,76,
  3,3,375,
35,77,181,
367,3,3,1,2,78,3,3,387,
137,137,137,79,182,56,
177,76,80,29,183,
177,76,81,28,184,
32,33,39,40,41,42,35,47,137,137,157,22,19,167,167,163,157,21,137,82,122,123,
  58,34,125,185,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
76,83,26,
76,84,25,
177,76,85,22,178,
32,33,39,40,41,42,35,37,137,137,157,22,19,167,167,163,157,21,137,86,122,123,
  58,34,125,187,186,38,39,186,186,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
35,87,188,
32,33,39,40,41,42,64,65,66,78,35,63,67,68,69,72,73,74,76,137,137,157,22,19,
  167,167,163,157,21,137,88,122,123,58,34,125,15,15,189,88,189,19,20,86,
  189,85,189,84,83,82,81,80,79,19,20,77,77,38,87,75,71,70,85,85,60,59,57,
  55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,167,56,31,30,29,23,
67,89,190,34,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,90,122,123,58,
  34,125,191,38,191,191,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,
  44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
32,33,39,40,41,42,64,65,66,78,35,63,67,68,69,72,73,74,76,137,137,157,22,19,
  167,167,163,157,21,137,91,122,123,58,34,125,2,2,2,88,2,19,20,86,2,85,2,
  84,83,82,81,80,79,19,20,77,77,38,87,75,71,70,85,85,60,59,57,55,54,53,52,
  51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,
  56,31,30,29,23,
32,33,39,40,41,42,64,65,66,78,35,63,67,68,69,72,73,74,76,137,137,157,22,19,
  167,167,163,157,21,137,1,92,122,123,58,34,125,12,12,12,88,12,19,20,86,
  12,85,12,84,83,82,81,80,79,19,20,77,77,38,87,75,71,70,85,85,60,59,57,55,
  54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,167,56,31,30,29,23,
175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,
  175,175,175,175,175,175,175,175,175,175,96,95,191,190,189,188,187,186,
  185,184,183,182,181,173,175,175,175,175,93,175,175,175,175,99,98,97,175,
94,94,94,152,153,94,
195,195,95,
201,200,200,200,201,96,198,
197,197,193,
196,196,192,
201,200,200,200,201,180,199,
202,100,
94,94,94,143,142,94,
192,193,193,193,193,102,141,193,
192,193,193,193,193,103,140,193,
151,109,152,
151,108,152,
367,3,3,1,2,106,3,3,432,
179,107,194,
367,3,3,1,2,108,3,3,431,
179,109,195,
177,179,110,105,178,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,111,3,3,428,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,112,122,123,
  58,34,125,38,113,47,46,104,104,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,167,56,31,30,29,23,
32,33,151,107,152,110,111,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,114,3,3,425,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,115,122,123,
  58,34,125,38,113,47,46,97,97,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,116,3,3,424,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,117,122,123,
  58,34,125,38,113,47,46,96,96,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,118,3,3,423,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,119,122,123,
  58,34,125,38,113,47,46,95,95,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,120,122,123,
  58,34,125,38,113,196,47,46,196,196,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,167,56,31,30,29,23,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,121,122,123,
  58,34,125,38,113,197,47,46,197,197,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,122,3,3,420,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,123,122,123,
  58,34,125,38,113,198,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,124,3,3,419,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,125,122,123,
  58,34,125,38,113,199,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,126,3,3,418,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,127,122,123,
  58,34,125,38,113,200,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,128,3,3,417,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,129,122,123,
  58,34,125,38,113,201,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,130,3,3,416,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,131,122,123,
  58,34,125,38,113,202,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,132,3,3,415,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,133,122,123,
  58,34,125,38,113,203,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,134,3,3,414,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,135,122,123,
  58,34,125,38,113,204,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,136,3,3,413,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,137,122,123,
  58,34,125,38,113,205,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,138,3,3,412,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,139,122,123,
  58,34,125,38,113,206,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,
  24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,140,3,3,411,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,141,122,123,
  58,34,125,38,113,207,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,142,3,3,410,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,143,122,123,
  58,34,125,38,113,208,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,
  34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,144,3,3,409,
72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,145,209,
32,33,39,40,41,42,35,118,137,137,157,22,19,167,167,163,157,21,137,146,122,
  123,58,34,125,38,120,120,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,211,210,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,147,3,3,408,
69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,69,148,212,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,149,3,3,407,
65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,150,213,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,151,3,3,380,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,152,122,123,
  58,34,125,214,38,214,214,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,153,3,3,405,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,154,122,123,
  58,34,125,38,63,63,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,155,3,3,404,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,156,122,123,
  58,34,125,38,62,62,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,157,3,3,403,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,158,122,123,
  58,34,125,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,159,3,3,402,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,160,122,123,
  58,34,125,38,60,60,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,161,3,3,401,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,162,122,123,
  58,34,125,38,59,59,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,163,3,3,400,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,164,122,123,
  58,34,125,38,58,58,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,165,3,3,399,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,166,122,123,
  58,34,125,38,57,57,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,167,3,3,398,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,168,122,123,
  58,34,125,38,56,56,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,169,3,3,397,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,170,122,123,
  58,34,125,38,55,55,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,171,3,3,396,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,172,122,123,
  58,34,125,38,54,54,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,173,3,3,395,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,174,122,123,
  58,34,125,38,53,53,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,175,3,3,394,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,176,122,123,
  58,34,125,38,52,52,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,177,3,3,392,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,178,122,123,
  58,34,125,38,50,50,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,367,179,3,3,374,
32,33,39,40,41,42,64,65,66,215,78,35,63,67,68,69,72,73,74,76,137,137,157,22,
  19,167,167,163,157,21,137,180,122,123,58,34,125,12,12,12,88,12,19,20,86,
  12,85,12,84,83,82,81,80,79,19,20,77,77,38,87,75,42,71,70,85,85,60,59,57,
  55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,167,56,31,30,29,23,
32,33,39,40,41,42,35,47,137,137,157,22,19,167,167,163,157,21,137,181,122,
  123,58,34,125,216,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
151,182,217,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,183,122,123,
  58,34,125,38,46,46,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
137,137,137,184,44,56,
76,185,27,
177,38,178,
76,187,218,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,188,122,123,
  58,34,125,219,38,219,219,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
220,14,221,41,
35,190,3,90,
177,179,191,35,178,
159,159,159,192,222,
159,159,159,193,223,
32,33,35,137,137,157,22,19,167,167,163,157,21,137,194,122,123,58,34,125,38,
  113,114,114,37,36,114,34,25,24,28,20,27,26,167,56,31,30,29,23,
32,33,35,137,137,157,22,19,167,167,163,157,21,137,195,122,123,58,34,125,38,
  113,113,113,37,36,113,34,25,24,28,20,27,26,167,56,31,30,29,23,
114,116,118,93,119,117,115,
114,116,118,92,119,117,115,
41,42,90,121,120,
41,42,89,121,120,
122,124,87,125,123,
122,124,86,125,123,
122,124,85,125,123,
122,124,84,125,123,
126,128,130,132,82,133,131,129,127,
126,128,130,132,81,133,131,129,127,
134,136,79,137,135,
138,77,139,
140,75,141,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,209,122,123,
  58,34,125,38,113,224,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,
  43,34,25,24,28,20,27,26,167,56,31,30,29,23,
177,119,225,
179,211,117,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,212,122,123,
  58,34,125,38,113,226,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,
  36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,213,122,123,
  58,34,125,227,38,227,227,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
177,228,214,116,178,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,367,215,3,3,
  389,
76,216,36,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,217,122,123,
  58,34,125,229,38,229,229,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
32,33,39,40,41,42,35,37,137,137,157,22,19,167,167,163,157,21,137,218,122,
  123,58,34,125,230,186,38,39,186,186,60,59,57,55,54,53,52,51,50,49,48,47,
  46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
177,179,219,40,178,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,3,367,367,367,367,367,367,367,367,3,1,2,220,3,3,388,
32,33,39,40,41,42,64,65,66,78,35,63,67,68,69,72,73,74,76,137,137,157,22,19,
  167,167,163,157,21,137,221,122,123,58,34,125,16,17,88,17,19,20,86,17,85,
  17,84,83,82,81,80,79,19,20,77,77,38,87,75,71,70,85,85,60,59,57,55,54,53,
  52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  167,56,31,30,29,23,
160,160,160,151,160,
160,160,160,150,160,
142,73,143,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,225,122,123,
  58,34,125,38,121,121,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,
144,70,145,
231,177,227,178,232,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,3,3,1,2,228,3,3,379,
177,228,229,233,178,
179,230,21,
367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,367,3,
  1,2,231,3,3,406,
66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,232,234,
76,233,30,
32,33,39,40,41,42,35,137,137,157,22,19,167,167,163,157,21,137,234,122,123,
  58,34,125,38,67,113,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,167,56,31,30,29,23,

};


static const unsigned short ag_sbt[] = {
     0,  13,  52,  92,  98, 115, 151, 153, 190, 192, 220, 258, 296, 305,
   344, 410, 509, 608, 612, 645, 694, 699, 705, 760, 794, 797, 799, 806,
   841, 877, 910, 943, 976,1012,1048,1052,1082,1089,1096,1163,1189,1215,
  1241,1267,1270,1320,1370,1420,1470,1477,1482,1487,1496,1501,1504,1507,
  1510,1565,1568,1571,1576,1607,1610,1615,1624,1650,1661,1700,1741,1782,
  1793,1855,1861,1888,1897,1906,1939,1981,1984,1993,1999,2004,2009,2074,
  2077,2080,2085,2151,2154,2253,2257,2320,2419,2519,2575,2581,2584,2591,
  2594,2597,2604,2606,2612,2620,2628,2631,2634,2643,2646,2655,2658,2663,
  2689,2739,2746,2772,2822,2848,2898,2924,2974,3025,3076,3102,3154,3180,
  3232,3258,3311,3337,3390,3416,3469,3495,3548,3574,3628,3654,3708,3734,
  3789,3815,3871,3897,3954,3980,4000,4065,4091,4111,4137,4157,4183,4246,
  4272,4334,4360,4422,4448,4510,4536,4598,4624,4686,4712,4774,4800,4862,
  4888,4950,4976,5038,5064,5126,5152,5214,5240,5302,5328,5390,5453,5554,
  5619,5622,5684,5690,5693,5696,5699,5762,5766,5770,5775,5780,5785,5825,
  5865,5872,5879,5884,5889,5894,5899,5904,5909,5918,5927,5932,5935,5938,
  5996,5999,6002,6061,6124,6129,6170,6173,6236,6302,6307,6345,6443,6448,
  6453,6456,6518,6521,6526,6574,6579,6582,6608,6628,6631,6692
};


static const unsigned short ag_sbe[] = {
     9,  49,  89,  96, 103, 150, 152, 189, 191, 216, 254, 292, 301, 340,
   364, 440, 539, 609, 643, 692, 697, 702, 751, 790, 796, 798, 804, 840,
   875, 906, 939, 972,1008,1044,1049,1078,1085,1092,1116,1185,1211,1237,
  1263,1268,1288,1338,1388,1438,1473,1479,1484,1491,1498,1502,1505,1508,
  1561,1566,1569,1573,1591,1608,1612,1620,1646,1657,1696,1737,1778,1789,
  1811,1858,1884,1893,1902,1937,1977,1982,1989,1996,2001,2006,2028,2075,
  2078,2082,2104,2152,2184,2254,2275,2350,2450,2566,2578,2583,2589,2593,
  2596,2602,2605,2609,2617,2625,2629,2632,2639,2644,2651,2656,2660,2685,
  2707,2742,2768,2790,2844,2866,2920,2942,2992,3043,3098,3120,3176,3198,
  3254,3276,3333,3355,3412,3434,3491,3513,3570,3592,3650,3672,3730,3752,
  3811,3833,3893,3915,3976,3998,4019,4087,4109,4133,4155,4179,4201,4268,
  4290,4356,4378,4444,4466,4532,4554,4620,4642,4708,4730,4796,4818,4884,
  4906,4972,4994,5060,5082,5148,5170,5236,5258,5324,5346,5449,5484,5573,
  5620,5640,5687,5691,5694,5697,5717,5763,5767,5772,5778,5783,5799,5839,
  5868,5875,5881,5886,5891,5896,5901,5906,5913,5922,5929,5933,5936,5956,
  5997,6000,6020,6079,6126,6166,6171,6191,6255,6304,6341,6375,6446,6451,
  6454,6474,6519,6523,6570,6576,6580,6604,6626,6629,6649,6692
};


static const unsigned char ag_fl[] = {
  2,3,3,4,2,3,1,1,1,1,1,0,2,1,1,2,4,4,1,1,1,5,2,1,1,2,2,3,2,2,6,1,1,1,1,
  3,4,0,1,1,4,1,3,2,3,2,3,0,1,1,3,1,3,3,3,3,3,3,3,3,3,3,3,3,1,0,0,7,1,0,
  4,1,0,4,1,3,1,3,1,3,1,3,3,1,3,3,3,3,1,3,3,1,3,3,1,3,3,3,1,2,2,2,2,1,3,
  3,1,1,2,2,2,2,1,4,4,1,4,4,0,1,1,3,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,
  3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,3,0,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned short ag_ptt[] = {
    0, 12, 12, 12, 12, 12,194,195,196,197,198, 13, 13, 16, 16, 25, 25, 26,
   26, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 30, 31, 48, 17, 15,
   32, 51, 51, 33, 27, 28, 37, 41, 41, 42, 42, 18, 18, 36, 36, 59, 59, 59,
   59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 60, 76, 78, 60, 74, 81, 74, 79,
   84, 79, 82, 82, 85, 85, 87, 87, 89, 89, 89, 91, 91, 91, 91, 91, 94, 94,
   94, 99, 99, 99,102,102,102,102,105,105,105,105,105,109,109,112,112,112,
  112,112,112,112,112,112,112, 61, 61,117,120,120,121,121,114,114,114,114,
    1,126,126,127,127,  1,131,131,132,132,  1,206,206,259,259,259,136,136,
  136,139,139,139,145,145,138,138,141,141,258,258,258,142,142,146,146,143,
  143,144,144,156,156,148,148,157,157,122,122,261,160,160,161,161,163,163,
  163,164,164,164,164,164,164,164,164,164,164,164,165,165,165,178,179,180,
  166,166,183,183,260,185,185,147, 19, 19,123,123,125,125,125,125,125,125,
  125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,
  125,125,125,125,125,125,125,125,125,125,125,125,130,130,130,130,130,130,
  130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
  130,130,130,130,130,130,130,130,130,130,130,134,134,134,135,135,135,135,
  135,135,149,149,150,150,150,153,153,158,158,162,162,162,162,162,162,162,
  162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,
  162,162,162,162,162,162,162,162,186,186,186,186,186,186,186,186,186,186,
  186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,
  186,186,186,186,186,192,192,193,193,  7,  8,  9, 10, 11, 35, 34, 38, 39,
   40, 45, 44,  4, 43, 46, 47, 49, 50, 52, 53, 55, 54, 56, 57, 58, 62, 63,
   64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 77, 75, 80, 83, 86, 88, 90, 92,
   93, 95, 96, 97, 98,100,101,103,104,106,107,108,110,111,113,115,116,118,
  119,  2,  3,  6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: VNO AG_WRAP_5(ag_rp_5(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER); break;
    case 12: ag_rp_15(PCB_POINTER); break;
    case 13: ag_rp_16(PCB_POINTER, V(1,(FileLocation *)), V(3,(FileLocation *))); break;
    case 14: VWD(0,AG_WRAP_5 *); break;
    case 15: ag_rp_17(PCB_POINTER); break;
    case 16: ag_rp_18(PCB_POINTER); break;
    case 17: ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 18: ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_5 *); break;
    case 19: ag_rp_21(PCB_POINTER); break;
    case 20: ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 21: ag_rp_23(PCB_POINTER); break;
    case 22: V(0,(FileLocation *)) = ag_rp_24(PCB_POINTER); break;
    case 23: ag_rp_25(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 24: ag_rp_26(PCB_POINTER); break;
    case 25: ag_rp_27(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 26: ag_rp_28(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 27: ag_rp_29(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 28: ag_rp_30(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 29: VNO AG_WRAP_5(ag_rp_31(PCB_POINTER)); break;
    case 30: VRO(AG_WRAP_5 *, ag_rp_32(PCB_POINTER, VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 31: VRO(AG_WRAP_5 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 32: VRO(AG_WRAP_5 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 33: VRO(AG_WRAP_5 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 34: VRO(AG_WRAP_5 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 35: VRO(AG_WRAP_5 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 36: VRO(AG_WRAP_5 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 37: VRO(AG_WRAP_5 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 38: VRO(AG_WRAP_5 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 39: VRO(AG_WRAP_5 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: VRO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 43: ag_rp_45(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 44: ag_rp_46(PCB_POINTER, VW(-5, AG_WRAP_5 *), VW(-2, AG_WRAP_5 *)); break;
    case 45: VRO(AG_WRAP_5 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 46: ag_rp_48(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 47: VRO(AG_WRAP_5 *, ag_rp_49(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 48: ag_rp_50(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 49: VRO(AG_WRAP_5 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 50: VRO(AG_WRAP_5 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 51: VRO(AG_WRAP_5 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 52: VRO(AG_WRAP_5 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 53: VRO(AG_WRAP_5 *, ag_rp_55(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 54: VRO(AG_WRAP_5 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 55: VRO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 56: VRO(AG_WRAP_5 *, ag_rp_58(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 57: VRO(AG_WRAP_5 *, ag_rp_59(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 58: VRO(AG_WRAP_5 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 59: VRO(AG_WRAP_5 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 60: VRO(AG_WRAP_5 *, ag_rp_62(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 61: VRO(AG_WRAP_5 *, ag_rp_63(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 62: VRO(AG_WRAP_5 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 63: VRO(AG_WRAP_5 *, ag_rp_65(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 64: VRO(AG_WRAP_5 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 65: VRO(AG_WRAP_5 *, ag_rp_67(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 66: VNO AG_WRAP_5(ag_rp_68(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 67: VNO AG_WRAP_5(ag_rp_69(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 68: VNO AG_WRAP_5(ag_rp_70(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 69: VNO AG_WRAP_5(ag_rp_71(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 70: VRO(AG_WRAP_5 *, ag_rp_72(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 71: VNO AG_WRAP_5(ag_rp_73(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 72: VRO(AG_WRAP_5 *, ag_rp_74(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 73: VNO AG_WRAP_5(ag_rp_75(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 74: VNO AG_WRAP_5(ag_rp_76(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 75: VRO(AG_WRAP_5 *, ag_rp_77(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 76: VRO(AG_WRAP_5 *, ag_rp_78(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 77: VNO AG_WRAP_5(ag_rp_79(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 78: VNO AG_WRAP_5(ag_rp_80(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 79: VRO(AG_WRAP_4 *, ag_rp_81(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 80: VRO(AG_WRAP_5 *, ag_rp_82(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 81: VRO(AG_WRAP_4 *, ag_rp_83(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 82: VNO AG_WRAP_6(ag_rp_84(PCB_POINTER)); break;
    case 83: VRO(AG_WRAP_5 *, ag_rp_85(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 84: VRO(AG_WRAP_6 *, ag_rp_86(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 85: VNO AG_WRAP_5(ag_rp_87(PCB_POINTER, V(0,(long *)))); break;
    case 86: VNO AG_WRAP_5(ag_rp_88(PCB_POINTER, V(0,(double *)))); break;
    case 87: VNO AG_WRAP_5(ag_rp_89(PCB_POINTER, V(0,(int *)))); break;
    case 88: VNO AG_WRAP_4(ag_rp_90(PCB_POINTER, V(0,(int *)))); break;
    case 89: VRO(AG_WRAP_4 *, ag_rp_91(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 90: V(0,(double *)) = ag_rp_92(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 91: V(0,(double *)) = ag_rp_93(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 92: V(0,(double *)) = ag_rp_94(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 93: V(0,(double *)) = ag_rp_95(PCB_POINTER, V(0,(double *))); break;
    case 94: V(0,(double *)) = ag_rp_96(PCB_POINTER, V(1,(double *))); break;
    case 95: V(0,(double *)) = ag_rp_97(PCB_POINTER, V(0,(long *))); break;
    case 96: V(0,(double *)) = ag_rp_98(PCB_POINTER, V(0,(long *))); break;
    case 97: V(0,(double *)) = ag_rp_99(PCB_POINTER, V(0,(long *))); break;
    case 98: V(0,(long *)) = ag_rp_100(PCB_POINTER, V(1,(long *))); break;
    case 99: V(0,(long *)) = ag_rp_101(PCB_POINTER, V(1,(long *))); break;
    case 100: V(0,(double *)) = ag_rp_102(PCB_POINTER, V(0,(int *))); break;
    case 101: V(0,(double *)) = ag_rp_103(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 102: V(0,(long *)) = ag_rp_104(PCB_POINTER, V(0,(int *))); break;
    case 103: V(0,(long *)) = ag_rp_105(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 104: V(0,(long *)) = ag_rp_106(PCB_POINTER, V(0,(int *))); break;
    case 105: V(0,(long *)) = ag_rp_107(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 106: V(0,(long *)) = ag_rp_108(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 107: V(0,(long *)) = ag_rp_109(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 108: V(0,(long *)) = ag_rp_110(PCB_POINTER); break;
    case 109: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 110: V(0,(long *)) = ag_rp_112(PCB_POINTER); break;
    case 111: V(0,(long *)) = ag_rp_113(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 112: V(0,(int *)) = ag_rp_114(PCB_POINTER, V(0,(int *))); break;
    case 113: V(0,(int *)) = ag_rp_115(PCB_POINTER, V(0,(int *))); break;
    case 114: VRO(AG_WRAP_5 *, ag_rp_116(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 115: VNO AG_WRAP_5(ag_rp_117(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 116: VNO AG_WRAP_4(ag_rp_118(PCB_POINTER)); break;
    case 117: VRO(AG_WRAP_4 *, ag_rp_119(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 118: V(0,(int *)) = ag_rp_120(PCB_POINTER); break;
    case 119: V(0,(int *)) = ag_rp_121(PCB_POINTER); break;
    case 120: V(0,(int *)) = ag_rp_122(PCB_POINTER); break;
    case 121: V(0,(int *)) = ag_rp_123(PCB_POINTER); break;
    case 122: V(0,(int *)) = ag_rp_124(PCB_POINTER); break;
    case 123: V(0,(int *)) = ag_rp_125(PCB_POINTER); break;
    case 124: V(0,(int *)) = ag_rp_126(PCB_POINTER); break;
    case 125: V(0,(int *)) = ag_rp_127(PCB_POINTER); break;
    case 126: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 127: V(0,(int *)) = ag_rp_129(PCB_POINTER); break;
    case 128: V(0,(int *)) = ag_rp_130(PCB_POINTER); break;
    case 129: V(0,(int *)) = ag_rp_131(PCB_POINTER, V(1,(int *))); break;
    case 130: V(0,(int *)) = ag_rp_132(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 131: V(0,(int *)) = ag_rp_133(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 132: V(0,(int *)) = ag_rp_134(PCB_POINTER, V(1,(int *))); break;
    case 133: V(0,(int *)) = ag_rp_135(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 134: V(0,(int *)) = ag_rp_136(PCB_POINTER, V(0,(int *))); break;
    case 135: V(0,(int *)) = ag_rp_137(PCB_POINTER, V(0,(int *))); break;
    case 136: V(0,(int *)) = ag_rp_138(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[262] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "syntax selection",
  "statement list",
  "eof",
  "loop condition",
  "statement",
  "DO WHILE",
  "optional expression",
  "",
  "\"script\"",
  "\"whileLoop\"",
  "\"doLoop\"",
  "\"statement\"",
  "\"forField\"",
  "open statement",
  "closed statement",
  "if condition",
  "else",
  "simple statement",
  "DO",
  "WHILE",
  "for initialization",
  "for field",
  "';'",
  "')'",
  "expression",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "\"array\"",
  "'['",
  "']'",
  "\"do\"",
  "\"while\"",
  "FOR",
  "\"for\"",
  "'('",
  "",
  "\"if\"",
  "\"else\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "conditional expression",
  "lvalue",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "logical or expression",
  "'\\?'",
  "",
  "':'",
  "",
  "logical and expression",
  "\"||\"",
  "",
  "inclusive or expression",
  "\"&&\"",
  "",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "\"==\"",
  "\"!=\"",
  "shift expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "additive expression",
  "\"<<\"",
  "\">>\"",
  "multiplicative expression",
  "'+'",
  "'-'",
  "unary expression",
  "'*'",
  "'/'",
  "'%'",
  "factor",
  "'!'",
  "'~'",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "')'",
  "';'",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "']'",
  "'['",
  "name",
  "\"array\"",
  "\"do\"",
  "\"while\"",
  "\"for\"",
  "'('",
  "\"if\"",
  "\"else\"",
  "'}'",
  "'{'",
  "\"dump\"",
  "','",
  "\"print\"",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'+'",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned short)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned short)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


