/*
 ast.syn -- Create an Abstract Syntax Tree

 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. Reduction procedures have been added to enable it
 to create an abstract syntax tree as it parses. Other than the addition of
 reduction procedures, there are no changes to the syntax.

 .syn files are compiled with the AnaGram parser generator, which creates a
 corresponding parser file and header file.

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that executes from an abstract syntax tree, compile and link
 as a single single project:
   astxi.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that compiles bytecode from an abstract syntax tree, compile
 and link as a single single project:
   astci.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 This parser analyzes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"
#include <math.h>
#include "astdefs.h"         // Defines node structure of abstract syntax tree

// Forward reference for parxer control block
struct parse_pcb_struct;

struct Tree : public AbstractSyntaxTree {
  parse_pcb_struct *pcbPointer;
  FileLocation context() const;
};

#define AST PCB.ast

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef AST_H_1027348054
#include "ast.h"
#endif

#ifndef AST_H_1027348054
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE parse_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer
static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 533, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

FileLocation Tree::context() const {
  return PCONTEXT(*pcbPointer);
}

// Function definitions for parser control block

// Constructor
parse_pcb_struct::parse_pcb_struct(const char *text)
  : pointer((const unsigned char *) text), loopDepth(0)
{
  ast.pcbPointer = this;
}

void parse_pcb_struct::checkLoop() {
  if (loopDepth) return;
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): No loop active", context.line, context.column);
  throw ErrorDiagnostic(buf);
}

// Record syntax error
void parse_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

// External Interface
AbstractSyntaxTree buildTree(const char *text) {
  parse_pcb_type pcb(text);
  parse(&pcb);
  return pcb.ast;
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif

static inline void ag_rp_1(PCB_DECL, AgStack<AstNode *> &sl) {
/* Line 139, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  AST.root = AST.node(typeScript, AST.listNode(typeStatementBlock, sl));
}

static inline AgStack<AstNode *> ag_rp_2(PCB_DECL) {
/* Line 143, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_3(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 144, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_4(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 162, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeIfStatement, x, s);
}

static inline AstNode * ag_rp_5(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 164, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeIfElseStatement, x, s1, s2);
}

static inline AstNode * ag_rp_6(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 166, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement, x, s);
}

static inline AstNode * ag_rp_7(PCB_DECL, AstNode * init, AstNode * cond, AstNode * inc, AstNode * s) {
/* Line 169, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.loopDepth--, AST.node(typeForStatement, init, cond, inc, s);
}

static inline AstNode * ag_rp_8(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 173, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeIfElseStatement,x,s1,s2);
}

static inline AstNode * ag_rp_9(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 175, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement,x,s);
}

static inline AstNode * ag_rp_10(PCB_DECL, AstNode * init, AstNode * cond, AstNode * inc, AstNode * s) {
/* Line 178, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.loopDepth--, AST.node(typeForStatement,init, cond, inc, s);
}

static inline AstNode * ag_rp_11(PCB_DECL, AstNode * x) {
/* Line 179, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_12(PCB_DECL, AstNode * s, AstNode * x) {
/* Line 188, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.loopDepth--, AST.node(typeDoStatement,s,x);
}

static inline AstNode * ag_rp_13(PCB_DECL, AstNode * x) {
/* Line 189, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeExpressionStatement,x);
}

static inline AstNode * ag_rp_14(PCB_DECL) {
/* Line 190, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_15(PCB_DECL) {
/* Line 192, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.checkLoop(), AST.node(typeBreak);
}

static inline AstNode * ag_rp_16(PCB_DECL) {
/* Line 193, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return PCB.checkLoop(), AST.node(typeContinue);
}

static inline AstNode * ag_rp_17(PCB_DECL, AstNode * x) {
/* Line 194, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeReturn, x);
}

static inline AstNode * ag_rp_18(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 195, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.listNode(typeDump, ns);
}

static inline AstNode * ag_rp_19(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 196, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.listNode(typePrint, ns);
}

static inline void ag_rp_20(PCB_DECL) {
/* Line 199, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_21(PCB_DECL) {
/* Line 202, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_22(PCB_DECL) {
/* Line 205, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  PCB.loopDepth++;
}

static inline AstNode * ag_rp_23(PCB_DECL, AstNode * x) {
/* Line 208, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_24(PCB_DECL, AgStack<AstNode *> &s) {
/* Line 211, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.listNode(typeStatementBlock, s);
}

static inline AgStack<AstNode *> ag_rp_25(PCB_DECL, AgString &k) {
/* Line 215, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgStack<AstNode *>().push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_26(PCB_DECL, AgStack<AstNode *> &ns, AgString &k) {
/* Line 216, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return ns.push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_27(PCB_DECL, AstNode * x) {
/* Line 220, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_28(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 221, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_29(PCB_DECL) {
/* Line 224, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_30(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 231, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeCommaExpression,x,y);
}

static inline AstNode * ag_rp_31(PCB_DECL, AstNode * lv, Opcode op, AstNode * x) {
/* Line 237, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.assignmentNode(lv, op, x);
}

static inline AstNode * ag_rp_32(PCB_DECL, AstNode * c, AstNode * x, AstNode * y) {
/* Line 243, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeConditionalExpression,c,x,y);
}

static inline AstNode * ag_rp_33(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 249, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeLogicalOrExpression, x, y);
}

static inline AstNode * ag_rp_34(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 254, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.node(typeLogicalAndExpression, x, y);
}

static inline AstNode * ag_rp_35(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 260, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(IOR, x, y);
}

static inline AstNode * ag_rp_36(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 265, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(XOR, x, y);
}

static inline AstNode * ag_rp_37(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 269, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(AND, x, y);
}

static inline AstNode * ag_rp_38(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 274, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_39(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 279, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_40(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 284, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_41(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 289, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_42(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 294, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_43(PCB_DECL, AstNode * x) {
/* Line 298, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_44(PCB_DECL, Opcode op, AstNode * x) {
/* Line 299, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(op, x);
}

static inline AstNode * ag_rp_45(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 307, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.binaryNode(POW, x, y);
}

static inline AstNode * ag_rp_46(PCB_DECL, AstNode * x) {
/* Line 315, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_47(PCB_DECL, AstNode * x) {
/* Line 317, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(FETCH, x);
}

static inline AstNode * ag_rp_48(PCB_DECL, AstNode * x) {
/* Line 318, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(I_FETCH, x);
}

static inline AstNode * ag_rp_49(PCB_DECL, AstNode * x) {
/* Line 319, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(D_FETCH, x);
}

static inline AstNode * ag_rp_50(PCB_DECL, AstNode * x) {
/* Line 320, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(FETCH_I, x);
}

static inline AstNode * ag_rp_51(PCB_DECL, AstNode * x) {
/* Line 321, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(FETCH_D, x);
}

static inline AstNode * ag_rp_52(PCB_DECL, AstNode * x) {
/* Line 323, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(CAST_LONG, x);
}

static inline AstNode * ag_rp_53(PCB_DECL, AstNode * x) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.unaryNode(CAST_DOUBLE, x);
}

static inline AstNode * ag_rp_54(PCB_DECL, AgString &n) {
/* Line 327, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.variableNode(n);
}

static inline AstNode * ag_rp_55(PCB_DECL, AgString &k, AgStack<AstNode *> &args) {
/* Line 330, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.functionCallNode(k, args);
}

static inline AgStack<AstNode *> ag_rp_56(PCB_DECL) {
/* Line 333, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_57(PCB_DECL, AstNode * x) {
/* Line 337, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_58(PCB_DECL, AgStack<AstNode *> &al, AstNode * x) {
/* Line 338, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return al.push(x);
}

static inline AstNode * ag_rp_59(PCB_DECL, long x) {
/* Line 341, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_60(PCB_DECL, double x) {
/* Line 342, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_61(PCB_DECL, Value &x) {
/* Line 343, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_62(PCB_DECL, int x) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AST.constantNode(x);
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 348, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return STORE;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 349, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return ADDM;
}

static inline Opcode ag_rp_65(PCB_DECL) {
/* Line 350, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return SUBM;
}

static inline Opcode ag_rp_66(PCB_DECL) {
/* Line 351, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return MULM;
}

static inline Opcode ag_rp_67(PCB_DECL) {
/* Line 352, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return DIVM;
}

static inline Opcode ag_rp_68(PCB_DECL) {
/* Line 353, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return MODM;
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 354, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return IORM;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 355, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return ANDM;
}

static inline Opcode ag_rp_71(PCB_DECL) {
/* Line 356, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return XORM;
}

static inline Opcode ag_rp_72(PCB_DECL) {
/* Line 357, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return LSM;
}

static inline Opcode ag_rp_73(PCB_DECL) {
/* Line 358, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return RSM;
}

static inline Opcode ag_rp_74(PCB_DECL) {
/* Line 359, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return POWM;
}

static inline Opcode ag_rp_75(PCB_DECL) {
/* Line 362, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return EQ;
}

static inline Opcode ag_rp_76(PCB_DECL) {
/* Line 363, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return NE;
}

static inline Opcode ag_rp_77(PCB_DECL) {
/* Line 366, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return LT;
}

static inline Opcode ag_rp_78(PCB_DECL) {
/* Line 367, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return LE;
}

static inline Opcode ag_rp_79(PCB_DECL) {
/* Line 368, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return GT;
}

static inline Opcode ag_rp_80(PCB_DECL) {
/* Line 369, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return GE;
}

static inline Opcode ag_rp_81(PCB_DECL) {
/* Line 372, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return LS;
}

static inline Opcode ag_rp_82(PCB_DECL) {
/* Line 373, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return RS;
}

static inline Opcode ag_rp_83(PCB_DECL) {
/* Line 376, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return ADD;
}

static inline Opcode ag_rp_84(PCB_DECL) {
/* Line 377, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return SUB;
}

static inline Opcode ag_rp_85(PCB_DECL) {
/* Line 380, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return MUL;
}

static inline Opcode ag_rp_86(PCB_DECL) {
/* Line 381, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return DIV;
}

static inline Opcode ag_rp_87(PCB_DECL) {
/* Line 382, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return MOD;
}

static inline Opcode ag_rp_88(PCB_DECL) {
/* Line 385, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return NEG;
}

static inline Opcode ag_rp_89(PCB_DECL) {
/* Line 386, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return NOT;
}

static inline Opcode ag_rp_90(PCB_DECL) {
/* Line 387, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return COM;
}

static inline AgString ag_rp_91(PCB_DECL, int c) {
/* Line 405, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_92(PCB_DECL, AgString &ns, int c) {
/* Line 406, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return ns.append(c);
}

static inline double ag_rp_93(PCB_DECL, double x, long e) {
/* Line 412, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_94(PCB_DECL, double x, long e) {
/* Line 413, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_95(PCB_DECL, double i, double f) {
/* Line 416, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return i+f;
}

static inline double ag_rp_96(PCB_DECL, double i) {
/* Line 417, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return i;
}

static inline double ag_rp_97(PCB_DECL, double f) {
/* Line 418, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return f;
}

static inline double ag_rp_98(PCB_DECL, long x) {
/* Line 421, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x;
}

static inline double ag_rp_99(PCB_DECL, long x) {
/* Line 422, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x;
}

static inline double ag_rp_100(PCB_DECL, long x) {
/* Line 423, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return makeDecimal(x);
}

static inline long ag_rp_101(PCB_DECL, long x) {
/* Line 426, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return (long)x;
}

static inline long ag_rp_102(PCB_DECL, long x) {
/* Line 427, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return -(long)x;
}

static inline double ag_rp_103(PCB_DECL, int d) {
/* Line 430, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_104(PCB_DECL, int d, double f) {
/* Line 431, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_105(PCB_DECL, int d) {
/* Line 439, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return d-'0';
}

static inline long ag_rp_106(PCB_DECL, long x, int d) {
/* Line 440, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_107(PCB_DECL, int d) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return d-'0';
}

static inline long ag_rp_108(PCB_DECL, long x, int d) {
/* Line 444, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_109(PCB_DECL, long x, int d) {
/* Line 447, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_110(PCB_DECL, long x, int d) {
/* Line 448, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_111(PCB_DECL) {
/* Line 451, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 0;
}

static inline long ag_rp_112(PCB_DECL, long n, int d) {
/* Line 452, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_113(PCB_DECL) {
/* Line 455, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 0;
}

static inline long ag_rp_114(PCB_DECL, long n, int d) {
/* Line 456, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_115(PCB_DECL, int x) {
/* Line 459, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return x - '0';
}

static inline int ag_rp_116(PCB_DECL, int d) {
/* Line 460, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_117(PCB_DECL, Value &s, Value &e) {
/* Line 466, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return s+=e;
}

static inline Value ag_rp_118(PCB_DECL, AgString &b) {
/* Line 469, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return Value(b);
}

static inline AgString ag_rp_119(PCB_DECL) {
/* Line 473, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return AgString();
}

static inline AgString ag_rp_120(PCB_DECL, AgString &s, int c) {
/* Line 474, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return s.append(c);
}

static inline int ag_rp_121(PCB_DECL) {
/* Line 486, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\'';
}

static inline int ag_rp_122(PCB_DECL) {
/* Line 487, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '"';
}

static inline int ag_rp_123(PCB_DECL) {
/* Line 488, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\?';
}

static inline int ag_rp_124(PCB_DECL) {
/* Line 489, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\\';
}

static inline int ag_rp_125(PCB_DECL) {
/* Line 490, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\a';
}

static inline int ag_rp_126(PCB_DECL) {
/* Line 491, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\b';
}

static inline int ag_rp_127(PCB_DECL) {
/* Line 492, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\f';
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 493, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\n';
}

static inline int ag_rp_129(PCB_DECL) {
/* Line 494, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\r';
}

static inline int ag_rp_130(PCB_DECL) {
/* Line 495, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\t';
}

static inline int ag_rp_131(PCB_DECL) {
/* Line 496, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return '\v';
}

static inline int ag_rp_132(PCB_DECL, int d) {
/* Line 502, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return d-'0';
}

static inline int ag_rp_133(PCB_DECL, int n, int d) {
/* Line 505, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_134(PCB_DECL, int n, int d) {
/* Line 508, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_135(PCB_DECL, int d) {
/* Line 511, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return d;
}

static inline int ag_rp_136(PCB_DECL, int n, int d) {
/* Line 512, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_137(PCB_DECL, int d) {
/* Line 515, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return d-'0';
}

static inline int ag_rp_138(PCB_DECL, int d) {
/* Line 516, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_139(PCB_DECL, int c) {
/* Line 525, W:/agdev/agex/packages/xidek/cll/base/ast.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 4,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<AstNode *> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static parse_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,  0,
   15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,  0,  0, 30,
    0, 31,  0, 32,  0, 33,  0, 34,  0, 35,  0, 36,  0, 37,  0, 38,  0, 39,
    0, 40,  0, 41,  0, 42,  0, 43, 44,  0, 45, 46,  0, 47, 48, 49, 50, 51,
    0, 52, 53, 54, 55, 56,  0, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67,
   68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85,
   86, 87, 88, 89, 90,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 91, 92,
    0, 93, 94, 95, 96, 97, 98, 99,100,  0,  0,101,102,103,104,  0,  0,  0,
  105,106,107,108,109,110,111,112,  0,  0,113,114,115,116,  0,117,118,119,
  120,  0,  0,  0,  0,  0,121,122,123,124,125,126,127,128,129,130,131,  0,
    0,  0,132,133,134,135,136,137,138,139
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,177,  0,181,  0,182,  0,183,  0,184,  0,185,  0,186,  0,187,
  0,191,  0,193,  0,205,  0,206,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 88,120,255,111,117,255, 43, 45, 47, 48, 98, 99,100,102,
  105,112,114,119,255, 42,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255,
   45, 61,255, 42, 47, 61,255, 88,120,255, 61,255, 60, 61,255, 61,255, 61,
   62,255,117,255,111,117,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 48,
   60, 61, 62, 94, 98, 99,100,101,102,105,108,112,114,119,124,255, 88,120,
  255,111,117,255, 43, 45, 48, 98, 99,100,102,105,112,114,119,255, 34, 39,
   63, 92, 97, 98,102,110,114,116,118,120,255, 92,255, 42, 47,255, 60, 61,
  255, 61, 62,255, 33, 38, 42, 47, 60, 61, 62,124,255, 60, 61,255, 61, 62,
  255, 33, 38, 42, 60, 61, 62,124,255, 42, 47,255, 88,120,255, 43, 45, 47,
   48,100,108,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,255, 88,120,255,
   43, 45, 48,100,108,255, 88,120,255, 43, 45, 48,255, 60, 61,255, 61, 62,
  255, 33, 38, 60, 61, 62,124,255, 33, 38, 60, 61, 62,124,255, 33, 38, 61,
  124,255, 38,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,
  255, 42, 47, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255,
   33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255,124,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 61,255, 60, 61,255, 61,255,
   61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,
  255, 42, 47,255, 47,255, 42, 47,255, 88,120,255,111,117,255, 43, 45, 47,
   48, 98, 99,100,101,102,105,112,114,119,255, 60, 61,255, 61, 62,255, 33,
   38, 42, 43, 45, 60, 61, 62,124,255,119,255, 88,120,255,111,117,255, 43,
   45, 48, 98, 99,100,101,102,105,112,114,119,255, 42, 47,255, 88,120,255,
   60, 61,255, 61, 62,255,111,117,255, 33, 38, 42, 43, 45, 47, 48, 60, 61,
   62, 98, 99,100,102,105,112,114,119,124,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,4,3,4,0,0,4,0,4,1,0,4,0,0,
  4,0,0,4,0,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,7,4,6,7,4,0,0,4,3,3,2,2,2,2,
  2,2,2,3,2,3,7,7,2,7,7,7,7,7,7,7,2,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,
  4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,
  0,4,0,0,4,3,3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,0,0,4,3,3,2,
  2,4,0,0,4,3,3,2,7,7,4,0,0,4,3,3,2,4,0,0,4,0,0,4,3,3,2,3,2,3,4,3,3,3,3,
  3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,4,1,0,4,0,
  4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,
  4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,3,2,4,0,0,4,2,4,0,0,4,
  0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,7,4,0,0,4,0,0,4,3,3,3,3,3,2,3,2,3,
  4,7,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,7,4,0,0,4,0,0,4,0,0,4,0,0,4,5,
  7,4,3,3,3,3,3,2,2,2,3,2,7,7,2,7,7,7,7,7,3,4
};

static const unsigned char ag_key_parm[] = {
    0,107,112,  0,138,137,  0, 10, 16,  0,203,204,  0,  0,  4,  6,  0, 12,
   14, 18,  8,  2,  0,111,  0,197,217,  0,221,  0,202,213,  0,203,211,  0,
  204,212,  0,107,112,214,  0,138,137,  0,219,  0,228,225,  0,220,  0,227,
  229,  0, 22,  0, 10, 16,  0,216,196,  0,223,215,  0,  0,  0,  0,  0,  0,
    0,222,  0,218,  4,  6,  0,  0, 12, 14, 20, 18,  8,  2,  0,  0,138,137,
    0, 10, 16,  0,203,204,  0,  4,  6,  0, 12, 14, 18,  8,  2,  0,151,150,
  152,153,154,155,156,157,158,159,160,165,  0,  0,  0,107,112,  0,228,225,
    0,227,229,  0,223,197,202,  0,  0,222,  0,196,  0,228,225,  0,227,229,
    0,223,197,202,  0,222,  0,196,  0,107,112,  0,138,137,  0,203,204,  0,
    0, 22, 20,  0,107,112,  0,138,137,  0,203,204,  0,  0,  0,138,137,  0,
  203,204,  0, 22, 20,  0,138,137,  0,203,204,  0,  0,228,225,  0,227,229,
    0,223,197,  0,222,  0,196,  0,223,197,225,222,227,196,  0,223,197,222,
  196,  0,197,196,  0,197,217,  0,221,  0,202,213,  0,203,211,  0,204,212,
    0,107,112,214,  0,219,  0,228,225,  0,220,  0,227,229,  0,216,196,  0,
  223,215,  0,  0,  0,  0,  0,  0,222,  0,218,  0,  0,196,  0,197,217,  0,
  221,  0,202,213,  0,203,211,  0,204,212,  0,219,  0,228,225,  0,220,  0,
  227,229,  0,216,196,  0,223,215,  0,  0,  0,  0,214,  0,222,  0,218,  0,
    0,107,112,  0,  0,  0,107,112,  0,138,137,  0, 10, 16,  0,203,204,  0,
    0,  4,  6,  0,  0, 12, 14, 18,  8,  2,  0,228,225,  0,227,229,  0,223,
  197,202,203,204,  0,222,  0,196,  0,  2,  0,138,137,  0, 10, 16,  0,203,
  204,  0,  4,  6,  0,  0, 12, 14, 18,  8,  2,  0,107,112,  0,138,137,  0,
  228,225,  0,227,229,  0, 10, 16,  0,223,197,202,203,204,  0,  0,  0,222,
    0,  4,  6,  0, 12, 14, 18,  8,  2,196,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0,  0,  0,  0,  0, 17,  0,  0,  2,  1,  4,  4,  9,  7, 20,
   23, 25, 30, 36,  0, 41,  0,  0,  0,  0,  0,  0, 28,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 46,  0,  0,  0,  0,  0,
   51,  0, 64,  0, 56, 68,  0,  0,  0,  0, 43, 45, 25, 30, 33, 36, 39, 43,
   48, 47, 53, 49, 51, 56, 58, 71, 75, 78, 80, 84, 89, 95, 61,  0,  0,  0,
    0,  0,117,  0,100,102, 88,104,109, 91,120,123,125,130,136,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,106,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,141,143,145,121,124,147,127,149,  0,  0,  0,  0,  0,  0,
    0,151,153,155,139,157,142,159,  0,  0,  0,  0,  0,  0,  0,161,163,153,
  156,165,171,  0,  0,  0,  0,  0,  0,  0,175,177,166,169,  0,  0,  0,  0,
  179,181,177,183,189,  0,  0,  0,  0,193,195,186,  0,  0,  0,  0,  0,  0,
    0,197,199,193,201,196,203,  0,205,207,209,211,213,215,  0,217,219,221,
  223,  0,225,227,  0,  0,  0,  0,  0,  0,224,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,239,  0,  0,  0,  0,  0,244,  0,  0,  0,  0,
  229,231,221,226,229,232,235,241,233,246,235,249,  0,237,  0,  0,  0,  0,
    0,  0,270,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,281,  0,  0,  0,  0,
    0,286,  0,  0,  0,  0,239,241,267,272,275,278,243,283,245,288,247,291,
    0,  0,  0,  0,307,  0,  0,  0,  0,  0,  0,  0,  0,266,  0,249,251,312,
  315,253,258,318,269,273,276,278,283,289,  0,  0,  0,  0,  0,  0,  0,294,
  296,298,300,302,335,304,338,306,  0,308,  0,  0,  0,  0,  0,330,  0,313,
  315,353,317,322,356,333,337,340,342,347,353,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,383,  0,358,360,362,364,366,372,375,378,368,
  381,370,375,384,386,389,391,396,402,407,  0
};

static const unsigned short ag_key_index[] = {
   10,  0, 23, 64, 94,  0,  0, 23, 23, 94,119,  0,  0,119,130,  0,130,130,
  130,130,130,130,130,130,130,145,159,172,172,172,  0,  0,180,172,145,189,
  189,199,199,199,206,213,218,218,218,252,265,294,  0,172,310, 10,294, 10,
  189,  0,172,310,310, 94,321,310,310,310,  0,  0,189,  0,  0,  0, 94,  0,
    0,  0, 94,119,130,  0,  0,119,119,119,  0,130,  0,  0,310,  0,310,  0,
    0,172,189,341,172,172,172,189,189,172,172,189,172,172,172,172,189,172,
  172,189,172,189,172,189,172,189,172,189,172,189,189,172,189,172,172,172,
  172,172,172,172,172,172,172,172,172,189, 94,172,189,  0,  0,  0,189,351,
  189,189,189,359,  0,  0,387,189,189,199,199,199,206,213,218,218,  0,  0,
    0,321,  0,  0,  0,  0, 10, 94,130,130,189,172,189,189,189, 94,  0,  0,
    0,189,  0, 94
};

static const unsigned char ag_key_ends[] = {
43,0, 45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
104,105,108,101,0, 47,0, 61,0, 61,0, 61,0, 61,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 98,108,101,0, 109,112,0, 
108,115,101,0, 111,114,0, 102,0, 111,110,103,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 43,0, 45,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 
42,0, 61,0, 124,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 61,0, 38,0, 61,0, 124,0, 61,0, 
38,0, 61,0, 61,0, 61,0, 124,0, 61,0, 38,0, 61,0, 124,0, 
38,0, 124,0, 61,0, 61,0, 61,0, 61,0, 124,0, 61,0, 61,0, 
61,0, 61,0, 61,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 104,105,108,101,0, 
43,0, 45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 61,0, 38,0, 42,0, 
43,0, 45,0, 61,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
104,105,108,101,0, 124,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,171,171,171,171,171,171,171,171,170,116,170,170,170,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,170,234,142,171,
  171,233,200,167,179,178,231,201,192,230,123,232,135,172,172,172,172,172,
  172,172,134,134,194,180,224,210,226,195,171,173,173,173,173,120,173,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,171,164,171,199,174,171,173,173,173,173,120,173,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,189,198,188,
  235,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (parse_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,170,167,142,138,137,135,134,123,120,116,112,107,9,0,1,175,176,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,167,164,142,135,134,123,120,116,0,114,115,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,167,164,142,135,134,123,120,116,111,0,109,
  110,
170,116,112,107,0,1,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,9,0,7,8,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,167,164,142,135,134,123,120,0,
116,0,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,167,164,142,135,134,123,120,116,0,
111,0,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,9,0,2,3,4,5,6,10,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,36,38,39,40,41,43,46,48,
  50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,
  125,126,127,131,139,190,207,208,209,236,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,167,165,164,160,159,158,157,156,155,154,153,
  152,151,150,142,135,134,123,120,0,143,
172,135,134,0,130,
172,135,134,0,124,130,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,165,164,160,159,158,157,156,155,154,153,152,
  151,150,142,135,134,123,120,0,146,147,148,149,161,162,163,168,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,170,142,116,112,107,0,1,175,176,
123,120,0,
120,0,
173,172,135,134,120,0,140,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,172,170,135,134,123,120,116,112,107,0,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,172,170,135,134,123,120,116,112,107,0,130,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,170,116,112,107,0,1,175,176,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,170,116,112,107,0,1,175,176,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,170,116,112,107,0,1,175,176,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,174,173,170,120,116,112,107,0,1,175,176,
233,232,231,230,229,228,227,226,225,224,223,222,202,201,200,199,198,197,196,
  195,194,192,180,178,174,173,170,120,116,112,107,0,1,175,176,
142,0,5,236,
235,234,230,206,205,204,203,201,180,179,178,174,173,172,170,167,142,138,137,
  135,134,123,120,116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
174,173,120,0,4,41,190,
174,173,120,0,4,41,190,
235,234,230,206,205,204,203,201,179,174,173,172,167,142,138,137,135,134,123,
  120,0,2,3,4,5,6,16,17,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,
  68,69,71,72,73,74,75,76,79,100,104,105,119,122,125,126,127,131,139,190,
  207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
202,0,70,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,126,
  127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,126,
  127,131,139,190,207,208,209,236,
233,232,231,0,65,101,102,103,
230,201,0,63,67,100,
229,228,0,61,98,99,
227,226,225,224,0,59,94,95,96,97,
223,222,0,57,92,93,
200,0,55,
199,0,53,
198,197,0,49,51,
233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,218,217,216,215,
  214,213,212,211,210,204,203,202,201,200,199,198,197,196,195,194,192,180,
  179,178,174,173,172,170,135,134,120,116,112,107,0,1,175,176,
196,0,47,
179,0,16,
195,0,44,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
174,173,170,120,116,112,107,0,1,175,176,
235,234,230,204,203,201,193,191,189,188,187,186,185,184,183,182,181,180,179,
  174,173,172,170,167,142,138,137,135,134,123,120,116,112,107,0,1,175,176,
221,220,219,218,217,216,215,214,213,212,211,210,204,203,0,42,72,73,80,81,82,
  83,84,85,86,87,88,89,90,91,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,170,167,142,138,137,135,134,123,120,116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,
  73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
174,173,120,0,4,190,
235,234,230,204,203,201,180,179,174,173,172,170,167,142,138,137,135,134,123,
  120,116,112,107,0,1,175,176,
180,170,116,112,107,0,1,175,176,
180,170,116,112,107,0,1,175,176,
235,234,230,204,203,201,193,191,189,188,187,186,185,184,183,182,181,180,179,
  174,173,172,167,142,138,137,135,134,123,120,0,8,
235,234,230,204,203,201,193,191,189,188,187,186,185,184,183,182,181,180,179,
  178,177,174,173,172,170,167,142,138,137,135,134,123,120,116,112,107,9,0,
  1,175,176,
179,170,116,112,107,0,1,175,176,
179,170,116,112,107,0,1,175,176,
179,170,116,112,107,0,1,175,176,
192,180,0,21,37,
192,180,0,21,37,
235,234,230,204,203,201,180,179,174,173,172,167,142,138,137,135,134,123,120,
  0,2,3,4,5,6,16,17,20,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,
  69,71,72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,
  209,236,
180,0,21,
180,0,21,
192,180,0,21,37,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,10,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,36,38,39,40,41,43,46,48,
  50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,
  125,126,127,131,139,190,207,208,209,236,
179,0,16,
179,0,16,
179,0,16,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,10,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,36,38,39,40,41,43,46,48,
  50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,
  125,126,127,131,139,190,207,208,209,236,
235,234,233,232,231,230,226,224,210,201,200,199,198,195,194,192,189,188,180,
  179,178,174,173,172,171,170,167,165,164,160,159,158,157,156,155,154,153,
  152,151,150,142,135,134,123,120,0,144,146,147,148,149,161,162,163,
172,135,134,0,124,130,
172,135,0,
173,172,135,134,120,0,166,
172,135,0,
172,135,0,
173,172,135,134,120,0,166,
167,0,
172,135,134,0,124,130,
230,201,172,135,134,0,121,128,
230,201,172,135,134,0,121,128,
178,170,116,112,107,0,1,175,176,
178,0,18,
178,170,116,112,107,0,1,175,176,
178,0,18,
192,178,0,18,37,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,126,
  127,131,139,190,207,208,209,236,
204,203,0,72,73,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,126,
  127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,
  126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,
  125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,
  122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,
  122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,
  119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,
  105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,
  104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,
  100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,
  79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,178,174,173,172,167,142,138,137,135,134,123,120,
  0,2,3,4,5,6,16,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,
  72,73,74,77,78,79,100,104,105,119,122,125,126,127,131,139,190,207,208,
  209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,17,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,
  72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,
  73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,193,191,189,188,187,186,185,184,183,182,181,180,179,
  174,173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,10,11,12,13,15,
  16,17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,38,39,40,41,43,
  46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,
  119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,
  73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
174,173,120,0,4,190,
192,0,37,
180,0,21,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,
  73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
181,0,24,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,17,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,
  72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,180,179,174,173,172,167,142,138,137,135,134,123,120,
  0,2,3,4,5,6,16,17,20,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,
  69,71,72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,
  209,236,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,17,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,
  72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
177,0,14,
172,135,134,0,129,
172,135,134,0,129,
235,234,233,232,231,230,229,228,227,226,225,224,223,222,204,203,202,201,200,
  199,198,197,196,195,194,193,192,191,189,187,186,185,184,183,182,181,180,
  179,178,174,173,172,170,167,142,138,137,135,134,123,120,116,112,107,0,1,
  175,176,
204,203,179,174,173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,16,41,
  69,71,72,73,74,79,119,122,125,126,127,131,139,190,207,208,209,236,
204,203,179,174,173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,16,41,
  69,71,72,73,74,79,119,122,125,126,127,131,139,190,207,208,209,236,
233,232,231,0,65,101,102,103,
230,201,0,63,67,100,
229,228,0,61,98,99,
227,226,225,224,0,59,94,95,96,97,
223,222,0,57,92,93,
200,0,55,
199,0,53,
192,0,37,
178,0,18,
194,192,0,37,45,
235,234,230,204,203,201,193,191,189,188,187,186,185,184,183,182,181,180,179,
  177,174,173,172,170,167,142,138,137,135,134,123,120,116,112,107,9,0,1,
  175,176,
179,0,16,
192,178,0,18,37,
180,0,21,
192,178,0,18,37,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,170,167,142,138,137,135,134,123,120,116,112,107,0,1,175,176,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,11,12,13,15,16,17,
  19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,36,38,39,40,41,43,46,48,50,
  52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,
  126,127,131,139,190,207,208,209,236,
172,135,134,0,130,
172,135,134,0,130,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,
  73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,170,167,142,138,137,135,134,123,120,
  116,112,107,0,1,175,176,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,72,73,
  74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,179,174,173,172,167,142,138,137,135,134,123,120,0,2,
  3,4,5,6,16,17,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,69,71,
  72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,209,236,
235,234,230,204,203,201,180,179,174,173,172,167,142,138,137,135,134,123,120,
  0,2,3,4,5,6,16,17,20,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,
  69,71,72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,
  209,236,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,11,12,13,15,16,17,
  19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,36,38,39,40,41,43,46,48,50,
  52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,
  126,127,131,139,190,207,208,209,236,
192,178,0,18,37,
180,0,21,
180,0,21,
235,234,230,204,203,201,179,178,174,173,172,167,142,138,137,135,134,123,120,
  0,2,3,4,5,6,16,17,20,39,40,41,43,46,48,50,52,54,56,58,60,62,64,66,67,68,
  69,71,72,73,74,79,100,104,105,119,122,125,126,127,131,139,190,207,208,
  209,236,
178,0,18,
235,234,230,204,203,201,193,191,189,187,186,185,184,183,182,181,180,179,174,
  173,172,167,142,138,137,135,134,123,120,0,2,3,4,5,6,11,12,13,15,16,17,
  19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,36,38,39,40,41,43,46,48,50,
  52,54,56,58,60,62,64,66,67,68,69,71,72,73,74,79,100,104,105,119,122,125,
  126,127,131,139,190,207,208,209,236,

};


static unsigned const char ag_astt[4932] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,8,8,8,8,8,8,8,1,1,1,8,7,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,
  1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,7,0,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  5,3,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,2,7,2,2,1,
  1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,
  10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,4,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,2,1,2,2,2,7,2,2,1,1,1,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,1,1,2,2,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,
  2,1,1,1,1,1,1,1,1,5,1,2,2,2,1,1,5,1,2,2,1,1,5,1,2,2,1,1,1,1,5,1,2,2,2,2,1,
  1,5,1,2,2,1,5,1,1,5,1,1,1,5,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,5,
  1,1,4,1,1,5,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,
  7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,
  1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,1,1,7,2,1,1,1,
  7,2,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,2,1,7,
  2,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,1,2,1,1,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,7,1,1,7,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,
  2,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,
  2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,
  5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,5,1,
  1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,4,2,2,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,1,
  1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,
  2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,
  1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,1,2,7,2,2,1,1,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,
  1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,
  1,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,5,1,1,7,2,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,1,5,1,2,2,2,7,1,2,2,2,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,1,4,1,2,2,2,1,1,4,1,2,2,1,1,4,1,2,2,1,1,1,1,4,1,2,2,2,2,1,1,4,
  1,2,2,1,4,1,1,4,1,1,5,1,1,7,2,1,1,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,1,1,7,2,1,1,7,1,1,1,7,
  1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,4,2,10,10,10,4,2,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,4,
  1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,1,1,7,1,1,1,7,1,1,7,2,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,
  7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,4,4,4,4,4,4,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
343,343,1,2,345,343,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,
  120,120,120,120,120,120,120,120,120,120,120,120,120,120,122,
123,6,
115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,
  115,115,115,115,115,115,115,115,115,115,115,115,115,115,115,117,
118,8,
27,28,29,23,24,33,49,50,51,63,61,53,56,57,58,62,60,26,124,124,144,13,10,154,
  154,150,144,12,124,1,9,81,82,47,25,84,3,3,3,74,73,32,69,72,16,13,70,24,
  13,68,67,66,65,64,23,25,71,59,55,54,69,69,52,48,46,44,43,42,41,40,39,38,
  37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,
  22,21,20,14,
161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,
  161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,
  161,161,161,161,161,161,161,161,10,75,
149,149,149,133,149,
76,76,76,12,131,76,
82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,
  82,78,77,178,177,176,175,174,173,172,171,170,169,168,82,82,82,82,82,13,
  82,82,82,81,80,79,82,82,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,3,344,3,1,2,14,3,3,405,
83,84,15,
85,126,
157,156,156,156,157,143,155,
142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,151,142,151,148,134,134,142,142,142,18,
141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
  141,141,141,141,141,145,141,145,145,132,132,141,141,141,19,145,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,3,3,1,2,20,3,3,378,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,3,3,1,2,21,3,3,377,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,3,3,1,2,22,3,3,376,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,344,344,3,344,3,1,2,23,3,3,373,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,344,344,3,344,3,1,2,24,3,3,372,
10,83,159,14,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,
  344,344,344,344,3,1,2,26,3,3,348,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,27,3,3,404,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,28,3,3,403,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,29,3,3,399,
124,124,124,30,75,69,45,
124,124,124,31,75,68,45,
27,28,29,86,88,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,32,81,
  82,47,25,84,32,90,90,90,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  34,31,30,34,89,87,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,33,3,3,370,
91,63,92,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,35,81,82,47,
  25,84,32,93,62,62,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,
  154,45,22,21,20,14,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,36,81,82,47,
  25,84,32,93,61,61,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,
  154,45,22,21,20,14,
94,95,96,56,97,107,108,109,
29,33,54,98,105,106,
99,100,52,101,103,104,
102,103,104,105,50,106,99,100,101,102,
107,108,48,109,97,98,
110,46,111,
112,44,113,
114,116,42,117,115,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,125,125,125,3,125,125,125,3,1,2,45,3,3,359,
118,40,119,
26,75,120,
121,38,122,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,49,3,3,362,
344,344,3,344,3,1,2,50,3,3,360,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,3,344,344,344,344,344,344,344,344,3,1,2,51,3,3,358,
123,124,125,126,127,128,129,130,131,132,133,134,23,24,67,135,70,71,85,86,87,
  88,89,90,91,92,93,94,95,96,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,3,344,344,344,344,344,344,344,344,3,1,2,53,3,3,354,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,54,81,82,47,
  25,84,32,30,30,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,
  34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
124,124,124,55,28,45,
344,344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,
  344,3,1,2,56,3,3,353,
344,3,3,1,2,57,3,3,352,
344,3,3,1,2,58,3,3,351,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,59,136,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,1,2,344,60,3,3,
  349,
344,3,3,1,2,61,3,3,355,
344,3,3,1,2,62,3,3,350,
344,3,3,1,2,63,3,3,356,
137,60,64,22,138,
137,60,65,21,139,
27,28,29,23,24,33,32,26,124,124,144,13,10,154,154,150,144,12,124,66,81,82,
  47,25,84,32,140,141,140,140,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
60,67,19,
60,68,18,
137,60,69,15,142,
27,28,29,23,24,33,49,50,51,63,61,53,56,57,58,62,60,26,124,124,144,13,10,154,
  154,150,144,12,124,70,81,82,47,25,84,143,143,143,74,73,32,69,72,16,13,
  70,24,13,68,67,66,65,64,23,25,71,59,55,54,69,69,52,48,46,44,43,42,41,40,
  39,38,37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,
  154,45,22,21,20,14,
26,71,144,
26,72,145,
26,73,146,
27,28,29,23,24,33,49,50,51,63,61,53,56,57,58,62,60,26,124,124,144,13,10,154,
  154,150,144,12,124,74,81,82,47,25,84,6,6,147,74,73,32,69,72,16,13,70,24,
  13,68,67,66,65,64,23,25,71,59,55,54,69,69,52,48,46,44,43,42,41,40,39,38,
  37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,
  22,21,20,14,
162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,
  162,162,162,162,162,162,162,162,78,77,178,177,176,175,174,173,172,171,
  170,169,168,160,162,162,162,162,75,162,162,162,162,81,80,79,162,
76,76,76,139,140,76,
182,182,77,
188,187,187,187,188,78,185,
184,184,180,
183,183,179,
188,187,187,187,188,167,186,
189,82,
76,76,76,130,129,76,
148,149,149,149,149,84,128,149,
148,149,149,149,149,85,127,149,
344,3,3,1,2,86,3,3,375,
150,87,151,
344,3,3,1,2,88,3,3,374,
150,89,152,
137,150,90,65,142,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,91,3,3,371,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,92,81,82,47,
  25,84,32,93,64,64,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,
  154,45,22,21,20,14,
23,24,67,70,71,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,94,3,3,402,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,95,3,3,401,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,96,3,3,400,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,97,81,82,47,
  25,84,32,93,59,59,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,
  154,45,22,21,20,14,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,98,81,82,47,
  25,84,32,93,153,153,153,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,
  18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,99,3,3,398,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,100,3,3,397,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,101,81,82,47,
  25,84,32,93,154,37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,
  11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,102,3,3,396,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,103,3,3,395,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,104,3,3,394,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,105,3,3,393,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,106,81,82,47,
  25,84,32,93,155,38,37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,
  19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,107,3,3,392,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,108,3,3,391,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,109,81,82,47,
  25,84,32,93,156,39,38,37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,
  15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,110,3,3,369,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,111,81,82,47,
  25,84,32,93,157,40,39,38,37,37,37,36,35,34,34,31,30,34,25,110,111,112,
  16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,112,3,3,368,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,113,81,82,47,
  25,84,32,93,158,41,40,39,38,37,37,37,36,35,34,34,31,30,34,25,110,111,
  112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,114,3,3,367,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,115,81,82,47,
  25,84,32,93,159,42,41,40,39,38,37,37,37,36,35,34,34,31,30,34,25,110,111,
  112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,116,3,3,366,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,117,81,82,47,
  25,84,32,93,43,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,34,25,
  110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,118,3,3,365,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,119,81,82,47,
  25,84,32,93,41,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,34,25,
  110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,26,77,124,124,144,13,10,154,154,150,144,12,124,120,81,82,
  47,25,84,32,79,79,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,
  30,34,161,160,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,121,3,3,364,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,122,81,82,47,
  25,84,32,162,162,162,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,
  31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,123,3,3,390,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,124,3,3,389,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,125,3,3,388,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,126,3,3,387,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,127,3,3,386,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,128,3,3,385,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,129,3,3,384,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,130,3,3,383,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,131,3,3,382,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,132,3,3,381,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,133,3,3,380,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,134,3,3,379,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,135,81,82,47,
  25,84,32,37,37,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,
  34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,49,50,51,163,63,61,53,56,57,58,62,60,26,124,124,144,13,10,
  154,154,150,144,12,124,136,81,82,47,25,84,3,3,3,74,73,32,69,72,16,13,70,
  24,13,68,67,66,65,64,23,25,71,59,27,55,54,69,69,52,48,46,44,43,42,41,40,
  39,38,37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,
  154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,137,3,3,361,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,138,81,82,47,
  25,84,32,31,31,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,
  34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
124,124,124,139,29,45,
137,33,142,
60,141,20,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,142,81,82,47,
  25,84,32,35,35,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,
  34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
62,143,164,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,144,81,82,47,
  25,84,32,165,165,165,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,
  31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,32,26,124,124,144,13,10,154,154,150,144,12,124,145,81,82,
  47,25,84,32,140,166,140,140,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,146,81,82,47,
  25,84,32,167,167,167,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,
  31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
168,5,169,
146,146,146,148,170,
146,146,146,149,171,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,1,2,150,3,3,347,
23,24,26,124,124,144,13,10,154,154,150,144,12,124,151,81,82,47,25,84,32,93,
  74,74,31,30,74,25,16,15,19,11,18,17,154,45,22,21,20,14,
23,24,26,124,124,144,13,10,154,154,150,144,12,124,152,81,82,47,25,84,32,93,
  73,73,31,30,73,25,16,15,19,11,18,17,154,45,22,21,20,14,
94,95,96,57,97,107,108,109,
29,33,55,98,105,106,
99,100,53,101,103,104,
102,103,104,105,51,106,99,100,101,102,
107,108,49,109,97,98,
110,47,111,
112,45,113,
137,78,172,
150,161,76,
173,137,162,142,174,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,344,344,3,344,344,344,344,344,344,344,344,3,1,2,344,163,3,3,357,
26,164,175,
137,150,165,26,142,
60,166,176,
137,150,167,177,142,
344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,344,
  344,344,3,344,344,344,344,344,344,344,344,3,1,2,168,3,3,346,
27,28,29,23,24,33,49,50,51,63,61,53,56,57,58,62,60,26,124,124,144,13,10,154,
  154,150,144,12,124,169,81,82,47,25,84,7,10,74,73,32,69,72,16,13,70,24,
  13,68,67,66,65,64,23,25,71,59,55,54,69,69,52,48,46,44,43,42,41,40,39,38,
  37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,
  22,21,20,14,
147,147,147,138,147,
147,147,147,137,147,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,172,81,82,47,
  25,84,32,80,80,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,
  34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
344,344,344,344,344,344,344,344,344,344,3,344,344,344,344,344,344,344,344,3,
  1,2,173,3,3,363,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,174,81,82,47,
  25,84,32,39,93,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,31,30,34,
  25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,26,124,124,144,13,10,154,154,150,144,12,124,175,81,82,47,
  25,84,32,178,178,178,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,34,
  31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,32,26,124,124,144,13,10,154,154,150,144,12,124,176,81,82,
  47,25,84,32,140,179,140,140,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
27,28,29,23,24,33,49,50,51,63,61,53,56,57,58,62,60,26,124,124,144,13,10,154,
  154,150,144,12,124,177,81,82,47,25,84,8,11,74,73,32,69,72,16,13,70,24,
  13,68,67,66,65,64,23,25,71,59,55,54,69,69,52,48,46,44,43,42,41,40,39,38,
  37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,
  22,21,20,14,
137,150,178,180,142,
60,179,181,
60,180,14,
27,28,29,23,24,33,26,32,124,124,144,13,10,154,154,150,144,12,124,181,81,82,
  47,25,84,32,140,182,140,140,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,22,21,20,14,
150,182,183,
27,28,29,23,24,33,49,50,51,63,61,53,56,57,58,62,60,26,124,124,144,13,10,154,
  154,150,144,12,124,183,81,82,47,25,84,9,12,74,73,32,69,72,16,13,70,24,
  13,68,67,66,65,64,23,25,71,59,55,54,69,69,52,48,46,44,43,42,41,40,39,38,
  37,37,37,36,35,34,34,31,30,34,25,110,111,112,16,15,19,11,18,17,154,45,
  22,21,20,14,

};


static const unsigned short ag_sbt[] = {
     0,  38,  75, 113, 119, 152, 186, 188, 223, 225, 323, 370, 375, 381,
   434, 467, 470, 472, 479, 513, 548, 580, 612, 644, 679, 714, 718, 748,
   774, 800, 826, 833, 840, 908, 934, 937, 988,1039,1047,1053,1059,1069,
  1075,1078,1081,1086,1139,1142,1145,1148,1174,1185,1223,1253,1290,1353,
  1359,1386,1395,1404,1436,1477,1486,1495,1504,1509,1514,1580,1583,1586,
  1591,1688,1691,1694,1697,1794,1848,1854,1857,1864,1867,1870,1877,1879,
  1885,1893,1901,1910,1913,1922,1925,1930,1956,2007,2012,2038,2064,2090,
  2141,2193,2219,2245,2298,2324,2350,2376,2402,2456,2482,2508,2563,2589,
  2645,2671,2728,2754,2812,2838,2898,2924,2985,3051,3077,3141,3167,3193,
  3219,3245,3271,3297,3323,3349,3375,3401,3427,3453,3516,3615,3641,3704,
  3710,3713,3716,3779,3782,3846,3912,3976,3979,3984,3989,4047,4087,4127,
  4135,4141,4147,4157,4163,4166,4169,4172,4175,4180,4220,4223,4228,4231,
  4236,4273,4369,4374,4379,4442,4468,4530,4594,4660,4756,4761,4764,4767,
  4833,4836,4932
};


static const unsigned short ag_sbe[] = {
    34,  72, 110, 117, 149, 185, 187, 222, 224, 255, 368, 373, 378, 425,
   463, 469, 471, 477, 512, 546, 576, 608, 640, 675, 710, 715, 744, 770,
   796, 822, 829, 836, 860, 930, 935, 955,1006,1042,1049,1055,1063,1071,
  1076,1079,1083,1135,1140,1143,1146,1170,1181,1219,1237,1286,1308,1356,
  1382,1391,1400,1434,1473,1482,1491,1500,1506,1511,1533,1581,1584,1588,
  1620,1689,1692,1695,1726,1839,1851,1856,1862,1866,1869,1875,1878,1882,
  1890,1898,1906,1911,1918,1923,1927,1952,1974,2009,2034,2060,2086,2108,
  2159,2215,2241,2263,2320,2346,2372,2398,2420,2478,2504,2526,2585,2607,
  2667,2689,2750,2772,2834,2856,2920,2942,3004,3073,3095,3163,3189,3215,
  3241,3267,3293,3319,3345,3371,3397,3423,3449,3471,3546,3637,3659,3707,
  3711,3714,3734,3780,3800,3865,3930,3977,3982,3987,4043,4061,4101,4130,
  4137,4143,4151,4159,4164,4167,4170,4173,4177,4216,4221,4225,4229,4233,
  4269,4302,4372,4377,4397,4464,4486,4548,4613,4689,4758,4762,4765,4786,
  4834,4865,4932
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,5,9,4,5,9,1,7,2,1,1,2,2,3,2,2,1,1,1,4,3,2,3,2,3,0,1,1,
  3,1,3,1,5,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,2,2,1,3,3,1,1,2,2,
  2,2,1,4,4,1,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,
  2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 22, 22, 22, 22,
   22, 22, 22, 22, 22, 23, 15, 19, 13, 25, 29, 29, 30, 30, 20, 20, 17, 17,
   39, 39, 40, 40, 43, 43, 46, 46, 48, 48, 50, 50, 52, 52, 54, 54, 56, 56,
   58, 58, 60, 60, 62, 62, 64, 64, 64, 66, 66, 69, 69, 69, 69, 69, 69, 69,
   69, 69, 69, 41, 74, 77, 77, 78, 78, 71, 71, 71, 71, 42, 42, 42, 42, 42,
   42, 42, 42, 42, 42, 42, 42, 57, 57, 59, 59, 59, 59, 61, 61, 63, 63, 65,
   65, 65, 68, 68, 68,  1,109,109,110,110,  1,114,114,115,115,  1,190,190,
  208,208,208,119,119,119,122,122,122,128,128,121,121,124,124,207,207,207,
  125,125,129,129,126,126,127,127,139,139,131,131,140,140, 79, 79,236,143,
  143,144,144,146,146,146,147,147,147,147,147,147,147,147,147,147,147,148,
  148,148,161,162,163,149,149,166,166,209,168,168,130,106,106,108,108,108,
  108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,
  108,108,108,108,108,108,108,108,108,108,108,108,108,113,113,113,113,113,
  113,113,113,113,113,113,113,113,113,113,113,113,113,113,113,113,113,113,
  113,113,113,113,113,113,113,113,113,113,117,117,117,118,118,118,118,118,
  118,132,132,133,133,133,136,136,141,141,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,169,169,169,169,169,169,169,169,169,169,169,169,169,
  169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
  175,175,176,176, 14, 18, 16, 21, 24, 26, 27, 28, 31, 32, 33, 35, 34,  4,
   36, 37, 38, 45, 44, 47, 49, 51, 53, 55, 67, 70, 72, 73, 75, 76,  2,  3,
    6, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96,
   97, 98, 99,100,101,102,103,104,105,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *)); 
            VWD(0, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(AstNode * *)))); break;
    case 4: V(0,(AstNode * *)) = ag_rp_4(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *))); break;
    case 5: V(0,(AstNode * *)) = ag_rp_5(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 6: V(0,(AstNode * *)) = ag_rp_6(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *))); break;
    case 7: V(0,(AstNode * *)) = ag_rp_7(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 8: V(0,(AstNode * *)) = ag_rp_8(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 9: V(0,(AstNode * *)) = ag_rp_9(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *))); break;
    case 10: V(0,(AstNode * *)) = ag_rp_10(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 11: V(0,(AstNode * *)) = ag_rp_11(PCB_POINTER, V(0,(AstNode * *))); break;
    case 12: V(0,(AstNode * *)) = ag_rp_12(PCB_POINTER, V(1,(AstNode * *)), V(4,(AstNode * *))); break;
    case 13: V(0,(AstNode * *)) = ag_rp_13(PCB_POINTER, V(0,(AstNode * *))); break;
    case 14: V(0,(AstNode * *)) = ag_rp_14(PCB_POINTER); break;
    case 15: V(0,(AstNode * *)) = ag_rp_15(PCB_POINTER); break;
    case 16: V(0,(AstNode * *)) = ag_rp_16(PCB_POINTER); break;
    case 17: V(0,(AstNode * *)) = ag_rp_17(PCB_POINTER, V(1,(AstNode * *))); break;
    case 18: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 19: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 20: ag_rp_20(PCB_POINTER); break;
    case 21: ag_rp_21(PCB_POINTER); break;
    case 22: ag_rp_22(PCB_POINTER); break;
    case 23: V(0,(AstNode * *)) = ag_rp_23(PCB_POINTER, V(2,(AstNode * *))); break;
    case 24: V(0,(AstNode * *)) = ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 25: VNO AG_WRAP_6(ag_rp_25(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 26: VRO(AG_WRAP_6 *, ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 27: VNO AG_WRAP_6(ag_rp_27(PCB_POINTER, V(1,(AstNode * *)))); break;
    case 28: VRO(AG_WRAP_6 *, ag_rp_28(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 29: V(0,(AstNode * *)) = ag_rp_29(PCB_POINTER); break;
    case 30: V(0,(AstNode * *)) = ag_rp_30(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 31: V(0,(AstNode * *)) = ag_rp_31(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 32: V(0,(AstNode * *)) = ag_rp_32(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *)), V(4,(AstNode * *))); break;
    case 33: V(0,(AstNode * *)) = ag_rp_33(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 34: V(0,(AstNode * *)) = ag_rp_34(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 35: V(0,(AstNode * *)) = ag_rp_35(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 36: V(0,(AstNode * *)) = ag_rp_36(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 37: V(0,(AstNode * *)) = ag_rp_37(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 38: V(0,(AstNode * *)) = ag_rp_38(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 39: V(0,(AstNode * *)) = ag_rp_39(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 40: V(0,(AstNode * *)) = ag_rp_40(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 41: V(0,(AstNode * *)) = ag_rp_41(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 42: V(0,(AstNode * *)) = ag_rp_42(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 43: V(0,(AstNode * *)) = ag_rp_43(PCB_POINTER, V(1,(AstNode * *))); break;
    case 44: V(0,(AstNode * *)) = ag_rp_44(PCB_POINTER, V(0,(Opcode *)), V(1,(AstNode * *))); break;
    case 45: V(0,(AstNode * *)) = ag_rp_45(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 46: V(0,(AstNode * *)) = ag_rp_46(PCB_POINTER, V(1,(AstNode * *))); break;
    case 47: V(0,(AstNode * *)) = ag_rp_47(PCB_POINTER, V(0,(AstNode * *))); break;
    case 48: V(0,(AstNode * *)) = ag_rp_48(PCB_POINTER, V(1,(AstNode * *))); break;
    case 49: V(0,(AstNode * *)) = ag_rp_49(PCB_POINTER, V(1,(AstNode * *))); break;
    case 50: V(0,(AstNode * *)) = ag_rp_50(PCB_POINTER, V(0,(AstNode * *))); break;
    case 51: V(0,(AstNode * *)) = ag_rp_51(PCB_POINTER, V(0,(AstNode * *))); break;
    case 52: V(0,(AstNode * *)) = ag_rp_52(PCB_POINTER, V(3,(AstNode * *))); break;
    case 53: V(0,(AstNode * *)) = ag_rp_53(PCB_POINTER, V(3,(AstNode * *))); break;
    case 54: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 55: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_55(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 56: VNO AG_WRAP_6(ag_rp_56(PCB_POINTER)); break;
    case 57: VNO AG_WRAP_6(ag_rp_57(PCB_POINTER, V(0,(AstNode * *)))); break;
    case 58: VRO(AG_WRAP_6 *, ag_rp_58(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 59: V(0,(AstNode * *)) = ag_rp_59(PCB_POINTER, V(0,(long *))); break;
    case 60: V(0,(AstNode * *)) = ag_rp_60(PCB_POINTER, V(0,(double *))); break;
    case 61: V(0,(AstNode * *)) = VDO(AG_WRAP_5 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 62: V(0,(AstNode * *)) = ag_rp_62(PCB_POINTER, V(0,(int *))); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: V(0,(Opcode *)) = ag_rp_65(PCB_POINTER); break;
    case 66: V(0,(Opcode *)) = ag_rp_66(PCB_POINTER); break;
    case 67: V(0,(Opcode *)) = ag_rp_67(PCB_POINTER); break;
    case 68: V(0,(Opcode *)) = ag_rp_68(PCB_POINTER); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: V(0,(Opcode *)) = ag_rp_71(PCB_POINTER); break;
    case 72: V(0,(Opcode *)) = ag_rp_72(PCB_POINTER); break;
    case 73: V(0,(Opcode *)) = ag_rp_73(PCB_POINTER); break;
    case 74: V(0,(Opcode *)) = ag_rp_74(PCB_POINTER); break;
    case 75: V(0,(Opcode *)) = ag_rp_75(PCB_POINTER); break;
    case 76: V(0,(Opcode *)) = ag_rp_76(PCB_POINTER); break;
    case 77: V(0,(Opcode *)) = ag_rp_77(PCB_POINTER); break;
    case 78: V(0,(Opcode *)) = ag_rp_78(PCB_POINTER); break;
    case 79: V(0,(Opcode *)) = ag_rp_79(PCB_POINTER); break;
    case 80: V(0,(Opcode *)) = ag_rp_80(PCB_POINTER); break;
    case 81: V(0,(Opcode *)) = ag_rp_81(PCB_POINTER); break;
    case 82: V(0,(Opcode *)) = ag_rp_82(PCB_POINTER); break;
    case 83: V(0,(Opcode *)) = ag_rp_83(PCB_POINTER); break;
    case 84: V(0,(Opcode *)) = ag_rp_84(PCB_POINTER); break;
    case 85: V(0,(Opcode *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(Opcode *)) = ag_rp_86(PCB_POINTER); break;
    case 87: V(0,(Opcode *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(Opcode *)) = ag_rp_88(PCB_POINTER); break;
    case 89: V(0,(Opcode *)) = ag_rp_89(PCB_POINTER); break;
    case 90: V(0,(Opcode *)) = ag_rp_90(PCB_POINTER); break;
    case 91: VNO AG_WRAP_4(ag_rp_91(PCB_POINTER, V(0,(int *)))); break;
    case 92: VRO(AG_WRAP_4 *, ag_rp_92(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 93: V(0,(double *)) = ag_rp_93(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 94: V(0,(double *)) = ag_rp_94(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 95: V(0,(double *)) = ag_rp_95(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 96: V(0,(double *)) = ag_rp_96(PCB_POINTER, V(0,(double *))); break;
    case 97: V(0,(double *)) = ag_rp_97(PCB_POINTER, V(1,(double *))); break;
    case 98: V(0,(double *)) = ag_rp_98(PCB_POINTER, V(0,(long *))); break;
    case 99: V(0,(double *)) = ag_rp_99(PCB_POINTER, V(0,(long *))); break;
    case 100: V(0,(double *)) = ag_rp_100(PCB_POINTER, V(0,(long *))); break;
    case 101: V(0,(long *)) = ag_rp_101(PCB_POINTER, V(1,(long *))); break;
    case 102: V(0,(long *)) = ag_rp_102(PCB_POINTER, V(1,(long *))); break;
    case 103: V(0,(double *)) = ag_rp_103(PCB_POINTER, V(0,(int *))); break;
    case 104: V(0,(double *)) = ag_rp_104(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 105: V(0,(long *)) = ag_rp_105(PCB_POINTER, V(0,(int *))); break;
    case 106: V(0,(long *)) = ag_rp_106(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 107: V(0,(long *)) = ag_rp_107(PCB_POINTER, V(0,(int *))); break;
    case 108: V(0,(long *)) = ag_rp_108(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 109: V(0,(long *)) = ag_rp_109(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 110: V(0,(long *)) = ag_rp_110(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 111: V(0,(long *)) = ag_rp_111(PCB_POINTER); break;
    case 112: V(0,(long *)) = ag_rp_112(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 113: V(0,(long *)) = ag_rp_113(PCB_POINTER); break;
    case 114: V(0,(long *)) = ag_rp_114(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 115: V(0,(int *)) = ag_rp_115(PCB_POINTER, V(0,(int *))); break;
    case 116: V(0,(int *)) = ag_rp_116(PCB_POINTER, V(0,(int *))); break;
    case 117: VRO(AG_WRAP_5 *, ag_rp_117(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 118: VNO AG_WRAP_5(ag_rp_118(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 119: VNO AG_WRAP_4(ag_rp_119(PCB_POINTER)); break;
    case 120: VRO(AG_WRAP_4 *, ag_rp_120(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 121: V(0,(int *)) = ag_rp_121(PCB_POINTER); break;
    case 122: V(0,(int *)) = ag_rp_122(PCB_POINTER); break;
    case 123: V(0,(int *)) = ag_rp_123(PCB_POINTER); break;
    case 124: V(0,(int *)) = ag_rp_124(PCB_POINTER); break;
    case 125: V(0,(int *)) = ag_rp_125(PCB_POINTER); break;
    case 126: V(0,(int *)) = ag_rp_126(PCB_POINTER); break;
    case 127: V(0,(int *)) = ag_rp_127(PCB_POINTER); break;
    case 128: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 129: V(0,(int *)) = ag_rp_129(PCB_POINTER); break;
    case 130: V(0,(int *)) = ag_rp_130(PCB_POINTER); break;
    case 131: V(0,(int *)) = ag_rp_131(PCB_POINTER); break;
    case 132: V(0,(int *)) = ag_rp_132(PCB_POINTER, V(1,(int *))); break;
    case 133: V(0,(int *)) = ag_rp_133(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 134: V(0,(int *)) = ag_rp_134(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 135: V(0,(int *)) = ag_rp_135(PCB_POINTER, V(1,(int *))); break;
    case 136: V(0,(int *)) = ag_rp_136(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 137: V(0,(int *)) = ag_rp_137(PCB_POINTER, V(0,(int *))); break;
    case 138: V(0,(int *)) = ag_rp_138(PCB_POINTER, V(0,(int *))); break;
    case 139: V(0,(int *)) = ag_rp_139(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES parse_token_names
const char *const parse_token_names[237] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"else\"",
  "WHILE",
  "'('",
  "expression",
  "')'",
  "FOR",
  "optional expression",
  "';'",
  "simple statement",
  "DO",
  "\"while\"",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "conditional expression",
  "lvalue",
  "assignment op",
  "logical or expression",
  "'\\?'",
  "':'",
  "logical and expression",
  "\"||\"",
  "inclusive or expression",
  "\"&&\"",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "equality op",
  "shift expression",
  "relational op",
  "additive expression",
  "shift op",
  "multiplicative expression",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"else\"",
  "')'",
  "'('",
  "';'",
  "\"while\"",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'}'",
  "'{'",
  "name",
  "\"dump\"",
  "','",
  "\"print\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "'+'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_parse(parse_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void parse(parse_pcb_type *PCB_POINTER) {
  init_parse(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


