/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In cll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146130
#include "dxi.h"
#endif

#ifndef DXI_H_1031146130
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 587, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    loopActive(0),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

void dxi_pcb_struct::forLoop
    (const FileLocation &condLoc, const FileLocation &incLoc)
{
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    if (loopPCB.mode == executeMode) {
      loopPCB.parseSyntax("forField", condLoc);    // Evaluate condition field
      Value result = dxi_value(&loopPCB);
      if (result.isDefined() && result.isFalse()) loopPCB.mode = breakMode;
    }
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // Mode could have been set to continueMode
    loopPCB.parseSyntax("forField", incLoc);
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

void dxi_pcb_struct::interruptLoop(Mode desiredMode) {
  if (mode != executeMode) return;
  if (loopActive == 0) reportError("No loop active");
  mode = desiredMode;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

Value dxi_pcb_struct::powm(Value &lvalue, const Value &exp) const {
  if (mode != executeMode) return Value();
  Value base = lvalue;
  lvalue = pow(base.deref(), exp);
  return lvalue.deref();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  return pcb.returnValue;
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 184, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return Value();
}

static inline Value ag_rp_5(PCB_DECL, Value &x) {
/* Line 188, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x;
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 192, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 195, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 198, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 201, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 204, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 227, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 229, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 233, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_14(PCB_DECL) {
/* Line 247, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.simpleLoop("doLoop");
}

static inline void ag_rp_15(PCB_DECL) {
/* Line 248, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_16(PCB_DECL, FileLocation c, FileLocation i) {
/* Line 249, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.forLoop(c, i);
}

static inline void ag_rp_17(PCB_DECL) {
/* Line 253, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.interruptLoop(breakMode);
}

static inline void ag_rp_18(PCB_DECL) {
/* Line 254, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.interruptLoop(continueMode);
}

static inline void ag_rp_19(PCB_DECL, Value &x) {
/* Line 255, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_20(PCB_DECL) {
/* Line 269, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static void ag_rp_21(PCB_DECL, Value &x) {
/* Line 274, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static inline void ag_rp_22(PCB_DECL) {
/* Line 277, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.mode = scanMode;
}

static inline FileLocation ag_rp_23(PCB_DECL) {
/* Line 280, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return CONTEXT;
}

static void ag_rp_24(PCB_DECL, Value &x) {
/* Line 285, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_25(PCB_DECL) {
/* Line 289, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_26(PCB_DECL, AgString &n) {
/* Line 296, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_27(PCB_DECL, AgString &n) {
/* Line 297, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_28(PCB_DECL, Value &x) {
/* Line 301, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_29(PCB_DECL, Value &x) {
/* Line 302, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  PCB.print(x);
}

static inline Value ag_rp_30(PCB_DECL) {
/* Line 305, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return Value();
}

static inline Value ag_rp_31(PCB_DECL, Value &x) {
/* Line 311, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x;
}

static inline Value ag_rp_32(PCB_DECL, Value &lv, Value &x) {
/* Line 315, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv = x);
}

static inline Value ag_rp_33(PCB_DECL, Value &lv, Value &x) {
/* Line 316, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv += x);
}

static inline Value ag_rp_34(PCB_DECL, Value &lv, Value &x) {
/* Line 317, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv -= x);
}

static inline Value ag_rp_35(PCB_DECL, Value &lv, Value &x) {
/* Line 318, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv *= x);
}

static inline Value ag_rp_36(PCB_DECL, Value &lv, Value &x) {
/* Line 319, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv /= x);
}

static inline Value ag_rp_37(PCB_DECL, Value &lv, Value &x) {
/* Line 320, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv %= x);
}

static inline Value ag_rp_38(PCB_DECL, Value &lv, Value &x) {
/* Line 321, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv |= x);
}

static inline Value ag_rp_39(PCB_DECL, Value &lv, Value &x) {
/* Line 322, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv ^= x);
}

static inline Value ag_rp_40(PCB_DECL, Value &lv, Value &x) {
/* Line 323, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv &= x);
}

static inline Value ag_rp_41(PCB_DECL, Value &lv, Value &x) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv <<= x);
}

static inline Value ag_rp_42(PCB_DECL, Value &lv, Value &x) {
/* Line 325, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv >>= x);
}

static inline Value ag_rp_43(PCB_DECL, Value &lv, Value &x) {
/* Line 326, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return PCB.powm(lv,x);
}

static void ag_rp_44(PCB_DECL, Value &c) {
/* Line 332, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
if (PCB.mode == executeMode && c.isFalse())  PCB.mode = elseMode;
}

static void ag_rp_45(PCB_DECL, Value &c, Value &x) {
/* Line 334, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
if (!PCB.changeMode(elseMode, executeMode))
         PCB.changeMode(executeMode, blockMode);
}

static inline Value ag_rp_46(PCB_DECL, Value &c, Value &x, Value &y) {
/* Line 337, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return PCB.mode = CONTEXT.mode, VALUE(c.isTrue() ? x : y);
}

static void ag_rp_47(PCB_DECL, Value &x) {
/* Line 343, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
if (PCB.mode == executeMode && x.isTrue()) PCB.mode = blockMode;
}

static inline Value ag_rp_48(PCB_DECL, Value &x, Value &y) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? x : y);
}

static void ag_rp_49(PCB_DECL, Value &x) {
/* Line 349, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = blockMode;
}

static inline Value ag_rp_50(PCB_DECL, Value &x, Value &y) {
/* Line 350, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? y : x);
}

static inline Value ag_rp_51(PCB_DECL, Value &x, Value &y) {
/* Line 355, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x |= y);
}

static inline Value ag_rp_52(PCB_DECL, Value &x, Value &y) {
/* Line 359, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x ^= y);
}

static inline Value ag_rp_53(PCB_DECL, Value &x, Value &y) {
/* Line 363, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x &= y);
}

static inline Value ag_rp_54(PCB_DECL, Value &x, Value &y) {
/* Line 367, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_55(PCB_DECL, Value &x, Value &y) {
/* Line 368, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_56(PCB_DECL, Value &x, Value &y) {
/* Line 372, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_57(PCB_DECL, Value &x, Value &y) {
/* Line 373, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_58(PCB_DECL, Value &x, Value &y) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_59(PCB_DECL, Value &x, Value &y) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_60(PCB_DECL, Value &x, Value &y) {
/* Line 379, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x <<= y);
}

static inline Value ag_rp_61(PCB_DECL, Value &x, Value &y) {
/* Line 380, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x >>= y);
}

static inline Value ag_rp_62(PCB_DECL, Value &x, Value &y) {
/* Line 384, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x += y);
}

static inline Value ag_rp_63(PCB_DECL, Value &x, Value &y) {
/* Line 385, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x -= y);
}

static inline Value ag_rp_64(PCB_DECL, Value &x, Value &y) {
/* Line 389, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x *= y);
}

static inline Value ag_rp_65(PCB_DECL, Value &x, Value &y) {
/* Line 390, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x /= y);
}

static inline Value ag_rp_66(PCB_DECL, Value &x, Value &y) {
/* Line 391, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x %= y);
}

static inline Value ag_rp_67(PCB_DECL, Value &x) {
/* Line 395, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_68(PCB_DECL, Value &x) {
/* Line 396, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_69(PCB_DECL, Value &x) {
/* Line 397, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_70(PCB_DECL, Value &x) {
/* Line 398, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(~x);
}

static inline Value ag_rp_71(PCB_DECL, Value &x, Value &y) {
/* Line 406, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_72(PCB_DECL, Value &x) {
/* Line 414, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x;
}

static inline Value ag_rp_73(PCB_DECL, Value &lv) {
/* Line 416, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_74(PCB_DECL, Value &lv) {
/* Line 417, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(++lv);
}

static inline Value ag_rp_75(PCB_DECL, Value &lv) {
/* Line 418, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(--lv);
}

static inline Value ag_rp_76(PCB_DECL, Value &lv) {
/* Line 419, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv++);
}

static inline Value ag_rp_77(PCB_DECL, Value &lv) {
/* Line 420, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(lv--);
}

static inline Value ag_rp_78(PCB_DECL, Value &x) {
/* Line 422, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_79(PCB_DECL, Value &x) {
/* Line 423, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_80(PCB_DECL, AgString &n) {
/* Line 426, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_81(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 429, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(callFunction(n, args));
}

static inline AgStack<Value> ag_rp_82(PCB_DECL) {
/* Line 432, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_83(PCB_DECL, Value &x) {
/* Line 436, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_84(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 437, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_85(PCB_DECL, long x) {
/* Line 440, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_86(PCB_DECL, double x) {
/* Line 441, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_87(PCB_DECL, int x) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_88(PCB_DECL, int c) {
/* Line 461, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_89(PCB_DECL, AgString &ns, int c) {
/* Line 462, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return ns.append(c);
}

static inline double ag_rp_90(PCB_DECL, double x, long e) {
/* Line 467, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_91(PCB_DECL, double x, long e) {
/* Line 468, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_92(PCB_DECL, double i, double f) {
/* Line 471, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return i+f;
}

static inline double ag_rp_93(PCB_DECL, double i) {
/* Line 472, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return i;
}

static inline double ag_rp_94(PCB_DECL, double f) {
/* Line 473, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return f;
}

static inline double ag_rp_95(PCB_DECL, long x) {
/* Line 476, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x;
}

static inline double ag_rp_96(PCB_DECL, long x) {
/* Line 477, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x;
}

static inline double ag_rp_97(PCB_DECL, long x) {
/* Line 478, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_98(PCB_DECL, long x) {
/* Line 481, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return (long)x;
}

static inline long ag_rp_99(PCB_DECL, long x) {
/* Line 482, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_100(PCB_DECL, int d) {
/* Line 485, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_101(PCB_DECL, int d, double f) {
/* Line 486, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_102(PCB_DECL, int d) {
/* Line 494, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return d-'0';
}

static inline long ag_rp_103(PCB_DECL, long x, int d) {
/* Line 495, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_104(PCB_DECL, int d) {
/* Line 498, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return d-'0';
}

static inline long ag_rp_105(PCB_DECL, long x, int d) {
/* Line 499, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_106(PCB_DECL, long x, int d) {
/* Line 502, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_107(PCB_DECL, long x, int d) {
/* Line 503, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_108(PCB_DECL) {
/* Line 506, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 0;
}

static inline long ag_rp_109(PCB_DECL, long n, int d) {
/* Line 507, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_110(PCB_DECL) {
/* Line 510, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 0;
}

static inline long ag_rp_111(PCB_DECL, long n, int d) {
/* Line 511, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_112(PCB_DECL, int x) {
/* Line 514, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return x - '0';
}

static inline int ag_rp_113(PCB_DECL, int d) {
/* Line 515, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_114(PCB_DECL, Value &s, Value &e) {
/* Line 521, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_115(PCB_DECL, AgString &b) {
/* Line 524, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_116(PCB_DECL) {
/* Line 527, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_117(PCB_DECL, AgString &s, int c) {
/* Line 528, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_118(PCB_DECL) {
/* Line 540, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\'';
}

static inline int ag_rp_119(PCB_DECL) {
/* Line 541, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '"';
}

static inline int ag_rp_120(PCB_DECL) {
/* Line 542, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\?';
}

static inline int ag_rp_121(PCB_DECL) {
/* Line 543, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\\';
}

static inline int ag_rp_122(PCB_DECL) {
/* Line 544, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\a';
}

static inline int ag_rp_123(PCB_DECL) {
/* Line 545, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\b';
}

static inline int ag_rp_124(PCB_DECL) {
/* Line 546, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\f';
}

static inline int ag_rp_125(PCB_DECL) {
/* Line 547, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\n';
}

static inline int ag_rp_126(PCB_DECL) {
/* Line 548, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\r';
}

static inline int ag_rp_127(PCB_DECL) {
/* Line 549, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\t';
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 550, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return '\v';
}

static inline int ag_rp_129(PCB_DECL, int d) {
/* Line 556, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return d-'0';
}

static inline int ag_rp_130(PCB_DECL, int n, int d) {
/* Line 559, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_131(PCB_DECL, int n, int d) {
/* Line 562, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_132(PCB_DECL, int d) {
/* Line 565, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return d;
}

static inline int ag_rp_133(PCB_DECL, int n, int d) {
/* Line 566, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_134(PCB_DECL, int d) {
/* Line 569, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return d-'0';
}

static inline int ag_rp_135(PCB_DECL, int d) {
/* Line 570, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_136(PCB_DECL, int c) {
/* Line 579, W:/agdev/agex/packages/xidek/cll/base/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0,
  5, 5, 5, 5, 5, 5, 5, 5, 4, 5, 4, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 5, 0,
  0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 6, 6, 0, 0, 0, 5,
  0, 5, 0, 0, 0, 0, 5, 0, 5, 5, 0, 0, 0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  6,  7,  0,  0,  0,  0,  8,  9, 10,
    0, 11, 12, 13, 14,  0,  0, 15, 16, 17,  0,  0,  0,  0,  0, 18, 19, 20,
    0, 14, 21, 22, 23,  0, 24, 25, 26, 27, 28,  0,  0, 29,  0, 30, 31, 32,
   33, 34, 35, 36, 37, 38, 39, 40, 41,  0, 42, 43, 44,  0, 45, 46,  0, 47,
   48,  0, 49,  0, 50,  0, 51,  0, 52, 53,  0, 54, 55, 56, 57,  0, 58, 59,
    0, 60, 61,  0, 62, 63, 64,  0, 65, 66, 67, 68,  0, 69, 70,  0, 71, 72,
   73, 74, 75,  0, 76, 77, 78, 79, 80,  0, 81, 82, 83, 84,  0, 85,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0, 86, 87,  0, 88, 89, 90, 91, 92, 93,
   94, 95,  0,  0, 96, 97, 98, 99,  0,  0,  0,100,101,102,103,104,105,106,
  107,  0,  0,108,109,110,111,  0,112,113,114,115,  0,  0,  0,  0,  0,116,
  117,118,119,120,121,122,123,124,125,126,  0,  0,  0,127,128,129,130,131,
  132,133,134
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0, 20,  0, 21,  0, 22,  0, 23,  0, 24,  0,198,  0,199,  0,200,
  0,201,  0,202,  0,203,  0,205,  0,206,  0,210,  0,212,  0,250,
  0,251,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 99,116,255, 47,100,102,115,119,255, 42,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 88,120,255,
   61,255, 60, 61,255, 61,255, 61, 62,255, 76,117,255,111,117,255, 70,255,
  114,255,111,255, 99,116,255, 76,255,101,255,108,255,105,255,104,255, 61,
  124,255, 33, 37, 38, 42, 43, 45, 47, 48, 60, 61, 62, 94, 98, 99,100,101,
  102,105,108,112,114,115,119,124,255, 99,116,255,100,102,115,119,255, 42,
   47,255, 88,120,255, 43, 45, 47, 48,255, 42, 47,255, 88,120,255,111,117,
  255, 43, 45, 47, 48, 98, 99,100,102,105,112,114,119,255, 42, 47,255, 47,
  255, 88,120,255, 43, 45, 48,255, 88,120,255,111,117,255, 43, 45, 48, 98,
   99,100,102,105,112,114,119,255, 34, 39, 63, 92, 97, 98,102,110,114,116,
  118,120,255, 92,255, 42, 47,255, 60, 61,255, 61, 62,255, 33, 38, 42, 47,
   60, 61, 62,124,255, 60, 61,255, 61, 62,255, 33, 38, 42, 60, 61, 62,124,
  255, 42, 47,255, 88,120,255, 43, 45, 47, 48,100,108,255, 88,120,255, 43,
   45, 48,100,108,255, 60, 61,255, 61, 62,255, 33, 38, 60, 61, 62,124,255,
   33, 38, 60, 61, 62,124,255, 33, 38, 61,124,255, 38,124,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 61,255, 60,
   61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 60,
   61, 62, 94,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,
  255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42,
   43, 45, 47, 60, 61, 62, 94,124,255,124,255, 42, 47,255, 88,120,255,111,
  117,255, 43, 45, 47, 48, 98, 99,100,101,102,105,112,114,119,255,119,255,
   60, 61,255, 61, 62,255, 33, 38, 42, 43, 45, 60, 61, 62,124,255, 42, 47,
  255, 88,120,255, 60, 61,255, 61, 62,255,111,117,255, 33, 38, 42, 43, 45,
   47, 48, 60, 61, 62, 98, 99,100,101,102,105,112,114,119,124,255, 88,120,
  255,111,117,255, 43, 45, 48, 98, 99,100,101,102,105,112,114,119,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,7,2,7,4,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,0,
  4,0,4,1,0,4,0,4,0,1,4,7,7,4,6,7,4,7,4,6,4,2,4,7,7,4,7,4,6,4,2,4,2,4,2,
  4,0,0,4,3,3,2,2,2,2,2,2,2,3,2,3,7,7,2,7,2,7,7,7,7,2,2,2,4,7,7,4,7,7,2,
  7,4,0,0,4,0,0,4,3,3,2,2,4,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,4,
  0,0,4,2,4,0,0,4,3,3,2,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,4,0,0,0,0,0,
  0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,0,4,0,0,4,3,
  3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,3,3,2,7,7,4,0,0,4,0,0,4,
  3,3,2,3,2,3,4,3,3,3,3,3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,
  4,0,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,0,0,4,0,
  4,1,0,4,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,3,2,
  4,3,4,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,7,4,7,4,0,0,4,0,0,4,3,
  3,3,3,3,2,3,2,3,4,0,0,4,0,0,4,0,0,4,0,0,4,5,7,4,3,3,3,3,3,2,2,2,3,2,7,
  7,2,7,7,7,7,7,7,3,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,7,4
};

static const unsigned char ag_key_parm[] = {
    0,121,126,  0,  0,  6,  0,  0,  4,  8,  0,  2,  0,125,  0,228,221,  0,
  224,  0,247,216,  0,248,214,  0,249,215,  0,121,126,217,  0,152,151,  0,
  222,  0,238,235,  0,223,  0,237,239,  0,  4, 32,  0, 16, 26,  0,  8,  0,
   20,  0,  0,  0,  0,  6,  0,  2,  0, 18,  0,  0,  0,  0,  0,  0,  0,219,
  227,  0,233,218,  0,  0,  0,  0,  0,  0,  0,232,  0,220, 10, 12,  0, 24,
    0, 22, 30, 28, 14,  0,  0,  0,  0,  0,  6,  0,  4,  8,  0,  2,  0,121,
  126,  0,152,151,  0,248,249,  0,  0,  0,121,126,  0,152,151,  0, 16, 26,
    0,248,249,  0,  0, 10, 12,  0, 20, 22, 28, 14, 18,  0,121,126,  0,  0,
    0,152,151,  0,248,249,  0,  0,152,151,  0, 16, 26,  0,248,249,  0, 10,
   12,  0, 20, 22, 28, 14, 18,  0,165,164,166,167,168,169,170,171,172,173,
  174,179,  0,  0,  0,121,126,  0,238,235,  0,237,239,  0,233,228,247,  0,
    0,232,  0,227,  0,238,235,  0,237,239,  0,233,228,247,  0,232,  0,227,
    0,121,126,  0,152,151,  0,248,249,  0,  0, 32, 30,  0,152,151,  0,248,
  249,  0, 32, 30,  0,238,235,  0,237,239,  0,233,228,  0,232,  0,227,  0,
  233,228,235,232,237,227,  0,233,228,232,227,  0,228,227,  0,228,221,  0,
  224,  0,247,216,  0,248,214,  0,249,215,  0,121,126,217,  0,222,  0,238,
  235,  0,223,  0,237,239,  0,219,227,  0,233,218,  0,  0,  0,  0,  0,  0,
  232,  0,220,  0,  0,228,221,  0,224,  0,247,216,  0,248,214,  0,249,215,
    0,222,  0,238,235,  0,223,  0,237,239,  0,219,227,  0,233,218,  0,  0,
    0,  0,217,  0,232,  0,220,  0,  0,227,  0,121,126,  0,152,151,  0, 16,
   26,  0,248,249,  0,  0, 10, 12,  0, 24, 20, 22, 28, 14, 18,  0, 18,  0,
  238,235,  0,237,239,  0,233,228,247,248,249,  0,232,  0,227,  0,121,126,
    0,152,151,  0,238,235,  0,237,239,  0, 16, 26,  0,233,228,247,248,249,
    0,  0,  0,232,  0, 10, 12,  0, 24, 20, 22, 28, 14, 18,227,  0,152,151,
    0, 16, 26,  0,248,249,  0, 10, 12,  0, 24, 20, 22, 28, 14, 18,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 14, 19,  0,  1,  0,  6,  4, 27,  0, 36,  0,  0,  0,  0,
    0,  0, 18,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0, 36,  0,  0,  0,  0,  0, 41,  0, 59, 63,  0, 46, 67,  0, 74,  0,
   52,  0, 54,  0, 96,101,  0,109,  0, 61,  0, 63,  0, 65,  0, 67,  0,  0,
    0,  0, 38, 40, 15, 20, 23, 26, 29, 33, 38, 42, 43, 44, 46, 51, 49, 70,
   56, 79, 81, 85, 90, 58, 69, 71,  0,127,132,  0,113,119, 99,140,  0,  0,
    0,  0,  0,  0,  0,149,151,107,110,  0,  0,  0,  0,  0,  0,  0,  0,170,
    0,153,155,118,121,157,162,124,173,176,178,183,189,  0,  0,  0,  0,140,
    0,  0,  0,  0,194,196,145,  0,  0,  0,  0,  0,215,  0,198,200,152,202,
  207,155,218,221,223,228,234,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,170,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,239,241,243,185,
  188,245,191,247,  0,  0,  0,  0,  0,  0,  0,249,251,253,203,255,206,257,
    0,  0,  0,  0,  0,  0,  0,259,261,217,220,263,269,  0,  0,  0,  0,273,
  275,230,277,283,  0,  0,  0,  0,  0,  0,  0,287,289,239,291,242,293,  0,
  295,297,299,301,303,305,  0,307,309,311,313,  0,315,317,  0,  0,  0,  0,
    0,  0,270,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,285,
    0,  0,  0,  0,  0,290,  0,  0,  0,  0,319,321,267,272,275,278,281,287,
  323,292,325,295,  0,  0,  0,  0,  0,  0,314,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,325,  0,  0,  0,  0,  0,330,  0,  0,  0,  0,327,329,311,316,
  319,322,331,327,333,332,335,335,  0,337,  0,  0,  0,  0,  0,  0,  0,  0,
  356,  0,339,341,353,356,343,348,359,359,363,366,368,373,379,  0,384,  0,
    0,  0,  0,  0,  0,  0,389,391,393,395,397,378,399,381,401,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,428,  0,403,405,407,409,411,
  394,397,400,413,403,415,420,406,431,435,438,440,445,451,456,  0,  0,  0,
    0,  0,475,  0,458,460,430,462,467,433,478,482,485,487,492,498,  0
};

static const unsigned short ag_key_index[] = {
    7,  0, 13, 74,102,  0,  0, 13, 13,113,127,127,143,127,148,158,158,  0,
  158,183,  0,  0,183,194,  0,194,194,194,194,194,194,194,194,194,209,223,
    0,  0,233,113,113,113,113,209,148,148,148,148,245,245,245,252,259,264,
  264,264,298,264,338,351,338,  0,  0,143,113,143,127,362,362,148,  0,113,
  143,143,158,362,  0,143,  0,  0,148,  0,  0,  0,148,  0,158,376,148,158,
  158,183,194,  0,  0,183,183,183,  0,194,  0,  0,143,  0,143,  0,  0,113,
  148,384,113,148,113,148,113,148,148,148,113,148,113,148,113,148,113,148,
  113,148,113,148,113,148,113,148,113,148,113,148,113,148,113,148,148,113,
  148,113,148,113,148,113,148,113,148,113,148,113,148,113,148,113,148,113,
  148,113,148,113,148,113,148,113,148,113,148,409,158,148,148,  0,  0,  0,
    0,148,436,  0,  0,  0,  0,148,148,245,245,245,245,245,245,245,245,252,
  252,259,264,264,148,  0,  0,148,148,362,  0,148,  0,127,158,194,194,264,
  148,264,  0,  0,113,148,148
};

static const unsigned char ag_key_ends[] = {
111,76,111,111,112,0, 111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
47,0, 61,0, 61,0, 61,0, 61,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 111,111,112,0, 98,108,101,0, 
109,112,0, 108,115,101,0, 105,101,108,100,0, 102,0, 111,110,103,0, 
114,105,110,116,0, 101,116,117,114,110,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 111,111,112,0, 111,76,111,111,112,0, 
111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
43,0, 45,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 43,0, 
45,0, 43,0, 45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 
42,0, 61,0, 124,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 111,117,98,108,101,0, 111,110,103,0, 
61,0, 38,0, 61,0, 124,0, 61,0, 38,0, 61,0, 61,0, 61,0, 124,0, 
61,0, 38,0, 61,0, 124,0, 38,0, 124,0, 61,0, 61,0, 61,0, 
61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 124,0, 43,0, 45,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 104,105,108,101,0, 61,0, 
38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 61,0, 38,0, 42,0, 43,0, 
45,0, 61,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 124,0, 43,0, 45,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   14,185,185,185,185,185,185,185,185,184,130,184,184,184,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,184,245,156,185,
  185,244,231,181,204,196,242,240,211,241,137,243,149,186,186,186,186,186,
  186,186,148,148,225,197,234,213,236,226,185,187,187,187,187,134,187,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,185,178,185,230,188,185,187,187,187,187,134,187,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,208,229,207,
  246,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,185,185,185
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
184,130,126,121,24,23,22,21,20,0,1,189,190,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,181,178,156,149,148,137,134,130,0,128,129,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,181,178,156,149,148,137,134,130,125,0,123,
  124,
184,130,126,121,0,1,
24,23,22,21,20,0,7,8,9,10,11,12,191,192,193,194,195,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,181,178,156,149,148,137,134,0,
130,0,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,181,178,156,149,148,137,134,130,0,
125,0,
249,248,246,245,241,240,204,197,196,188,187,186,184,181,156,152,151,149,148,
  137,134,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,184,181,156,152,151,149,148,137,134,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,184,181,156,152,151,149,148,137,134,130,126,121,0,1,189,190,
204,184,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,184,181,156,152,151,149,148,137,134,130,126,121,14,0,1,189,190,
249,248,246,245,241,240,204,197,196,188,187,186,181,156,152,151,149,148,137,
  134,0,2,3,4,5,6,18,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,
  102,106,107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,
  252,253,254,255,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,16,25,26,27,29,30,
  31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,53,55,56,57,58,71,76,
  79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,16,25,26,27,29,30,
  31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,53,55,56,57,58,71,76,
  79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
204,0,15,47,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,14,0,13,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,181,179,178,174,173,172,171,170,169,168,167,
  166,165,164,156,149,148,137,134,0,157,
186,149,148,0,144,
186,149,148,0,138,144,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,179,178,174,173,172,171,170,169,168,167,166,
  165,164,156,149,148,137,134,0,160,161,162,163,175,176,177,182,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,184,156,130,126,121,0,1,189,190,
137,134,0,
134,0,
187,186,149,148,134,0,154,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,186,184,149,148,137,134,130,126,121,0,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,186,184,149,148,137,134,130,126,121,0,144,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,184,130,126,121,0,1,189,190,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,184,130,126,121,0,1,189,190,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,184,130,126,121,0,1,189,190,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,188,187,184,134,130,126,121,0,1,189,190,
247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,
  226,225,211,197,196,188,187,184,134,130,126,121,0,1,189,190,
156,0,5,255,
251,250,249,248,246,245,241,240,204,197,196,188,187,186,184,181,156,152,151,
  149,148,137,134,130,126,121,0,1,189,190,
188,187,134,0,4,58,209,
188,187,134,0,4,58,209,
251,250,249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,
  134,0,2,3,4,5,6,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,
  102,106,107,108,109,111,112,113,114,115,116,119,133,136,139,140,141,145,
  153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
247,0,110,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
244,243,242,0,103,104,105,
241,240,0,100,101,
239,238,0,97,98,
237,236,235,234,0,92,93,94,95,
233,232,0,89,90,
231,0,87,
230,0,85,
229,0,83,
249,248,247,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,
  228,227,226,225,224,223,222,221,220,219,218,217,216,215,214,213,211,204,
  197,196,188,187,186,184,149,148,134,130,126,121,0,1,189,190,
228,0,80,
204,0,47,
227,226,0,72,77,
249,248,224,223,222,221,220,219,218,217,216,215,214,213,0,59,60,61,62,63,64,
  65,66,67,68,69,70,112,113,
211,0,54,
197,196,0,34,35,
204,184,130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
188,187,184,134,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,207,205,204,203,202,201,200,199,198,197,
  188,187,186,184,181,156,152,151,149,148,137,134,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,207,206,205,204,203,202,201,200,199,198,
  197,188,187,186,184,181,156,152,151,149,148,137,134,130,126,121,14,0,1,
  189,190,
249,248,246,245,241,240,212,210,208,207,206,205,204,203,202,201,200,199,198,
  197,188,187,186,184,181,156,152,151,149,148,137,134,130,126,121,14,0,1,
  189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
188,187,134,0,4,209,
249,248,246,245,241,240,204,197,188,187,186,184,181,156,152,151,149,148,137,
  134,130,126,121,0,1,189,190,
197,184,130,126,121,0,1,189,190,
197,184,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,207,205,204,203,202,201,200,199,198,197,
  188,187,186,181,156,152,151,149,148,137,134,0,13,
249,248,246,245,241,240,212,210,208,207,206,205,204,203,202,201,200,199,198,
  197,196,188,187,186,184,181,156,152,151,149,148,137,134,130,126,121,14,
  0,1,189,190,
204,0,47,
204,184,130,126,121,0,1,189,190,
211,197,0,34,54,
211,197,0,34,54,
249,248,246,245,241,240,204,197,188,187,186,181,156,152,151,149,148,137,134,
  0,2,3,4,5,6,18,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,
  106,107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,
  253,254,255,
197,0,34,
197,0,34,
211,197,0,34,54,
249,248,246,245,241,240,204,197,188,187,186,181,156,152,151,149,148,137,134,
  0,2,3,4,5,6,33,36,47,48,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,
  102,106,107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,
  252,253,254,255,
204,0,47,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,16,25,26,27,29,30,
  31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,53,55,56,57,58,71,76,
  79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
202,0,17,44,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,
  107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,
  254,255,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,16,25,26,27,29,30,
  31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,53,55,56,57,58,71,76,
  79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,14,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,53,55,56,57,58,71,
  76,79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,
  119,133,136,139,140,141,145,153,209,252,253,254,255,
246,245,244,243,242,241,240,236,234,231,230,229,226,225,213,211,208,207,204,
  197,196,188,187,186,185,184,181,179,178,174,173,172,171,170,169,168,167,
  166,165,164,156,149,148,137,134,0,158,160,161,162,163,175,176,177,
186,149,148,0,138,144,
186,149,0,
187,186,149,148,134,0,180,
186,149,0,
186,149,0,
187,186,149,148,134,0,180,
181,0,
186,149,148,0,138,144,
241,240,186,149,148,0,135,142,
241,240,186,149,148,0,135,142,
196,184,130,126,121,0,1,189,190,
196,0,35,
196,184,130,126,121,0,1,189,190,
196,0,35,
211,196,0,35,54,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,0,112,113,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,100,101,102,106,107,108,109,111,112,113,114,119,133,136,
  139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,99,100,101,102,106,107,108,109,111,112,113,114,119,133,
  136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,99,100,101,102,106,107,108,109,111,112,113,114,119,133,
  136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,96,99,100,101,102,106,107,108,109,111,112,113,114,119,133,
  136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,96,99,100,101,102,106,107,108,109,111,112,113,114,119,133,
  136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,
  119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,
  119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,
  114,119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,
  114,119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,
  113,114,119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,
  81,
249,248,246,245,241,240,204,196,188,187,186,181,156,152,151,149,148,137,134,
  0,2,3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,
  107,108,109,111,112,113,114,117,118,119,133,136,139,140,141,145,153,209,
  252,253,254,255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,
  78,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,
  73,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,228,227,226,225,212,211,210,208,207,206,205,204,203,202,201,200,
  199,198,197,196,188,187,186,184,181,156,152,151,149,148,137,134,130,126,
  121,14,0,1,189,190,
249,248,246,245,241,240,212,210,208,207,205,204,203,202,201,200,199,198,197,
  188,187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,52,53,55,56,57,58,
  71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,
  114,119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,197,188,187,186,181,156,152,151,149,148,137,134,
  0,2,3,4,5,6,18,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,
  106,107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,
  253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
188,187,134,0,4,209,
197,0,34,
211,0,54,
197,0,34,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,
  107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,
  254,255,
206,0,28,50,
204,0,15,47,
211,196,0,35,54,
186,149,148,0,143,
186,149,148,0,143,
249,248,204,188,187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,47,58,
  109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,204,188,187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,47,58,
  109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,255,
244,243,242,0,103,104,105,
244,243,242,0,103,104,105,
241,240,0,100,101,
241,240,0,100,101,
239,238,0,97,98,
239,238,0,97,98,
239,238,0,97,98,
239,238,0,97,98,
237,236,235,234,0,92,93,94,95,
237,236,235,234,0,92,93,94,95,
233,232,0,89,90,
231,0,87,
230,0,85,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,
  112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,255,
211,0,54,
196,0,35,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,58,76,79,82,84,86,88,91,96,99,100,101,102,106,107,108,109,
  111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,255,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,36,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,
  107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,
  254,255,
249,248,246,245,241,240,212,210,208,207,206,205,204,203,202,201,200,199,198,
  197,188,187,186,184,181,156,152,151,149,148,137,134,130,126,121,14,0,1,
  189,190,
197,0,34,
249,248,246,245,241,240,204,196,188,187,186,181,156,152,151,149,148,137,134,
  0,2,3,4,5,6,33,36,47,48,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,
  102,106,107,108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,
  252,253,254,255,
211,196,0,35,54,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,184,181,156,152,151,149,148,137,134,130,126,121,0,1,189,190,
249,248,246,245,241,240,212,210,208,205,204,203,202,201,200,199,198,197,188,
  187,186,181,156,152,151,149,148,137,134,0,2,3,4,5,6,25,26,27,29,30,31,
  32,34,36,37,38,39,40,41,42,43,44,45,46,47,49,51,53,55,56,57,58,71,76,79,
  82,84,86,88,91,96,99,100,101,102,106,107,108,109,111,112,113,114,119,
  133,136,139,140,141,145,153,209,252,253,254,255,
186,149,148,0,144,
186,149,148,0,144,
229,0,83,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,56,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,
  108,109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,
  255,
228,0,80,
225,211,0,54,74,
196,0,35,
249,248,246,245,241,240,204,188,187,186,184,181,156,152,151,149,148,137,134,
  130,126,121,0,1,189,190,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,
  75,
249,248,246,245,241,240,204,188,187,186,181,156,152,151,149,148,137,134,0,2,
  3,4,5,6,47,57,58,71,76,79,82,84,86,88,91,96,99,100,101,102,106,107,108,
  109,111,112,113,114,119,133,136,139,140,141,145,153,209,252,253,254,255,

};


static unsigned const char ag_astt[6398] = {
  1,1,1,1,8,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,2,2,2,2,2,7,1,1,1,1,1,0,1,1,1,1,1,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,
  1,1,1,1,1,1,1,4,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,2,2,2,1,2,1,
  2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  1,1,7,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,
  1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,
  2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,
  4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,5,2,1,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,2,1,2,2,2,7,2,2,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,1,1,1,5,1,1,1,1,1,5,1,1,1,1,5,1,1,1,1,1,1,5,1,1,1,1,1,1,5,1,1,1,5,1,1,
  5,1,1,5,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,5,1,1,4,1,1,1,5,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,5,1,1,1,7,2,2,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,5,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,
  1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,1,7,3,1,1,1,7,3,1,
  1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,2,1,7,2,1,1,7,
  3,1,1,1,1,1,1,1,1,5,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,2,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,2,7,2,2,1,1,2,3,3,
  3,1,3,2,2,1,3,1,3,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,
  2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,
  5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,5,1,
  1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,4,2,2,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,1,1,1,
  4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  7,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,
  7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,
  2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,
  1,2,2,1,1,1,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,7,2,1,
  5,1,1,7,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,1,7,
  2,1,1,1,7,2,1,2,2,2,7,1,2,2,2,7,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,
  1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,
  1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,1,1,4,1,1,
  1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,4,1,1,4,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,1,1,1,1,1,1,5,1,1,7,2,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,2,1,1,1,1,1,1,1,5,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,2,2,1,2,2,2,1,2,1,2,1,1,1,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,10,10,10,4,2,10,10,10,4,2,1,4,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,4,1,1,1,7,1,1,1,7,2,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
356,356,1,2,358,356,
10,9,8,7,6,4,18,17,16,15,14,0,13,12,11,10,9,
131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
  131,131,131,131,131,131,131,131,131,131,131,131,131,131,133,
134,6,
126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,
  126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,128,
129,8,
357,357,357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,
  357,357,3,1,2,9,3,3,363,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,3,357,357,357,357,357,357,357,357,3,1,2,10,3,3,362,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,3,357,357,357,357,357,357,357,357,3,1,2,11,3,3,361,
357,3,3,1,2,12,3,3,360,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,3,357,357,357,357,357,357,357,357,3,1,2,357,13,3,3,359,
32,33,39,40,41,42,35,46,46,135,135,155,22,19,165,165,161,155,21,135,14,120,
  121,58,34,123,62,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
32,33,39,40,41,42,64,65,66,77,35,63,67,68,71,72,73,75,135,135,155,22,19,165,
  165,161,155,21,135,15,120,121,58,34,123,4,4,4,86,4,19,20,84,4,83,4,82,
  81,80,79,78,19,20,76,76,38,85,74,70,69,83,83,60,59,57,55,54,53,52,51,50,
  49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,
  30,29,23,
32,33,39,40,41,42,64,65,66,77,35,63,67,68,71,72,73,75,135,135,155,22,19,165,
  165,161,155,21,135,16,120,121,58,34,123,87,87,87,86,87,19,20,84,87,83,
  87,82,81,80,79,78,19,20,76,76,38,85,74,70,69,83,83,60,59,57,55,54,53,52,
  51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,
  56,31,30,29,23,
35,17,89,88,
90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,
  90,90,90,90,90,18,90,
172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
  172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
  172,172,172,172,172,172,172,172,19,91,
160,160,160,144,160,
92,92,92,21,142,92,
98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,
  98,94,93,189,188,187,186,185,184,183,182,181,180,179,98,98,98,98,98,22,
  98,98,98,97,96,95,98,98,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,3,357,3,1,2,23,3,3,423,
99,100,24,
101,137,
168,167,167,167,168,154,166,
153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,162,153,162,159,145,145,153,153,153,27,
152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,152,
  152,152,152,152,152,156,152,156,156,143,143,152,152,152,28,156,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,3,3,1,2,29,3,3,422,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,3,3,1,2,30,3,3,421,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,3,3,1,2,31,3,3,420,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,357,357,3,357,3,1,2,32,3,3,417,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,357,357,3,357,3,1,2,33,3,3,416,
19,122,170,23,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,
  357,357,357,357,3,1,2,35,3,3,372,
135,135,135,36,114,108,56,
135,135,135,37,114,107,56,
102,104,32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,38,
  120,121,58,34,123,106,38,106,106,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,105,103,34,25,24,28,20,27,26,165,56,31,30,29,
  23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,39,3,3,414,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,40,3,3,413,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,41,3,3,409,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,42,3,3,408,
107,102,108,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,44,120,121,58,
  34,123,38,109,47,46,101,101,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,45,120,121,58,
  34,123,38,109,47,46,100,100,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,46,120,121,58,
  34,123,38,109,47,46,99,99,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,
  56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,47,120,121,58,
  34,123,38,109,47,46,98,98,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,
  56,31,30,29,23,
110,112,114,90,115,113,111,
41,42,87,117,116,
118,120,82,121,119,
122,124,126,128,79,129,127,125,123,
130,132,77,133,131,
134,75,135,
136,73,137,
138,70,139,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,136,136,136,3,136,136,136,3,1,2,56,3,3,377,
140,67,141,
35,114,142,
143,145,63,146,144,
32,33,147,149,151,153,155,157,159,161,163,165,167,169,106,170,168,166,164,
  162,160,158,156,154,152,150,148,109,110,
171,47,172,
75,173,62,5,5,
357,3,3,1,2,63,3,3,371,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,64,3,3,380,
357,357,3,357,3,1,2,65,3,3,378,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,3,357,357,357,357,357,357,357,357,3,1,2,66,3,3,376,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,3,357,357,357,357,357,357,357,357,3,1,2,357,67,3,3,370,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,3,357,357,357,357,357,357,357,357,3,1,2,357,68,3,3,369,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,69,120,121,58,
  34,123,38,44,44,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
135,135,135,70,42,56,
357,357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,
  357,3,1,2,71,3,3,368,
357,3,3,1,2,72,3,3,367,
357,3,3,1,2,73,3,3,366,
174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,74,174,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,1,2,357,75,3,3,
  365,
35,76,175,
357,3,3,1,2,77,3,3,373,
171,75,78,29,176,
171,75,79,28,177,
32,33,39,40,41,42,35,46,135,135,155,22,19,165,165,161,155,21,135,80,120,121,
  58,34,123,178,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
75,81,26,
75,82,25,
171,75,83,22,172,
32,33,39,40,41,42,35,36,135,135,155,22,19,165,165,161,155,21,135,84,120,121,
  58,34,123,180,179,38,38,179,179,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
35,85,181,
32,33,39,40,41,42,64,65,66,77,35,63,67,68,71,72,73,75,135,135,155,22,19,165,
  165,161,155,21,135,86,120,121,58,34,123,15,15,182,86,182,19,20,84,182,
  83,182,82,81,80,79,78,19,20,76,76,38,85,74,70,69,83,83,60,59,57,55,54,
  53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
67,87,183,33,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,88,120,121,58,
  34,123,184,38,184,184,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,
  44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
32,33,39,40,41,42,64,65,66,77,35,63,67,68,71,72,73,75,135,135,155,22,19,165,
  165,161,155,21,135,89,120,121,58,34,123,2,2,2,86,2,19,20,84,2,83,2,82,
  81,80,79,78,19,20,76,76,38,85,74,70,69,83,83,60,59,57,55,54,53,52,51,50,
  49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,
  30,29,23,
32,33,39,40,41,42,64,65,66,77,35,63,67,68,71,72,73,75,135,135,155,22,19,165,
  165,161,155,21,135,1,90,120,121,58,34,123,12,12,12,86,12,19,20,84,12,83,
  12,82,81,80,79,78,19,20,76,76,38,85,74,70,69,83,83,60,59,57,55,54,53,52,
  51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,
  56,31,30,29,23,
173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,173,
  173,173,173,173,173,173,173,173,94,93,189,188,187,186,185,184,183,182,
  181,180,179,171,173,173,173,173,91,173,173,173,173,97,96,95,173,
92,92,92,150,151,92,
193,193,93,
199,198,198,198,199,94,196,
195,195,191,
194,194,190,
199,198,198,198,199,178,197,
200,98,
92,92,92,141,140,92,
185,186,186,186,186,100,139,186,
185,186,186,186,186,101,138,186,
357,3,3,1,2,102,3,3,419,
173,103,187,
357,3,3,1,2,104,3,3,418,
173,105,188,
171,173,106,104,172,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,107,3,3,415,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,108,120,121,
  58,34,123,38,109,47,46,103,103,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,165,56,31,30,29,23,
32,33,106,109,110,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,110,3,3,412,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,111,120,121,
  58,34,123,38,109,47,46,96,96,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,112,3,3,411,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,113,120,121,
  58,34,123,38,109,47,46,95,95,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,114,3,3,410,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,115,120,121,
  58,34,123,38,109,47,46,94,94,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,116,120,121,
  58,34,123,38,109,189,47,46,189,189,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,165,56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,117,120,121,
  58,34,123,38,109,190,47,46,190,190,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,118,3,3,407,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,119,120,121,
  58,34,123,38,109,191,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,120,3,3,406,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,121,120,121,
  58,34,123,38,109,192,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,122,3,3,405,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,123,120,121,
  58,34,123,38,109,193,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,124,3,3,404,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,125,120,121,
  58,34,123,38,109,194,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,126,3,3,403,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,127,120,121,
  58,34,123,38,109,195,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,128,3,3,402,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,129,120,121,
  58,34,123,38,109,196,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,130,3,3,401,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,131,120,121,
  58,34,123,38,109,197,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,132,3,3,400,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,133,120,121,
  58,34,123,38,109,198,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,134,3,3,399,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,135,120,121,
  58,34,123,38,109,199,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,
  24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,136,3,3,398,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,137,120,121,
  58,34,123,38,109,200,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,138,3,3,397,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,139,120,121,
  58,34,123,38,109,201,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,
  34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,140,3,3,396,
71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,141,202,
32,33,39,40,41,42,35,116,135,135,155,22,19,165,165,161,155,21,135,142,120,
  121,58,34,123,38,118,118,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,204,203,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,143,3,3,395,
68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,68,144,205,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,145,3,3,394,
64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,146,206,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,147,3,3,392,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,148,120,121,
  58,34,123,38,62,62,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,149,3,3,391,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,150,120,121,
  58,34,123,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,151,3,3,390,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,152,120,121,
  58,34,123,38,60,60,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,153,3,3,389,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,154,120,121,
  58,34,123,38,59,59,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,155,3,3,388,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,156,120,121,
  58,34,123,38,58,58,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,157,3,3,387,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,158,120,121,
  58,34,123,38,57,57,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,159,3,3,386,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,160,120,121,
  58,34,123,38,56,56,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,161,3,3,385,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,162,120,121,
  58,34,123,38,55,55,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,163,3,3,384,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,164,120,121,
  58,34,123,38,54,54,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,165,3,3,383,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,166,120,121,
  58,34,123,38,53,53,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,167,3,3,382,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,168,120,121,
  58,34,123,38,52,52,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,169,3,3,381,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,170,120,121,
  58,34,123,38,51,51,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,171,3,3,379,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,172,120,121,
  58,34,123,38,49,49,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,1,2,357,
  173,3,3,364,
32,33,39,40,41,42,64,65,66,207,77,35,63,67,68,71,72,73,75,135,135,155,22,19,
  165,165,161,155,21,135,174,120,121,58,34,123,12,12,12,86,12,19,20,84,12,
  83,12,82,81,80,79,78,19,20,76,76,38,85,74,41,70,69,83,83,60,59,57,55,54,
  53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  165,56,31,30,29,23,
32,33,39,40,41,42,35,46,135,135,155,22,19,165,165,161,155,21,135,175,120,
  121,58,34,123,208,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,176,120,121,
  58,34,123,38,45,45,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
135,135,135,177,43,56,
75,178,27,
171,37,172,
75,180,209,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,181,120,121,
  58,34,123,210,38,210,210,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
211,14,212,40,
35,183,3,88,
171,173,184,34,172,
157,157,157,185,213,
157,157,157,186,214,
32,33,35,135,135,155,22,19,165,165,161,155,21,135,187,120,121,58,34,123,38,
  109,113,113,37,36,113,34,25,24,28,20,27,26,165,56,31,30,29,23,
32,33,35,135,135,155,22,19,165,165,161,155,21,135,188,120,121,58,34,123,38,
  109,112,112,37,36,112,34,25,24,28,20,27,26,165,56,31,30,29,23,
110,112,114,92,115,113,111,
110,112,114,91,115,113,111,
41,42,89,117,116,
41,42,88,117,116,
118,120,86,121,119,
118,120,85,121,119,
118,120,84,121,119,
118,120,83,121,119,
122,124,126,128,81,129,127,125,123,
122,124,126,128,80,129,127,125,123,
130,132,78,133,131,
134,76,135,
136,74,137,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,202,120,121,
  58,34,123,38,109,215,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,
  43,34,25,24,28,20,27,26,165,56,31,30,29,23,
171,117,216,
173,204,115,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,205,120,121,
  58,34,123,38,109,217,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,
  36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,206,120,121,
  58,34,123,218,38,218,218,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,357,357,3,357,357,357,357,357,357,357,357,3,1,2,357,207,3,3,375,
75,208,35,
32,33,39,40,41,42,35,36,135,135,155,22,19,165,165,161,155,21,135,209,120,
  121,58,34,123,219,179,38,38,179,179,60,59,57,55,54,53,52,51,50,49,48,47,
  46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
171,173,210,39,172,
357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,357,
  357,357,3,357,357,357,357,357,357,357,357,3,1,2,211,3,3,374,
32,33,39,40,41,42,64,65,66,77,35,63,67,68,71,72,73,75,135,135,155,22,19,165,
  165,161,155,21,135,212,120,121,58,34,123,16,17,86,17,19,20,84,17,83,17,
  82,81,80,79,78,19,20,76,76,38,85,74,70,69,83,83,60,59,57,55,54,53,52,51,
  50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,165,56,
  31,30,29,23,
158,158,158,149,158,
158,158,158,148,158,
138,72,139,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,216,120,121,
  58,34,123,38,119,119,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,
140,69,141,
220,171,218,172,221,
173,219,21,
357,357,357,357,357,357,357,357,357,357,3,357,357,357,357,357,357,357,357,3,
  1,2,220,3,3,393,
65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,65,221,222,
32,33,39,40,41,42,35,135,135,155,22,19,165,165,161,155,21,135,222,120,121,
  58,34,123,38,66,109,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,165,56,31,30,29,23,

};


static const unsigned short ag_sbt[] = {
     0,  13,  50,  88,  94, 111, 145, 147, 182, 184, 212, 249, 286, 295,
   333, 399, 496, 593, 597, 629, 676, 681, 687, 740, 773, 776, 778, 785,
   819, 854, 886, 918, 950, 985,1020,1024,1054,1061,1068,1135,1161,1187,
  1213,1239,1242,1292,1342,1392,1442,1449,1454,1459,1468,1473,1476,1479,
  1482,1535,1538,1541,1546,1575,1578,1583,1592,1618,1629,1667,1707,1747,
  1809,1815,1842,1851,1860,1892,1933,1936,1945,1950,1955,2020,2023,2026,
  2031,2097,2100,2197,2201,2264,2361,2459,2513,2519,2522,2529,2532,2535,
  2542,2544,2550,2558,2566,2575,2578,2587,2590,2595,2621,2671,2676,2702,
  2752,2778,2828,2854,2904,2955,3006,3032,3084,3110,3162,3188,3241,3267,
  3320,3346,3399,3425,3478,3504,3558,3584,3638,3664,3719,3745,3801,3827,
  3884,3910,3930,3995,4021,4041,4067,4087,4113,4175,4201,4263,4289,4351,
  4377,4439,4465,4527,4553,4615,4641,4703,4729,4791,4817,4879,4905,4967,
  4993,5055,5081,5143,5169,5231,5292,5391,5456,5518,5524,5527,5530,5533,
  5596,5600,5604,5609,5614,5619,5659,5699,5706,5713,5718,5723,5728,5733,
  5738,5743,5752,5761,5766,5769,5772,5830,5833,5836,5895,5958,5998,6001,
  6067,6072,6109,6205,6210,6215,6218,6280,6283,6288,6291,6317,6337,6398
};


static const unsigned short ag_sbe[] = {
     9,  47,  85,  92,  99, 144, 146, 181, 183, 208, 245, 282, 291, 329,
   353, 428, 525, 594, 627, 674, 679, 684, 731, 769, 775, 777, 783, 818,
   852, 882, 914, 946, 981,1016,1021,1050,1057,1064,1088,1157,1183,1209,
  1235,1240,1260,1310,1360,1410,1445,1451,1456,1463,1470,1474,1477,1480,
  1531,1536,1539,1543,1560,1576,1580,1588,1614,1625,1663,1703,1743,1765,
  1812,1838,1847,1856,1890,1929,1934,1941,1947,1952,1974,2021,2024,2028,
  2050,2098,2129,2198,2219,2293,2391,2504,2516,2521,2527,2531,2534,2540,
  2543,2547,2555,2563,2571,2576,2583,2588,2592,2617,2639,2673,2698,2720,
  2774,2796,2850,2872,2922,2973,3028,3050,3106,3128,3184,3206,3263,3285,
  3342,3364,3421,3443,3500,3522,3580,3602,3660,3682,3741,3763,3823,3845,
  3906,3928,3949,4017,4039,4063,4085,4109,4131,4197,4219,4285,4307,4373,
  4395,4461,4483,4549,4571,4637,4659,4725,4747,4813,4835,4901,4923,4989,
  5011,5077,5099,5165,5187,5288,5322,5410,5474,5521,5525,5528,5531,5551,
  5597,5601,5606,5612,5617,5633,5673,5702,5709,5715,5720,5725,5730,5735,
  5740,5747,5756,5763,5767,5770,5790,5831,5834,5854,5913,5994,5999,6020,
  6069,6105,6138,6208,6213,6216,6236,6281,6285,6289,6313,6335,6355,6398
};


static const unsigned char ag_fl[] = {
  2,3,3,4,2,3,1,1,1,1,1,0,2,1,1,2,4,4,1,1,1,5,2,1,1,2,2,3,2,2,1,1,1,1,3,
  4,0,1,1,4,1,3,2,3,2,3,0,1,1,3,1,3,3,3,3,3,3,3,3,3,3,3,3,1,0,0,7,1,0,4,
  1,0,4,1,3,1,3,1,3,1,3,3,1,3,3,3,3,1,3,3,1,3,3,1,3,3,3,1,2,2,2,2,1,3,3,
  1,1,2,2,2,2,1,4,4,1,4,0,1,1,3,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,
  3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,3,0,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0, 12, 12, 12, 12, 12,191,192,193,194,195, 13, 13, 16, 16, 25, 25, 26,
   26, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 30, 31, 45, 17, 15, 32,
   48, 48, 33, 27, 28, 37, 41, 41, 42, 42, 18, 18, 36, 36, 56, 56, 56, 56,
   56, 56, 56, 56, 56, 56, 56, 56, 56, 57, 73, 75, 57, 71, 78, 71, 76, 81,
   76, 79, 79, 82, 82, 84, 84, 86, 86, 86, 88, 88, 88, 88, 88, 91, 91, 91,
   96, 96, 96, 99, 99, 99, 99,102,102,102,102,102,106,106,109,109,109,109,
  109,109,109,109,109,109, 58,114,117,117,118,118,111,111,111,111,  1,123,
  123,124,124,  1,128,128,129,129,  1,209,209,253,253,253,133,133,133,136,
  136,136,142,142,135,135,138,138,252,252,252,139,139,143,143,140,140,141,
  141,153,153,145,145,154,154,119,119,255,157,157,158,158,160,160,160,161,
  161,161,161,161,161,161,161,161,161,161,162,162,162,175,176,177,163,163,
  180,180,254,182,182,144, 19, 19,120,120,122,122,122,122,122,122,122,122,
  122,122,122,122,122,122,122,122,122,122,122,122,122,122,122,122,122,122,
  122,122,122,122,122,122,122,122,127,127,127,127,127,127,127,127,127,127,
  127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,
  127,127,127,127,127,131,131,131,132,132,132,132,132,132,146,146,147,147,
  147,150,150,155,155,159,159,159,159,159,159,159,159,159,159,159,159,159,
  159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
  183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,
  183,183,183,183,183,183,183,183,183,183,183,183,183,189,189,190,190,  7,
    8,  9, 10, 11, 35, 34, 38, 39, 40, 43, 44, 46, 47, 49, 50, 52, 51,  4,
   53, 54, 55, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 74, 72, 77,
   80, 83, 85, 87, 89, 90, 92, 93, 94, 95, 97, 98,100,101,103,104,105,107,
  108,110,112,113,115,116,  2,  3,  6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: VNO AG_WRAP_5(ag_rp_5(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER); break;
    case 12: ag_rp_15(PCB_POINTER); break;
    case 13: ag_rp_16(PCB_POINTER, V(1,(FileLocation *)), V(3,(FileLocation *))); break;
    case 14: VWD(0,AG_WRAP_5 *); break;
    case 15: ag_rp_17(PCB_POINTER); break;
    case 16: ag_rp_18(PCB_POINTER); break;
    case 17: ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 18: ag_rp_20(PCB_POINTER); break;
    case 19: ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 20: ag_rp_22(PCB_POINTER); break;
    case 21: V(0,(FileLocation *)) = ag_rp_23(PCB_POINTER); break;
    case 22: ag_rp_24(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 23: ag_rp_25(PCB_POINTER); break;
    case 24: ag_rp_26(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 25: ag_rp_27(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 26: ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 27: ag_rp_29(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 28: VNO AG_WRAP_5(ag_rp_30(PCB_POINTER)); break;
    case 29: VRO(AG_WRAP_5 *, ag_rp_31(PCB_POINTER, VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 30: VRO(AG_WRAP_5 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 31: VRO(AG_WRAP_5 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 32: VRO(AG_WRAP_5 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 33: VRO(AG_WRAP_5 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 34: VRO(AG_WRAP_5 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 35: VRO(AG_WRAP_5 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 36: VRO(AG_WRAP_5 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 37: VRO(AG_WRAP_5 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 38: VRO(AG_WRAP_5 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 39: VRO(AG_WRAP_5 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: ag_rp_44(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 43: ag_rp_45(PCB_POINTER, VW(-5, AG_WRAP_5 *), VW(-2, AG_WRAP_5 *)); break;
    case 44: VRO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 45: ag_rp_47(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 46: VRO(AG_WRAP_5 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 47: ag_rp_49(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 48: VRO(AG_WRAP_5 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 49: VRO(AG_WRAP_5 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 50: VRO(AG_WRAP_5 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 51: VRO(AG_WRAP_5 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 52: VRO(AG_WRAP_5 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 53: VRO(AG_WRAP_5 *, ag_rp_55(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 54: VRO(AG_WRAP_5 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 55: VRO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 56: VRO(AG_WRAP_5 *, ag_rp_58(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 57: VRO(AG_WRAP_5 *, ag_rp_59(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 58: VRO(AG_WRAP_5 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 59: VRO(AG_WRAP_5 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 60: VRO(AG_WRAP_5 *, ag_rp_62(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 61: VRO(AG_WRAP_5 *, ag_rp_63(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 62: VRO(AG_WRAP_5 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 63: VRO(AG_WRAP_5 *, ag_rp_65(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 64: VRO(AG_WRAP_5 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 65: VNO AG_WRAP_5(ag_rp_67(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 66: VNO AG_WRAP_5(ag_rp_68(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 67: VNO AG_WRAP_5(ag_rp_69(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 68: VNO AG_WRAP_5(ag_rp_70(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 69: VRO(AG_WRAP_5 *, ag_rp_71(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 70: VNO AG_WRAP_5(ag_rp_72(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 71: VRO(AG_WRAP_5 *, ag_rp_73(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 72: VNO AG_WRAP_5(ag_rp_74(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 73: VNO AG_WRAP_5(ag_rp_75(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 74: VRO(AG_WRAP_5 *, ag_rp_76(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 75: VRO(AG_WRAP_5 *, ag_rp_77(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 76: VNO AG_WRAP_5(ag_rp_78(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 77: VNO AG_WRAP_5(ag_rp_79(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 78: VRO(AG_WRAP_4 *, ag_rp_80(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 79: VRO(AG_WRAP_4 *, ag_rp_81(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 80: VNO AG_WRAP_6(ag_rp_82(PCB_POINTER)); break;
    case 81: VRO(AG_WRAP_5 *, ag_rp_83(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 82: VRO(AG_WRAP_6 *, ag_rp_84(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 83: VNO AG_WRAP_5(ag_rp_85(PCB_POINTER, V(0,(long *)))); break;
    case 84: VNO AG_WRAP_5(ag_rp_86(PCB_POINTER, V(0,(double *)))); break;
    case 85: VNO AG_WRAP_5(ag_rp_87(PCB_POINTER, V(0,(int *)))); break;
    case 86: VNO AG_WRAP_4(ag_rp_88(PCB_POINTER, V(0,(int *)))); break;
    case 87: VRO(AG_WRAP_4 *, ag_rp_89(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 88: V(0,(double *)) = ag_rp_90(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 89: V(0,(double *)) = ag_rp_91(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 90: V(0,(double *)) = ag_rp_92(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 91: V(0,(double *)) = ag_rp_93(PCB_POINTER, V(0,(double *))); break;
    case 92: V(0,(double *)) = ag_rp_94(PCB_POINTER, V(1,(double *))); break;
    case 93: V(0,(double *)) = ag_rp_95(PCB_POINTER, V(0,(long *))); break;
    case 94: V(0,(double *)) = ag_rp_96(PCB_POINTER, V(0,(long *))); break;
    case 95: V(0,(double *)) = ag_rp_97(PCB_POINTER, V(0,(long *))); break;
    case 96: V(0,(long *)) = ag_rp_98(PCB_POINTER, V(1,(long *))); break;
    case 97: V(0,(long *)) = ag_rp_99(PCB_POINTER, V(1,(long *))); break;
    case 98: V(0,(double *)) = ag_rp_100(PCB_POINTER, V(0,(int *))); break;
    case 99: V(0,(double *)) = ag_rp_101(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 100: V(0,(long *)) = ag_rp_102(PCB_POINTER, V(0,(int *))); break;
    case 101: V(0,(long *)) = ag_rp_103(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 102: V(0,(long *)) = ag_rp_104(PCB_POINTER, V(0,(int *))); break;
    case 103: V(0,(long *)) = ag_rp_105(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 104: V(0,(long *)) = ag_rp_106(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 105: V(0,(long *)) = ag_rp_107(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 106: V(0,(long *)) = ag_rp_108(PCB_POINTER); break;
    case 107: V(0,(long *)) = ag_rp_109(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 108: V(0,(long *)) = ag_rp_110(PCB_POINTER); break;
    case 109: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 110: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(0,(int *))); break;
    case 111: V(0,(int *)) = ag_rp_113(PCB_POINTER, V(0,(int *))); break;
    case 112: VRO(AG_WRAP_5 *, ag_rp_114(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 113: VNO AG_WRAP_5(ag_rp_115(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 114: VNO AG_WRAP_4(ag_rp_116(PCB_POINTER)); break;
    case 115: VRO(AG_WRAP_4 *, ag_rp_117(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 116: V(0,(int *)) = ag_rp_118(PCB_POINTER); break;
    case 117: V(0,(int *)) = ag_rp_119(PCB_POINTER); break;
    case 118: V(0,(int *)) = ag_rp_120(PCB_POINTER); break;
    case 119: V(0,(int *)) = ag_rp_121(PCB_POINTER); break;
    case 120: V(0,(int *)) = ag_rp_122(PCB_POINTER); break;
    case 121: V(0,(int *)) = ag_rp_123(PCB_POINTER); break;
    case 122: V(0,(int *)) = ag_rp_124(PCB_POINTER); break;
    case 123: V(0,(int *)) = ag_rp_125(PCB_POINTER); break;
    case 124: V(0,(int *)) = ag_rp_126(PCB_POINTER); break;
    case 125: V(0,(int *)) = ag_rp_127(PCB_POINTER); break;
    case 126: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 127: V(0,(int *)) = ag_rp_129(PCB_POINTER, V(1,(int *))); break;
    case 128: V(0,(int *)) = ag_rp_130(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 129: V(0,(int *)) = ag_rp_131(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 130: V(0,(int *)) = ag_rp_132(PCB_POINTER, V(1,(int *))); break;
    case 131: V(0,(int *)) = ag_rp_133(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 132: V(0,(int *)) = ag_rp_134(PCB_POINTER, V(0,(int *))); break;
    case 133: V(0,(int *)) = ag_rp_135(PCB_POINTER, V(0,(int *))); break;
    case 134: V(0,(int *)) = ag_rp_136(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[256] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "syntax selection",
  "statement list",
  "eof",
  "loop condition",
  "statement",
  "DO WHILE",
  "optional expression",
  "",
  "\"script\"",
  "\"whileLoop\"",
  "\"doLoop\"",
  "\"statement\"",
  "\"forField\"",
  "open statement",
  "closed statement",
  "if condition",
  "else",
  "simple statement",
  "DO",
  "WHILE",
  "for initialization",
  "for field",
  "';'",
  "')'",
  "expression",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "\"do\"",
  "\"while\"",
  "FOR",
  "\"for\"",
  "'('",
  "",
  "\"if\"",
  "\"else\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "conditional expression",
  "lvalue",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "logical or expression",
  "'\\?'",
  "",
  "':'",
  "",
  "logical and expression",
  "\"||\"",
  "",
  "inclusive or expression",
  "\"&&\"",
  "",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "\"==\"",
  "\"!=\"",
  "shift expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "additive expression",
  "\"<<\"",
  "\">>\"",
  "multiplicative expression",
  "'+'",
  "'-'",
  "unary expression",
  "'*'",
  "'/'",
  "'%'",
  "factor",
  "'!'",
  "'~'",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "')'",
  "';'",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "\"do\"",
  "\"while\"",
  "\"for\"",
  "'('",
  "\"if\"",
  "\"else\"",
  "'}'",
  "'{'",
  "name",
  "\"dump\"",
  "','",
  "\"print\"",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'+'",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


