/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.
 Modified to support arrays, structs and functions.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In cll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146303
#include "dxi.h"
#endif

#ifndef DXI_H_1031146303
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 642, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    loopActive(0),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

void dxi_pcb_struct::forLoop
    (const FileLocation &condLoc, const FileLocation &incLoc)
{
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    if (loopPCB.mode == executeMode) {
      loopPCB.parseSyntax("forField", condLoc);    // Evaluate condition field
      Value result = dxi_value(&loopPCB);
      if (result.isDefined() && result.isFalse()) loopPCB.mode = breakMode;
    }
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // Mode could have been set to continueMode
    loopPCB.parseSyntax("forField", incLoc);
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

void dxi_pcb_struct::interruptLoop(Mode desiredMode) {
  if (mode != executeMode) return;
  if (loopActive == 0) reportError("No loop active");
  mode = desiredMode;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

Value dxi_pcb_struct::powm(Value &lvalue, const Value &exp) const {
  if (mode != executeMode) return Value();
  Value base = lvalue;
  lvalue = pow(base.deref(), exp);
  return lvalue.deref();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  int i;
  for (i = pcb.localList.size(); i--;) {
    d.value(pcb.localList[i]) = pcb.saveStack.pop();
  }
  return pcb.returnValue;
}

void dxi_pcb_struct::createArray(const AgString &n, const Value &x) {
  if (mode != executeMode) return;   // exit createArray
  dataset.value(n) = Value(x, 0);
}

// declare structure
void dxi_pcb_struct::defineStructure (
    const AgString &structureName,
    const AgStack<AgString> &fieldNames)
{
  if (mode != executeMode) return;
  AgDictionary<AgString> fieldDictionary(fieldNames.contents());
  Value structure(fieldDictionary);
  dataset.value(structureName) = structure;
}

// define instances of named structure
void dxi_pcb_struct::defineInstances (
    const AgString &structureName,
    const AgStack<AgString> &instanceNames)
{
  if (mode != executeMode) return;
  const AgDictionary<AgString> &fieldDictionary = dataset.value(structureName).getDictionary();
  for (int i = 0; i < instanceNames.size(); i++) {
    Instance instance(fieldDictionary);
    dataset.value(instanceNames[i]) = Value(instance);
  }
}

// define instances of an anonymous structure
void dxi_pcb_struct::defineInstances (
    const AgStack<AgString> &fieldNames,
    const AgStack<AgString> &instanceNames)
{
  if (mode != executeMode) return;
  AgDictionary<AgString> fieldDictionary(fieldNames.contents());
  for (int i = 0; i < instanceNames.size(); i++) {
    Instance instance(fieldDictionary);
    dataset.value(instanceNames[i]) = Value(instance);
  }
}

Value dxi_pcb_struct::callFunction(
  const AgString &name,
  AgStack<Value> &args)
{
  if (mode != executeMode) return Value();
  int nArgs = args.size();
  // Is there a variable with this name?
  int index = dataset.dictionary.lookup(name);
  if (index >= 0 && dataset[index].isFunction()) {
    LocalFunction &f = dataset[index].getFunction();
    if (nArgs == f.parameterList.size()) {
      int i;
      dxi_pcb_struct callPCB(dataset);
      // Save existing values for parameter variables
      for (i = 0; i < nArgs; i++) {
        Value &value = dataset.value(f.parameterList[i]);
        saveStack.push(value);
        value = args[i];                    // set variable to parameter value
      }

      const char *text = (const char *) f.text;
      callPCB.parseSyntax("script", FileLocation((const unsigned char *) text));

      // Restore local variables
      for (i = callPCB.localList.size(); i--;) {
        dataset.value(callPCB.localList[i]) = callPCB.saveStack.pop();
      }
      // Restore original values for parameter variables
      for (i = nArgs; i--;) {
        dataset.value(f.parameterList[i]) = saveStack.pop();
      }
      return callPCB.returnValue;
    }
  }
  // Not a locally defined function, try a library function
  return ::callFunction(idFunction(name, args.size()), args);
}


void dxi_pcb_struct::defineFunction(
    const AgString &n,
    const AgStack<AgString> &nameList,
    const AgString &text)
{
  mode = PCONTEXT(*this).mode;
  if (mode != executeMode) return;
  LocalFunction f(nameList, text);
  dataset.value(n) = Value(f);
}

void dxi_pcb_struct::localVariables(const AgStack<AgString> &list) {
  if (mode != executeMode) return;
  localList = list;
  int i;
  for (i = 0; i < list.size(); i++) {
    saveStack.push(dataset.value(list[i]));
  }
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 204, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return Value();
}

static inline Value ag_rp_5(PCB_DECL, Value &x) {
/* Line 208, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x;
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 212, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 215, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 218, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 221, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 224, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 247, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 249, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 253, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_14(PCB_DECL) {
/* Line 267, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.simpleLoop("doLoop");
}

static inline void ag_rp_15(PCB_DECL) {
/* Line 268, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_16(PCB_DECL, FileLocation c, FileLocation i) {
/* Line 269, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.forLoop(c, i);
}

static inline void ag_rp_17(PCB_DECL) {
/* Line 273, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.interruptLoop(breakMode);
}

static inline void ag_rp_18(PCB_DECL) {
/* Line 274, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.interruptLoop(continueMode);
}

static inline void ag_rp_19(PCB_DECL, Value &x) {
/* Line 275, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_20(PCB_DECL, AgString &n, Value &x) {
/* Line 278, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.createArray(n, x);
}

static inline void ag_rp_21(PCB_DECL) {
/* Line 292, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static inline void ag_rp_22(PCB_DECL, AgStack<AgString> &list) {
/* Line 296, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.localVariables(list);
}

static void ag_rp_23(PCB_DECL, Value &x) {
/* Line 302, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static inline void ag_rp_24(PCB_DECL) {
/* Line 305, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.mode = scanMode;
}

static inline FileLocation ag_rp_25(PCB_DECL) {
/* Line 308, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return CONTEXT;
}

static void ag_rp_26(PCB_DECL, Value &x) {
/* Line 313, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_27(PCB_DECL) {
/* Line 317, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_28(PCB_DECL, AgString &n) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_29(PCB_DECL, AgString &n) {
/* Line 325, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_30(PCB_DECL, Value &x) {
/* Line 329, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_31(PCB_DECL, Value &x) {
/* Line 330, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.print(x);
}

static void ag_rp_32(PCB_DECL, AgString &name, AgStack<AgString> &fields, AgStack<AgString> &instances) {
/* Line 334, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
      PCB.defineStructure(name, fields);
      PCB.defineInstances(name, instances);
    
}

static inline void ag_rp_33(PCB_DECL, AgString &name, AgStack<AgString> &fields) {
/* Line 338, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.defineStructure(name, fields);
}

static inline void ag_rp_34(PCB_DECL, AgStack<AgString> &fields, AgStack<AgString> &instances) {
/* Line 340, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.defineInstances(fields, instances);
}

static inline void ag_rp_35(PCB_DECL, AgString &name, AgStack<AgString> &instances) {
/* Line 342, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.defineInstances(name, instances);
}

static inline void ag_rp_36(PCB_DECL, AgString &n, AgStack<AgString> &list) {
/* Line 345, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.mode = scanMode;
}

static inline void ag_rp_37(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 346, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  PCB.defineFunction(n, list, text);
}

static AgString ag_rp_38(PCB_DECL) {
/* Line 349, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_39(PCB_DECL, AgString &n) {
/* Line 354, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_40(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 355, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return stack.push(n);
}

static inline Value ag_rp_41(PCB_DECL) {
/* Line 358, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return Value();
}

static inline Value ag_rp_42(PCB_DECL, Value &x) {
/* Line 364, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x;
}

static inline Value ag_rp_43(PCB_DECL, Value &lv, Value &x) {
/* Line 368, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv = x);
}

static inline Value ag_rp_44(PCB_DECL, Value &lv, Value &x) {
/* Line 369, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv += x);
}

static inline Value ag_rp_45(PCB_DECL, Value &lv, Value &x) {
/* Line 370, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv -= x);
}

static inline Value ag_rp_46(PCB_DECL, Value &lv, Value &x) {
/* Line 371, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv *= x);
}

static inline Value ag_rp_47(PCB_DECL, Value &lv, Value &x) {
/* Line 372, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv /= x);
}

static inline Value ag_rp_48(PCB_DECL, Value &lv, Value &x) {
/* Line 373, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv %= x);
}

static inline Value ag_rp_49(PCB_DECL, Value &lv, Value &x) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv |= x);
}

static inline Value ag_rp_50(PCB_DECL, Value &lv, Value &x) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv ^= x);
}

static inline Value ag_rp_51(PCB_DECL, Value &lv, Value &x) {
/* Line 376, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv &= x);
}

static inline Value ag_rp_52(PCB_DECL, Value &lv, Value &x) {
/* Line 377, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv <<= x);
}

static inline Value ag_rp_53(PCB_DECL, Value &lv, Value &x) {
/* Line 378, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv >>= x);
}

static inline Value ag_rp_54(PCB_DECL, Value &lv, Value &x) {
/* Line 379, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return PCB.powm(lv,x);
}

static void ag_rp_55(PCB_DECL, Value &c) {
/* Line 385, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
if (PCB.mode == executeMode && c.isFalse())  PCB.mode = elseMode;
}

static void ag_rp_56(PCB_DECL, Value &c, Value &x) {
/* Line 387, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
if (!PCB.changeMode(elseMode, executeMode))
         PCB.changeMode(executeMode, blockMode);
}

static inline Value ag_rp_57(PCB_DECL, Value &c, Value &x, Value &y) {
/* Line 390, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return PCB.mode = CONTEXT.mode, VALUE(c.isTrue() ? x : y);
}

static void ag_rp_58(PCB_DECL, Value &x) {
/* Line 396, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
if (PCB.mode == executeMode && x.isTrue()) PCB.mode = blockMode;
}

static inline Value ag_rp_59(PCB_DECL, Value &x, Value &y) {
/* Line 397, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? x : y);
}

static void ag_rp_60(PCB_DECL, Value &x) {
/* Line 402, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = blockMode;
}

static inline Value ag_rp_61(PCB_DECL, Value &x, Value &y) {
/* Line 403, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? y : x);
}

static inline Value ag_rp_62(PCB_DECL, Value &x, Value &y) {
/* Line 408, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x |= y);
}

static inline Value ag_rp_63(PCB_DECL, Value &x, Value &y) {
/* Line 412, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x ^= y);
}

static inline Value ag_rp_64(PCB_DECL, Value &x, Value &y) {
/* Line 416, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x &= y);
}

static inline Value ag_rp_65(PCB_DECL, Value &x, Value &y) {
/* Line 420, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_66(PCB_DECL, Value &x, Value &y) {
/* Line 421, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_67(PCB_DECL, Value &x, Value &y) {
/* Line 425, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_68(PCB_DECL, Value &x, Value &y) {
/* Line 426, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_69(PCB_DECL, Value &x, Value &y) {
/* Line 427, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_70(PCB_DECL, Value &x, Value &y) {
/* Line 428, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_71(PCB_DECL, Value &x, Value &y) {
/* Line 432, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x <<= y);
}

static inline Value ag_rp_72(PCB_DECL, Value &x, Value &y) {
/* Line 433, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x >>= y);
}

static inline Value ag_rp_73(PCB_DECL, Value &x, Value &y) {
/* Line 437, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x += y);
}

static inline Value ag_rp_74(PCB_DECL, Value &x, Value &y) {
/* Line 438, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x -= y);
}

static inline Value ag_rp_75(PCB_DECL, Value &x, Value &y) {
/* Line 442, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x *= y);
}

static inline Value ag_rp_76(PCB_DECL, Value &x, Value &y) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x /= y);
}

static inline Value ag_rp_77(PCB_DECL, Value &x, Value &y) {
/* Line 444, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x %= y);
}

static inline Value ag_rp_78(PCB_DECL, Value &x) {
/* Line 448, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_79(PCB_DECL, Value &x) {
/* Line 449, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_80(PCB_DECL, Value &x) {
/* Line 450, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_81(PCB_DECL, Value &x) {
/* Line 451, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(~x);
}

static inline Value ag_rp_82(PCB_DECL, Value &x, Value &y) {
/* Line 459, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_83(PCB_DECL, Value &x) {
/* Line 467, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x;
}

static inline Value ag_rp_84(PCB_DECL, Value &lv) {
/* Line 469, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_85(PCB_DECL, Value &lv) {
/* Line 470, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(++lv);
}

static inline Value ag_rp_86(PCB_DECL, Value &lv) {
/* Line 471, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(--lv);
}

static inline Value ag_rp_87(PCB_DECL, Value &lv) {
/* Line 472, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv++);
}

static inline Value ag_rp_88(PCB_DECL, Value &lv) {
/* Line 473, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv--);
}

static inline Value ag_rp_89(PCB_DECL, Value &x) {
/* Line 475, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_90(PCB_DECL, Value &x) {
/* Line 476, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_91(PCB_DECL, AgString &n) {
/* Line 479, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_92(PCB_DECL, Value &lv, Value &x) {
/* Line 480, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(lv[x]);
}

static inline Value ag_rp_93(PCB_DECL, Value &lv, AgString &n) {
/* Line 481, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(&lv.getMember(n));
}

static inline Value ag_rp_94(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 484, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return PCB.callFunction(n, args);
}

static inline AgStack<Value> ag_rp_95(PCB_DECL) {
/* Line 487, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_96(PCB_DECL, Value &x) {
/* Line 491, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_97(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 492, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_98(PCB_DECL, long x) {
/* Line 495, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_99(PCB_DECL, double x) {
/* Line 496, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_100(PCB_DECL, int x) {
/* Line 498, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_101(PCB_DECL, int c) {
/* Line 516, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_102(PCB_DECL, AgString &ns, int c) {
/* Line 517, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return ns.append(c);
}

static inline double ag_rp_103(PCB_DECL, double x, long e) {
/* Line 522, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_104(PCB_DECL, double x, long e) {
/* Line 523, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_105(PCB_DECL, double i, double f) {
/* Line 526, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return i+f;
}

static inline double ag_rp_106(PCB_DECL, double i) {
/* Line 527, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return i;
}

static inline double ag_rp_107(PCB_DECL, double f) {
/* Line 528, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return f;
}

static inline double ag_rp_108(PCB_DECL, long x) {
/* Line 531, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x;
}

static inline double ag_rp_109(PCB_DECL, long x) {
/* Line 532, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x;
}

static inline double ag_rp_110(PCB_DECL, long x) {
/* Line 533, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_111(PCB_DECL, long x) {
/* Line 536, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return (long)x;
}

static inline long ag_rp_112(PCB_DECL, long x) {
/* Line 537, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_113(PCB_DECL, int d) {
/* Line 540, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_114(PCB_DECL, int d, double f) {
/* Line 541, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_115(PCB_DECL, int d) {
/* Line 549, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return d-'0';
}

static inline long ag_rp_116(PCB_DECL, long x, int d) {
/* Line 550, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_117(PCB_DECL, int d) {
/* Line 553, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return d-'0';
}

static inline long ag_rp_118(PCB_DECL, long x, int d) {
/* Line 554, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_119(PCB_DECL, long x, int d) {
/* Line 557, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_120(PCB_DECL, long x, int d) {
/* Line 558, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_121(PCB_DECL) {
/* Line 561, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 0;
}

static inline long ag_rp_122(PCB_DECL, long n, int d) {
/* Line 562, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_123(PCB_DECL) {
/* Line 565, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 0;
}

static inline long ag_rp_124(PCB_DECL, long n, int d) {
/* Line 566, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_125(PCB_DECL, int x) {
/* Line 569, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return x - '0';
}

static inline int ag_rp_126(PCB_DECL, int d) {
/* Line 570, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_127(PCB_DECL, Value &s, Value &e) {
/* Line 576, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_128(PCB_DECL, AgString &b) {
/* Line 579, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_129(PCB_DECL) {
/* Line 582, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_130(PCB_DECL, AgString &s, int c) {
/* Line 583, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_131(PCB_DECL) {
/* Line 595, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\'';
}

static inline int ag_rp_132(PCB_DECL) {
/* Line 596, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '"';
}

static inline int ag_rp_133(PCB_DECL) {
/* Line 597, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\?';
}

static inline int ag_rp_134(PCB_DECL) {
/* Line 598, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\\';
}

static inline int ag_rp_135(PCB_DECL) {
/* Line 599, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\a';
}

static inline int ag_rp_136(PCB_DECL) {
/* Line 600, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\b';
}

static inline int ag_rp_137(PCB_DECL) {
/* Line 601, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\f';
}

static inline int ag_rp_138(PCB_DECL) {
/* Line 602, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\n';
}

static inline int ag_rp_139(PCB_DECL) {
/* Line 603, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\r';
}

static inline int ag_rp_140(PCB_DECL) {
/* Line 604, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\t';
}

static inline int ag_rp_141(PCB_DECL) {
/* Line 605, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return '\v';
}

static inline int ag_rp_142(PCB_DECL, int d) {
/* Line 611, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return d-'0';
}

static inline int ag_rp_143(PCB_DECL, int n, int d) {
/* Line 614, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_144(PCB_DECL, int n, int d) {
/* Line 617, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_145(PCB_DECL, int d) {
/* Line 620, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return d;
}

static inline int ag_rp_146(PCB_DECL, int n, int d) {
/* Line 621, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_147(PCB_DECL, int d) {
/* Line 624, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return d-'0';
}

static inline int ag_rp_148(PCB_DECL, int d) {
/* Line 625, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_149(PCB_DECL, int c) {
/* Line 634, W:/agdev/agex/packages/xidek/cll/combine/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0,
  5, 5, 5, 5, 5, 5, 5, 5, 4, 5, 4, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0,
  5, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 4, 0, 4, 0, 0, 4, 0, 0, 5, 5, 0, 0, 0, 0, 5, 7, 0, 0, 0, 0,
  0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 6, 6, 0, 0, 5, 0, 7, 7,
  0, 0, 5, 0, 0, 5, 0, 0, 0, 0, 0, 5, 0, 5, 5, 0, 7, 0, 7, 5, 0, 0, 0,
  0, 7, 0, 0, 0, 0, 7, 0, 0, 4, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  6,  7,  0,  0,  0,  0,  8,  9, 10,
    0, 11, 12, 13, 14,  0,  0, 15, 16, 17,  0,  0, 18,  0,  0,  0,  0,  0,
   19,  0, 20, 21, 22,  0, 14, 23, 24, 25,  0, 26, 27, 28, 29, 30, 31, 32,
   33, 34, 35, 36, 37, 38, 39,  0,  0, 40,  0, 41, 42, 43, 44, 45, 46, 47,
   48, 49, 50, 51, 52,  0, 53, 54, 55,  0, 56, 57,  0, 58, 59,  0, 60,  0,
   61,  0, 62,  0, 63, 64,  0, 65, 66, 67, 68,  0, 69, 70,  0, 71, 72,  0,
   73, 74, 75,  0, 76, 77, 78, 79,  0, 80, 81,  0, 82, 83, 84, 85, 86,  0,
   87, 88, 89, 90, 91, 92, 93,  0, 94, 95, 96, 97,  0, 98,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0, 99,100,  0,101,102,103,104,105,106,107,108,
    0,  0,109,110,111,112,  0,  0,  0,113,114,115,116,117,118,119,120,  0,
    0,121,122,123,124,  0,125,126,127,128,  0,  0,  0,  0,  0,129,130,131,
  132,133,134,135,136,137,138,139,  0,  0,  0,140,141,142,143,144,145,146,
  147
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0, 21,  0, 22,  0, 23,  0, 24,  0, 25,  0,210,  0,211,  0,212,
  0,216,  0,217,  0,218,  0,219,  0,220,  0,222,  0,223,  0,226,
  0,228,  0,229,  0,230,  0,268,  0,269,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 99,116,255, 47,100,102,115,119,255, 42,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 88,120,255,
   61,255, 60, 61,255, 61,255, 61, 62,255, 76,117,255,111,117,255, 70,255,
  114,255,111,117,255, 99,110,255,111,255, 97,114,255, 99,116,255, 76,255,
  101,255,108,255,105,255,104,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47,
   48, 60, 61, 62, 94, 97, 98, 99,100,101,102,105,108,112,114,115,119,124,
  255, 99,116,255,100,102,115,119,255, 42, 47,255, 88,120,255, 43, 45, 47,
   48,255, 42, 47,255, 88,120,255,111,117,255,111,117,255, 43, 45, 47, 48,
   97, 98, 99,100,102,105,112,114,115,119,255, 42, 47,255, 47,255, 42, 47,
  255, 88,120,255,111,117,255,111,117,255, 43, 45, 47, 48, 97, 98, 99,100,
  102,105,108,112,114,115,119,255, 88,120,255, 43, 45, 48,255, 88,120,255,
  111,117,255,111,117,255, 43, 45, 48, 97, 98, 99,100,102,105,112,114,115,
  119,255, 88,120,255,111,117,255,111,117,255, 43, 45, 48, 97, 98, 99,100,
  102,105,108,112,114,115,119,255, 34, 39, 63, 92, 97, 98,102,110,114,116,
  118,120,255, 92,255, 42, 47,255, 60, 61,255, 61, 62,255, 33, 38, 42, 47,
   60, 61, 62,124,255, 60, 61,255, 61, 62,255, 33, 38, 42, 60, 61, 62,124,
  255, 42, 47,255, 88,120,255, 43, 45, 47, 48,100,108,255, 88,120,255, 43,
   45, 48,100,108,255, 60, 61,255, 61, 62,255, 33, 38, 60, 61, 62,124,255,
   33, 38, 60, 61, 62,124,255, 33, 38, 61,124,255, 38,124,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 61,255, 60,
   61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 60,
   61, 62, 94,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,
  255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42,
   43, 45, 47, 60, 61, 62, 94,124,255,124,255, 42, 47,255, 88,120,255,111,
  117,255,111,117,255, 43, 45, 47, 48, 97, 98, 99,100,101,102,105,112,114,
  115,119,255,119,255, 60, 61,255, 61, 62,255, 33, 38, 42, 43, 45, 60, 61,
   62,124,255, 42, 47,255, 88,120,255, 60, 61,255, 61, 62,255,111,117,255,
  111,117,255, 33, 38, 42, 43, 45, 47, 48, 60, 61, 62, 97, 98, 99,100,101,
  102,105,112,114,115,119,124,255, 88,120,255,111,117,255,111,117,255, 43,
   45, 48, 97, 98, 99,100,101,102,105,112,114,115,119,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,7,2,7,4,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,0,
  4,0,4,1,0,4,0,4,0,1,4,7,7,4,6,7,4,7,4,6,4,2,7,4,7,7,4,2,4,7,7,4,7,2,4,
  7,4,6,4,2,4,2,4,2,4,0,0,4,3,3,2,2,2,2,2,2,2,3,2,3,7,7,7,2,7,2,7,2,7,7,
  2,2,2,4,7,7,4,7,7,2,7,4,0,0,4,0,0,4,3,3,2,2,4,0,0,4,0,0,4,5,7,4,7,7,4,
  3,3,2,2,7,7,7,2,2,7,7,7,7,7,4,0,0,4,2,4,0,0,4,0,0,4,5,7,4,7,7,4,3,3,2,
  2,7,7,7,2,2,7,7,7,7,7,7,4,0,0,4,3,3,2,4,0,0,4,5,7,4,7,7,4,3,3,2,7,7,7,
  2,2,7,7,7,7,7,4,0,0,4,5,7,4,7,7,4,3,3,2,7,7,7,2,2,7,7,7,7,7,7,4,0,0,0,
  0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,0,4,0,0,
  4,3,3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,3,3,2,7,7,4,0,0,4,0,
  0,4,3,3,2,3,2,3,4,3,3,3,3,3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,0,0,4,
  0,0,4,0,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,0,0,
  4,0,4,1,0,4,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,
  3,2,4,3,4,0,0,4,0,0,4,5,7,4,7,7,4,3,3,2,2,7,7,7,2,7,2,7,7,7,7,7,4,7,4,
  0,0,4,0,0,4,3,3,3,3,3,2,3,2,3,4,0,0,4,0,0,4,0,0,4,0,0,4,5,7,4,7,7,4,3,
  3,3,3,3,2,2,2,3,2,7,7,7,2,7,2,7,7,7,7,7,3,4,0,0,4,5,7,4,7,7,4,3,3,2,7,
  7,7,2,7,2,7,7,7,7,7,4
};

static const unsigned short ag_key_parm[] = {
    0,134,139,  0,  0,  6,  0,  0,  4,  8,  0,  2,  0,138,  0,246,239,  0,
  242,  0,265,234,  0,266,232,  0,267,233,  0,134,139,235,  0,164,163,  0,
  240,  0,256,253,  0,241,  0,255,257,  0,  4, 40,  0, 18, 30,  0,  8,  0,
   22,  0,  0, 36,  0, 24, 38,  0,  0,  0,  6, 34,  0,  0,  0,  0,  2,  0,
   20,  0,  0,  0,  0,  0,  0,  0,237,245,  0,251,236,  0,  0,  0,  0,  0,
    0,  0,250,  0,238, 16, 10, 12,  0, 28,  0, 26,  0, 32, 14,  0,  0,  0,
    0,  0,  6,  0,  4,  8,  0,  2,  0,134,139,  0,164,163,  0,266,267,  0,
    0,  0,134,139,  0,164,163,  0, 18, 30,  0, 22, 36,  0,266,267,  0,  0,
   16, 10, 12,  0,  0, 26, 32, 14, 34, 20,  0,134,139,  0,  0,  0,134,139,
    0,164,163,  0, 18, 30,  0, 22, 36,  0,266,267,  0,  0, 16, 10, 12,  0,
    0, 26, 24, 32, 14, 34, 20,  0,164,163,  0,266,267,  0,  0,164,163,  0,
   18, 30,  0, 22, 36,  0,266,267,  0, 16, 10, 12,  0,  0, 26, 32, 14, 34,
   20,  0,164,163,  0, 18, 30,  0, 22, 36,  0,266,267,  0, 16, 10, 12,  0,
    0, 26, 24, 32, 14, 34, 20,  0,177,176,178,179,180,181,182,183,184,185,
  186,191,  0,  0,  0,134,139,  0,256,253,  0,255,257,  0,251,246,265,  0,
    0,250,  0,245,  0,256,253,  0,255,257,  0,251,246,265,  0,250,  0,245,
    0,134,139,  0,164,163,  0,266,267,  0,  0, 40, 38,  0,164,163,  0,266,
  267,  0, 40, 38,  0,256,253,  0,255,257,  0,251,246,  0,250,  0,245,  0,
  251,246,253,250,255,245,  0,251,246,250,245,  0,246,245,  0,246,239,  0,
  242,  0,265,234,  0,266,232,  0,267,233,  0,134,139,235,  0,240,  0,256,
  253,  0,241,  0,255,257,  0,237,245,  0,251,236,  0,  0,  0,  0,  0,  0,
  250,  0,238,  0,  0,246,239,  0,242,  0,265,234,  0,266,232,  0,267,233,
    0,240,  0,256,253,  0,241,  0,255,257,  0,237,245,  0,251,236,  0,  0,
    0,  0,235,  0,250,  0,238,  0,  0,245,  0,134,139,  0,164,163,  0, 18,
   30,  0, 22, 36,  0,266,267,  0,  0, 16, 10, 12,  0, 28,  0, 26, 32, 14,
   34, 20,  0, 20,  0,256,253,  0,255,257,  0,251,246,265,266,267,  0,250,
    0,245,  0,134,139,  0,164,163,  0,256,253,  0,255,257,  0, 18, 30,  0,
   22, 36,  0,251,246,265,266,267,  0,  0,  0,250,  0, 16, 10, 12,  0, 28,
    0, 26, 32, 14, 34, 20,245,  0,164,163,  0, 18, 30,  0, 22, 36,  0,266,
  267,  0, 16, 10, 12,  0, 28,  0, 26, 32, 14, 34, 20,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 14, 19,  0,  1,  0,  6,  4, 27,  0, 36,  0,  0,  0,  0,
    0,  0, 18,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0, 36,  0,  0,  0,  0,  0, 41,  0, 64, 68,  0, 46, 72,  0, 79,  0,
   52,  0, 54, 84,  0, 93, 96,  0, 59,  0,114,121,  0,109, 64,  0,125,  0,
   70,  0, 72,  0, 74,  0, 76,  0,  0,  0,  0, 38, 40, 15, 20, 23, 26, 29,
   33, 38, 42, 43, 44, 46, 51, 56, 49, 75, 56, 91, 62, 98,103, 67, 78, 80,
    0,143,148,  0,129,135,109,156,  0,  0,  0,  0,  0,  0,  0,165,167,117,
  120,  0,  0,  0,  0,  0,  0,  0,  0,191,  0,194,196,  0,169,171,128,131,
  173,178,183,134,137,203,205,210,216,222,  0,  0,  0,  0,155,  0,  0,  0,
    0,  0,  0,  0,  0,249,  0,252,254,  0,227,229,160,163,231,236,241,166,
  169,261,263,268,273,279,285,  0,  0,  0,  0,290,292,188,  0,  0,  0,  0,
    0,316,  0,319,321,  0,294,296,195,298,303,308,198,201,328,330,335,341,
  347,  0,  0,  0,  0,  0,374,  0,377,379,  0,352,354,218,356,361,366,221,
  224,386,388,393,398,404,410,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,242,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,415,417,419,257,
  260,421,263,423,  0,  0,  0,  0,  0,  0,  0,425,427,429,275,431,278,433,
    0,  0,  0,  0,  0,  0,  0,435,437,289,292,439,445,  0,  0,  0,  0,449,
  451,302,453,459,  0,  0,  0,  0,  0,  0,  0,463,465,311,467,314,469,  0,
  471,473,475,477,479,481,  0,483,485,487,489,  0,491,493,  0,  0,  0,  0,
    0,  0,342,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,357,
    0,  0,  0,  0,  0,362,  0,  0,  0,  0,495,497,339,344,347,350,353,359,
  499,364,501,367,  0,  0,  0,  0,  0,  0,386,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,397,  0,  0,  0,  0,  0,402,  0,  0,  0,  0,503,505,383,388,
  391,394,507,399,509,404,511,407,  0,513,  0,  0,  0,  0,  0,  0,  0,  0,
  537,  0,544,546,  0,515,517,425,428,519,524,529,431,540,434,553,555,560,
  566,572,  0,577,  0,  0,  0,  0,  0,  0,  0,582,584,586,588,590,455,592,
  458,594,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,626,  0,
  633,635,  0,596,598,600,602,604,471,474,477,606,480,608,613,618,483,629,
  486,642,644,649,655,661,666,  0,  0,  0,  0,  0,690,  0,697,699,  0,668,
  670,512,672,677,682,515,693,518,706,708,713,719,725,  0
};

static const unsigned short ag_key_index[] = {
    7,  0, 13, 83,112,  0,  0, 13, 13,123,140,140,158,172,191,204,204,  0,
  227,255,  0,  0,255,266,  0,266,266,266,266,266,266,266,266,266,281,295,
    0,  0,305,123,123,123,123,281,191,191,191,191,317,317,317,324,331,336,
  336,336,370,336,410,423,410,  0,  0,158,158,158,123,158,172,437,437,  0,
    0,158,191,  0,123,158,158,204,437,  0,158,  0,  0,  0,  0,191,  0,  0,
    0,191,  0,204,453,191,204,158,  0,204,255,266,  0,  0,255,255,255,  0,
  266,  0,  0,281,281,158,  0,158,  0,  0,123,191,461,123,191,123,191,123,
  191,191,191,123,191,123,191,123,191,123,191,123,191,123,191,123,191,123,
  191,123,191,123,191,123,191,123,191,191,123,191,123,191,158,  0,123,191,
  123,191,123,191,123,191,123,191,123,191,123,191,123,191,123,191,123,191,
  123,191,123,191,123,191,123,191,489,  0,  0,  0,204,191,  0,191,  0,  0,
    0,  0,191,521,  0,  0,  0,204,  0,  0,191,191,317,317,317,317,317,317,
  317,317,324,324,331,336,336,191,  0,  0,191,191,  0,  0,  0,  0,  0,437,
    0,191,191,  0,140,204,  0,266,266,336,191,336,  0,370,  0,  0,  0,  0,
    0,123,191,  0,  0,  0,  0,191,  0,  0,227,204,  0,204
};

static const unsigned char ag_key_ends[] = {
111,76,111,111,112,0, 111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
47,0, 61,0, 61,0, 61,0, 61,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 111,111,112,0, 98,108,101,0, 
109,112,0, 108,115,101,0, 105,101,108,100,0, 110,99,116,105,111,110,0, 
102,0, 97,108,0, 103,0, 114,105,110,116,0, 101,116,117,114,110,0, 
114,105,112,116,0, 116,101,109,101,110,116,0, 117,99,116,0, 
111,111,112,0, 111,76,111,111,112,0, 111,114,70,105,101,108,100,0, 
114,105,112,116,0, 97,116,101,109,101,110,116,0, 
104,105,108,101,76,111,111,112,0, 43,0, 45,0, 43,0, 45,0, 
114,114,97,121,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 114,0, 110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 111,99,97,108,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
43,0, 45,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 111,99,97,108,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 42,0, 61,0, 124,0, 
43,0, 45,0, 111,117,98,108,101,0, 111,110,103,0, 43,0, 45,0, 
111,117,98,108,101,0, 111,110,103,0, 61,0, 38,0, 61,0, 124,0, 
61,0, 38,0, 61,0, 61,0, 61,0, 124,0, 61,0, 38,0, 61,0, 124,0, 
38,0, 124,0, 61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 
61,0, 124,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 114,114,97,121,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 114,0, 110,99,116,105,111,110,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 124,0, 43,0, 45,0, 114,114,97,121,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 114,0, 110,99,116,105,111,110,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned short ag_tcv[] = {
   15,197,197,197,197,197,197,197,197,196,143,196,196,196,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,196,263,168,197,
  197,262,249,193,221,208,260,258,227,259,270,261,161,198,198,198,198,198,
  198,198,160,160,243,209,252,231,254,244,197,199,199,199,199,147,199,200,
  200,200,200,200,200,200,200,200,200,200,200,200,200,200,200,200,200,200,
  200,214,190,213,248,200,197,199,199,199,199,147,199,200,200,200,200,200,
  200,200,200,200,200,200,200,200,200,200,200,200,200,200,200,225,247,224,
  264,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,197,
  197,197,197,197
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned short ag_tstt[] = {
196,143,139,134,25,24,23,22,21,0,1,201,202,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,193,190,168,161,160,147,143,0,
  141,142,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,193,190,168,161,160,147,143,138,
  0,136,137,
196,143,139,134,0,1,
25,24,23,22,21,0,7,8,9,10,11,12,203,204,205,206,207,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,193,190,168,161,160,147,0,
143,0,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,193,190,168,161,160,147,143,0,
138,0,
270,267,266,264,263,259,258,221,209,208,200,199,198,196,193,168,164,163,161,
  160,147,143,139,134,0,1,201,202,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,134,0,1,
  201,202,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,134,0,1,
  201,202,
221,196,143,139,134,0,1,201,202,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,220,219,218,217,216,
  212,211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,134,
  15,0,1,201,202,
270,267,266,264,263,259,258,221,209,208,200,199,198,193,168,164,163,161,160,
  147,0,2,3,4,5,6,19,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,
  113,114,118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,
  215,271,272,273,274,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,17,26,
  27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,59,
  61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,17,26,
  27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,59,
  61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
221,0,16,55,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,220,219,218,217,216,
  212,211,210,209,200,199,198,193,168,164,163,161,160,147,15,0,13,53,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,193,191,190,186,185,184,183,182,
  181,180,179,178,177,176,168,161,160,147,0,169,
198,161,160,0,156,
198,161,160,0,150,156,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,191,190,186,185,184,183,182,181,
  180,179,178,177,176,168,161,160,147,0,172,173,174,175,187,188,189,194,
265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,245,
  244,243,227,213,209,208,196,168,143,139,134,0,1,201,202,
270,147,0,
147,0,
199,198,161,160,147,0,166,
270,265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,
  245,244,243,227,213,209,208,198,196,161,160,147,143,139,134,0,
270,265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,
  245,244,243,227,213,209,208,198,196,161,160,147,143,139,134,0,156,
265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,245,
  244,243,227,213,209,208,196,143,139,134,0,1,201,202,
265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,245,
  244,243,227,213,209,208,196,143,139,134,0,1,201,202,
265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,245,
  244,243,227,213,209,208,196,143,139,134,0,1,201,202,
265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,245,
  244,243,227,213,209,208,200,199,196,147,143,139,134,0,1,201,202,
265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,247,246,245,
  244,243,227,213,209,208,200,199,196,147,143,139,134,0,1,201,202,
168,0,5,274,
270,269,268,267,266,264,263,259,258,221,209,208,200,199,198,196,193,168,164,
  163,161,160,147,143,139,134,0,1,201,202,
200,199,147,0,4,70,215,
200,199,147,0,4,70,215,
270,269,268,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,
  147,0,2,3,4,5,6,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,
  113,114,118,119,120,121,123,124,125,126,127,128,132,146,149,151,152,153,
  157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
265,0,122,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
262,261,260,0,115,116,117,
259,258,0,112,113,
257,256,0,109,110,
255,254,253,252,0,104,105,106,107,
251,250,0,101,102,
249,0,99,
248,0,97,
247,0,95,
270,267,266,265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,
  247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,227,
  225,224,221,214,213,209,208,200,199,198,196,161,160,147,143,139,134,0,1,
  201,202,
246,0,92,
221,0,55,
245,244,0,84,89,
270,267,266,242,241,240,239,238,237,236,235,234,233,232,231,214,0,45,71,72,
  73,74,75,76,77,78,79,80,81,82,124,125,129,
227,0,62,
209,208,0,35,36,
221,196,143,139,134,0,1,201,202,
200,199,196,147,143,139,134,0,1,201,202,
225,200,199,196,147,143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
200,199,196,147,143,139,134,0,1,201,202,
270,267,266,264,263,259,258,230,229,228,226,225,224,222,221,220,219,218,217,
  216,212,211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,
  134,0,1,201,202,
270,267,266,264,263,259,258,230,229,228,226,225,224,223,222,221,219,218,217,
  216,212,211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,
  134,15,0,1,201,202,
270,267,266,264,263,259,258,230,229,228,226,225,224,223,222,221,219,218,217,
  216,212,211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,
  134,15,0,1,201,202,
200,199,147,0,4,215,
225,200,199,147,0,4,59,215,
200,199,196,147,143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
200,199,147,0,4,215,
270,267,266,264,263,259,258,221,209,200,199,198,196,193,168,164,163,161,160,
  147,143,139,134,0,1,201,202,
209,196,143,139,134,0,1,201,202,
209,196,143,139,134,0,1,201,202,
270,267,266,264,263,259,258,230,229,228,226,225,224,222,221,219,218,217,216,
  212,211,210,209,200,199,198,193,168,164,163,161,160,147,0,14,
270,267,266,264,263,259,258,230,229,228,226,225,224,223,222,221,219,218,217,
  216,212,211,210,209,208,200,199,198,196,193,168,164,163,161,160,147,143,
  139,134,15,0,1,201,202,
221,0,55,
221,196,143,139,134,0,1,201,202,
209,0,35,
200,199,147,0,4,215,
227,209,0,35,62,
227,209,0,35,62,
270,267,266,264,263,259,258,221,209,200,199,198,193,168,164,163,161,160,147,
  0,2,3,4,5,6,19,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,
  114,118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,
  271,272,273,274,
209,0,35,
209,0,35,
227,209,0,35,62,
270,267,266,264,263,259,258,221,209,200,199,198,193,168,164,163,161,160,147,
  0,2,3,4,5,6,34,37,55,56,64,69,70,83,88,91,94,96,98,100,103,108,111,112,
  113,114,118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,
  215,271,272,273,274,
221,0,55,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,17,26,
  27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,59,
  61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
218,0,18,50,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,
  272,273,274,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,17,26,
  27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,59,
  61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
200,199,196,147,143,139,134,0,1,201,202,
200,199,147,0,4,54,215,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,15,0,14,
270,264,263,262,261,260,259,258,254,252,249,248,247,244,243,231,227,225,224,
  221,214,213,209,208,200,199,198,197,196,193,191,190,186,185,184,183,182,
  181,180,179,178,177,176,168,161,160,147,0,170,172,173,174,175,187,188,
  189,
198,161,160,0,150,156,
198,161,0,
199,198,161,160,147,0,192,
198,161,0,
198,161,0,
199,198,161,160,147,0,192,
193,0,
198,161,160,0,150,156,
259,258,198,161,160,0,148,154,
259,258,198,161,160,0,148,154,
270,214,0,45,129,
270,214,0,45,129,
208,196,143,139,134,0,1,201,202,
208,0,36,
208,196,143,139,134,0,1,201,202,
208,0,36,
227,208,0,36,62,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,214,0,45,124,125,129,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,112,113,114,118,119,120,121,123,124,125,126,132,146,149,
  151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,111,112,113,114,118,119,120,121,123,124,125,126,132,146,
  149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,111,112,113,114,118,119,120,121,123,124,125,126,132,146,
  149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,108,111,112,113,114,118,119,120,121,123,124,125,126,132,
  146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,108,111,112,113,114,118,119,120,121,123,124,125,126,132,
  146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,103,108,111,112,113,114,118,119,120,121,123,124,125,126,
  132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,103,108,111,112,113,114,118,119,120,121,123,124,125,126,
  132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,103,108,111,112,113,114,118,119,120,121,123,124,125,126,
  132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,103,108,111,112,113,114,118,119,120,121,123,124,125,126,
  132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,100,103,108,111,112,113,114,118,119,120,121,123,124,125,
  126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,100,103,108,111,112,113,114,118,119,120,121,123,124,125,
  126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,98,100,103,108,111,112,113,114,118,119,120,121,123,124,
  125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,96,98,100,103,108,111,112,113,114,118,119,120,121,123,124,
  125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,94,96,98,100,103,108,111,112,113,114,118,119,120,121,123,
  124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,
  93,
270,267,266,264,263,259,258,221,208,200,199,198,193,168,164,163,161,160,147,
  0,2,3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,130,131,132,146,149,151,152,153,157,165,
  215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,
  90,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,
  85,
200,199,196,147,143,139,134,0,1,201,202,
200,199,147,0,4,215,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,
  272,273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
270,267,266,265,264,263,262,261,260,259,258,257,256,255,254,253,252,251,250,
  249,248,247,246,245,244,243,230,229,228,227,226,225,224,223,222,221,219,
  218,217,216,213,212,211,210,209,208,200,199,198,196,193,168,164,163,161,
  160,147,143,139,134,15,0,1,201,202,
221,0,55,
200,199,147,0,4,54,215,
225,200,199,147,0,4,54,59,215,
270,267,266,264,263,259,258,230,229,228,226,225,224,222,221,219,218,217,216,
  212,211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,17,
  26,27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,
  59,60,61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,
  114,118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,
  271,272,273,274,
270,267,266,264,263,259,258,221,209,200,199,198,193,168,164,163,161,160,147,
  0,2,3,4,5,6,19,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,
  114,118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,
  271,272,273,274,
214,0,45,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
200,199,147,0,4,215,
209,0,35,
227,0,62,
209,0,35,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,
  272,273,274,
223,0,29,58,
221,0,16,55,
227,208,0,36,62,
227,209,0,35,62,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,15,0,2,3,4,5,6,17,
  26,27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,
  59,61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,
  272,273,274,
198,161,160,0,155,
198,161,160,0,155,
270,267,266,221,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,55,70,
  121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,221,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,55,70,
  121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
262,261,260,0,115,116,117,
262,261,260,0,115,116,117,
259,258,0,112,113,
259,258,0,112,113,
257,256,0,109,110,
257,256,0,109,110,
257,256,0,109,110,
257,256,0,109,110,
255,254,253,252,0,104,105,106,107,
255,254,253,252,0,104,105,106,107,
251,250,0,101,102,
249,0,99,
248,0,97,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,91,94,96,98,100,103,108,111,112,113,114,118,119,120,121,
  123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
227,0,62,
208,0,36,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,70,88,91,94,96,98,100,103,108,111,112,113,114,118,119,120,
  121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,274,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,
  272,273,274,
227,213,0,46,62,
200,199,147,0,4,54,215,
227,224,0,60,62,
227,0,62,
200,199,147,0,4,54,215,
270,267,266,264,263,259,258,230,229,228,226,225,224,223,222,221,219,218,217,
  216,212,211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,
  134,15,0,1,201,202,
209,0,35,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,37,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,
  118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,
  272,273,274,
270,267,266,264,263,259,258,221,208,200,199,198,193,168,164,163,161,160,147,
  0,2,3,4,5,6,34,37,55,56,64,69,70,83,88,91,94,96,98,100,103,108,111,112,
  113,114,118,119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,
  215,271,272,273,274,
227,208,0,36,62,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,196,193,168,164,163,161,160,147,143,139,134,0,1,
  201,202,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,26,27,
  28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,59,61,
  63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,119,
  120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,
  274,
200,199,147,0,4,215,
198,161,160,0,156,
198,161,160,0,156,
247,0,95,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,64,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,
246,0,92,
243,227,0,62,86,
270,267,266,265,262,261,260,259,258,257,256,255,254,253,252,251,250,249,248,
  247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,227,
  214,213,209,208,196,143,139,134,0,1,201,202,
227,208,0,36,62,
200,199,147,0,4,54,215,
227,224,0,60,62,
227,213,0,46,62,
208,0,36,
270,267,266,264,263,259,258,221,200,199,198,196,193,168,164,163,161,160,147,
  143,139,134,0,1,201,202,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,
  87,
225,0,67,
227,0,62,
200,199,147,0,4,54,215,
209,0,35,
270,267,266,264,263,259,258,221,200,199,198,193,168,164,163,161,160,147,0,2,
  3,4,5,6,55,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,119,
  120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,273,
  274,
225,0,59,
227,0,62,
270,267,266,264,263,259,258,230,229,228,226,225,224,222,221,220,219,218,217,
  216,212,211,210,209,200,199,198,193,168,164,163,161,160,147,0,13,53,68,
270,267,266,264,263,259,258,230,229,228,226,225,224,222,221,219,218,217,216,
  212,211,210,209,200,199,198,193,168,164,163,161,160,147,0,14,
224,0,60,
270,267,266,264,263,259,258,230,229,228,226,225,222,221,219,218,217,216,212,
  211,210,209,200,199,198,193,168,164,163,161,160,147,0,2,3,4,5,6,17,26,
  27,28,30,31,32,33,35,37,38,39,40,41,42,43,44,47,48,49,50,51,52,55,57,59,
  61,63,64,65,66,69,70,83,88,91,94,96,98,100,103,108,111,112,113,114,118,
  119,120,121,123,124,125,126,132,146,149,151,152,153,157,165,215,271,272,
  273,274,

};


static unsigned const char ag_astt[7153] = {
  1,1,1,1,8,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,2,2,2,2,2,7,1,1,1,1,1,0,1,
  1,1,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,5,3,7,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,1,1,1,1,1,1,1,4,4,2,2,2,1,1,2,2,2,2,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,2,7,2,2,1,1,2,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,
  2,2,1,1,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,
  1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,5,7,
  1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,7,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,
  7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,10,5,10,2,4,5,5,5,7,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,10,5,10,10,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,
  1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,
  3,1,5,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,
  2,7,2,1,1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,1,1,1,1,5,1,1,1,1,5,1,1,1,1,1,1,
  5,1,1,1,1,1,1,5,1,1,1,5,1,1,5,1,1,5,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,
  1,1,7,1,1,3,1,5,1,1,4,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,1,1,5,1,1,1,7,2,2,5,1,1,1,1,7,1,1,3,5,5,1,5,1,1,1,
  7,1,1,3,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,2,2,2,7,
  1,1,1,2,2,2,7,1,1,1,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,5,1,1,1,1,7,
  1,1,3,1,7,3,2,2,2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,2,1,7,2,1,1,7,3,1,1,1,1,1,1,1,1,1,5,2,2,2,1,
  1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,
  2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  2,7,2,2,1,1,2,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,
  2,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,
  2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,4,1,1,1,1,
  4,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,7,2,1,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
  1,4,1,2,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  7,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  7,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,7,1,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,
  2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,
  7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,2,2,2,
  7,2,1,1,1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,1,3,2,2,1,
  1,1,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  1,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,7,2,
  1,5,1,1,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,1,
  7,2,1,1,1,7,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,2,2,7,2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,1,3,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,2,2,2,7,1,2,2,2,7,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,
  2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,
  1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,1,1,4,1,
  1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,4,1,1,4,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,1,1,5,1,1,7,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,7,2,1,2,2,2,7,2,1,1,1,
  1,7,1,1,1,4,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,5,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,
  1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  1,1,1,1,1,2,2,2,7,2,1,10,10,10,4,2,10,10,10,4,2,1,4,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,4,1,1,1,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,1,
  7,1,1,2,2,2,7,2,1,1,1,1,7,1,1,1,1,7,1,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,4,7,1,1,4,
  1,2,2,2,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,
  1,1,4,1,8,8,8,8,8,8,8,8,8,8,8,8,5,8,8,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,7,1,1,1,8,8,8,8,8,8,8,8,8,8,8,8,5,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,7,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  2,4,2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,1,3,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,
  6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,
  7,8,
380,380,1,2,382,380,
10,9,8,7,6,4,18,17,16,15,14,0,13,12,11,10,9,
147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,149,
150,6,
142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,144,
145,8,
381,381,381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,
  381,381,3,1,2,9,3,3,387,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,10,3,3,386,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,11,3,3,385,
381,3,3,1,2,12,3,3,384,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,381,13,
  3,3,383,
21,32,33,39,40,41,42,35,60,60,151,151,171,22,19,181,181,177,171,151,14,136,
  137,58,34,139,62,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,15,136,137,58,34,139,4,4,4,93,4,19,20,91,
  4,90,4,89,88,87,86,85,84,83,4,19,20,81,81,38,92,79,75,74,90,72,71,90,60,
  59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,16,136,137,58,34,139,94,94,94,93,94,19,20,
  91,94,90,94,89,88,87,86,85,84,83,94,19,20,81,81,38,92,79,75,74,90,72,71,
  90,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,181,56,31,30,29,23,
35,17,96,95,
99,99,99,99,99,99,99,99,99,99,99,99,99,99,97,99,99,99,99,99,99,99,99,99,99,
  99,99,99,99,99,99,99,99,37,18,99,98,
188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,19,100,
176,176,176,160,176,
101,101,101,21,158,101,
107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,
  107,107,107,107,107,107,107,107,107,107,103,102,205,204,203,202,201,200,
  199,198,197,196,195,107,107,107,107,22,107,107,107,106,105,104,107,107,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,381,3,1,2,23,3,3,454,
108,109,24,
110,153,
184,183,183,183,184,170,182,
161,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
  169,169,169,169,169,169,169,178,169,178,175,161,169,169,169,27,
159,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,168,
  168,168,168,168,168,168,168,172,168,172,172,159,168,168,168,28,172,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,3,1,2,29,3,3,453,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,3,1,2,30,3,3,452,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,3,1,2,31,3,3,451,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,3,381,3,1,2,32,3,3,447,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,3,381,3,1,2,33,3,3,446,
19,138,186,23,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,
  381,381,381,381,3,1,2,35,3,3,401,
151,151,151,36,128,111,56,
151,151,151,37,128,112,56,
21,113,115,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,38,
  136,137,58,34,139,117,38,117,117,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,116,114,34,25,24,28,20,27,26,181,56,31,30,29,
  23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,39,3,3,444,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,40,3,3,443,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,41,3,3,439,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,42,3,3,438,
118,116,119,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,44,136,137,58,
  34,139,38,120,47,46,115,115,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  181,56,31,30,29,23,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,45,136,137,58,
  34,139,38,120,47,46,114,114,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  181,56,31,30,29,23,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,46,136,137,58,
  34,139,38,120,47,46,113,113,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  181,56,31,30,29,23,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,47,136,137,58,
  34,139,38,120,47,46,112,112,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  181,56,31,30,29,23,
121,123,125,104,126,124,122,
41,42,101,128,127,
129,131,96,132,130,
133,135,137,139,93,140,138,136,134,
141,143,91,144,142,
145,89,146,
147,87,148,
149,84,150,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,152,152,152,3,152,152,152,3,1,2,56,3,3,395,
151,81,152,
35,128,153,
154,156,77,157,155,
158,32,33,162,164,166,168,170,172,174,176,178,180,182,184,160,120,161,185,
  183,181,179,177,175,173,171,169,167,165,163,123,124,159,
186,61,187,
80,188,62,5,5,
381,3,3,1,2,63,3,3,399,
381,381,3,381,3,1,2,64,3,3,410,
381,381,381,3,381,3,1,2,65,3,3,409,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,66,3,3,408,
381,381,3,381,3,1,2,67,3,3,406,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,68,
  3,3,405,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,381,
  69,3,3,398,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,381,
  70,3,3,397,
151,151,151,71,189,56,
68,151,151,151,72,191,190,56,
381,381,3,381,3,1,2,73,3,3,396,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,74,136,137,58,
  34,139,38,49,49,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
151,151,151,75,47,56,
381,381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,
  381,3,1,2,76,3,3,392,
381,3,3,1,2,77,3,3,391,
381,3,3,1,2,78,3,3,390,
192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,
  192,192,192,192,192,192,192,192,192,192,192,192,192,192,79,192,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,
  381,80,3,3,389,
35,81,193,
381,3,3,1,2,82,3,3,402,
80,83,31,
151,151,151,84,194,56,
186,80,85,29,195,
186,80,86,28,196,
21,32,33,39,40,41,42,35,60,151,151,171,22,19,181,181,177,171,151,87,136,137,
  58,34,139,197,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
80,88,26,
80,89,25,
186,80,90,22,187,
21,32,33,39,40,41,42,35,41,151,151,171,22,19,181,181,177,171,151,91,136,137,
  58,34,139,199,198,38,43,198,198,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
35,92,200,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,93,136,137,58,34,139,15,15,201,93,201,19,
  20,91,201,90,201,89,88,87,86,85,84,83,201,19,20,81,81,38,92,79,75,74,90,
  72,71,90,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,
  43,34,25,24,28,20,27,26,181,56,31,30,29,23,
69,94,202,36,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,95,136,137,58,
  34,139,203,38,203,203,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,
  44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,96,136,137,58,34,139,2,2,2,93,2,19,20,91,
  2,90,2,89,88,87,86,85,84,83,2,19,20,81,81,38,92,79,75,74,90,72,71,90,60,
  59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,181,56,31,30,29,23,
381,381,3,381,3,1,2,97,3,3,400,
151,151,151,98,58,204,56,
205,205,205,205,205,205,205,205,205,205,205,205,205,205,205,205,205,205,205,
  205,205,205,205,205,205,205,205,205,205,205,205,205,205,99,205,
189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,189,
  189,189,189,189,189,189,189,189,189,189,189,103,102,205,204,203,202,201,
  200,199,198,197,196,195,187,189,189,189,100,189,189,189,189,106,105,104,
  189,
101,101,101,166,167,101,
209,209,102,
215,214,214,214,215,103,212,
211,211,207,
210,210,206,
215,214,214,214,215,194,213,
216,107,
101,101,101,157,156,101,
206,207,207,207,207,109,155,207,
206,207,207,207,207,110,154,207,
158,160,122,161,159,
158,160,121,161,159,
381,3,3,1,2,113,3,3,449,
188,114,208,
381,3,3,1,2,115,3,3,448,
188,116,209,
186,188,117,118,187,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,118,3,3,445,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,119,136,137,
  58,34,139,38,120,47,46,117,117,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,181,56,31,30,29,23,
158,32,33,160,120,161,123,124,159,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,121,3,3,442,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,122,136,137,
  58,34,139,38,120,47,46,110,110,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,123,3,3,441,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,124,136,137,
  58,34,139,38,120,47,46,109,109,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,125,3,3,440,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,126,136,137,
  58,34,139,38,120,47,46,108,108,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,181,56,31,30,29,23,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,127,136,137,
  58,34,139,38,120,210,47,46,210,210,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,128,136,137,
  58,34,139,38,120,211,47,46,211,211,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,129,3,3,437,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,130,136,137,
  58,34,139,38,120,212,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,131,3,3,436,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,132,136,137,
  58,34,139,38,120,213,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,133,3,3,435,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,134,136,137,
  58,34,139,38,120,214,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,135,3,3,434,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,136,136,137,
  58,34,139,38,120,215,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,137,3,3,433,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,138,136,137,
  58,34,139,38,120,216,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,139,3,3,432,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,140,136,137,
  58,34,139,38,120,217,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,141,3,3,431,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,142,136,137,
  58,34,139,38,120,218,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,143,3,3,430,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,144,136,137,
  58,34,139,38,120,219,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,145,3,3,429,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,146,136,137,
  58,34,139,38,120,220,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,
  24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,147,3,3,428,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,148,136,137,
  58,34,139,38,120,221,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,149,3,3,427,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,150,136,137,
  58,34,139,38,120,222,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,
  34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,151,3,3,426,
85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,152,223,
21,32,33,39,40,41,42,35,132,151,151,171,22,19,181,181,177,171,151,153,136,
  137,58,34,139,38,134,134,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,225,224,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,154,3,3,425,
82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,155,226,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,156,3,3,424,
78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,157,227,
381,381,3,381,3,1,2,158,3,3,450,
151,151,151,159,130,56,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,160,3,3,394,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,161,136,137,
  58,34,139,228,38,228,228,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,162,3,3,422,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,163,136,137,
  58,34,139,38,76,76,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,164,3,3,421,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,165,136,137,
  58,34,139,38,75,75,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,166,3,3,420,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,167,136,137,
  58,34,139,38,74,74,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,168,3,3,419,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,169,136,137,
  58,34,139,38,73,73,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,170,3,3,418,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,171,136,137,
  58,34,139,38,72,72,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,172,3,3,417,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,173,136,137,
  58,34,139,38,71,71,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,174,3,3,416,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,175,136,137,
  58,34,139,38,70,70,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,176,3,3,415,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,177,136,137,
  58,34,139,38,69,69,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,178,3,3,414,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,179,136,137,
  58,34,139,38,68,68,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,180,3,3,413,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,181,136,137,
  58,34,139,38,67,67,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,182,3,3,412,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,183,136,137,
  58,34,139,38,66,66,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,184,3,3,411,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,185,136,137,
  58,34,139,38,65,65,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,186,3,3,407,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,187,136,137,
  58,34,139,38,63,63,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,
  381,381,3,1,2,381,188,3,3,388,
35,189,229,
151,151,151,190,58,230,56,
68,151,151,151,191,58,231,232,56,
21,32,33,39,40,41,42,64,65,66,67,68,233,82,35,63,69,70,73,76,77,78,80,151,
  151,171,22,19,181,181,177,171,151,192,136,137,58,34,139,12,12,12,93,12,
  19,20,91,12,90,12,89,88,87,86,85,84,83,12,19,20,81,81,38,92,79,46,75,74,
  90,72,71,90,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,
  36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,35,60,151,151,171,22,19,181,181,177,171,151,193,136,
  137,58,34,139,234,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
160,194,235,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,195,136,137,
  58,34,139,38,50,50,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
151,151,151,196,48,56,
80,197,27,
186,42,187,
80,199,236,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,200,136,137,
  58,34,139,237,38,237,237,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
238,14,239,45,
35,202,3,95,
186,188,203,39,187,
186,80,204,38,240,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,1,205,136,137,58,34,139,12,12,12,93,12,19,
  20,91,12,90,12,89,88,87,86,85,84,83,12,19,20,81,81,38,92,79,75,74,90,72,
  71,90,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,
  34,25,24,28,20,27,26,181,56,31,30,29,23,
173,173,173,206,241,
173,173,173,207,242,
21,32,33,35,151,151,171,22,19,181,181,177,171,151,208,136,137,58,34,139,38,
  120,127,127,37,36,127,34,25,24,28,20,27,26,181,56,31,30,29,23,
21,32,33,35,151,151,171,22,19,181,181,177,171,151,209,136,137,58,34,139,38,
  120,126,126,37,36,126,34,25,24,28,20,27,26,181,56,31,30,29,23,
121,123,125,106,126,124,122,
121,123,125,105,126,124,122,
41,42,103,128,127,
41,42,102,128,127,
129,131,100,132,130,
129,131,99,132,130,
129,131,98,132,130,
129,131,97,132,130,
133,135,137,139,95,140,138,136,134,
133,135,137,139,94,140,138,136,134,
141,143,92,144,142,
145,90,146,
147,88,148,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,223,136,137,
  58,34,139,38,120,243,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,
  43,34,25,24,28,20,27,26,181,56,31,30,29,23,
186,133,244,
188,225,131,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,226,136,137,
  58,34,139,38,120,245,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,
  36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,227,136,137,
  58,34,139,246,38,246,246,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
186,247,228,129,187,
151,151,151,229,58,248,56,
186,233,230,249,240,
186,54,240,
151,151,151,232,58,250,56,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,381,
  233,3,3,404,
80,234,40,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,235,136,137,
  58,34,139,251,38,251,251,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
21,32,33,39,40,41,42,35,41,151,151,171,22,19,181,181,177,171,151,236,136,
  137,58,34,139,252,198,38,43,198,198,60,59,57,55,54,53,52,51,50,49,48,47,
  46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
186,188,237,44,187,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,1,2,238,3,3,403,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,239,136,137,58,34,139,16,17,93,17,19,20,
  91,17,90,17,89,88,87,86,85,84,83,17,19,20,81,81,38,92,79,75,74,90,72,71,
  90,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,181,56,31,30,29,23,
151,151,151,240,59,56,
174,174,174,165,174,
174,174,174,164,174,
149,86,150,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,244,136,137,
  58,34,139,38,135,135,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
151,83,152,
253,186,246,187,254,
381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,381,
  381,381,381,381,3,3,1,2,247,3,3,393,
186,188,248,255,240,
151,151,151,249,58,256,56,
186,233,250,257,240,
186,247,251,258,187,
188,252,21,
381,381,381,381,381,381,381,381,381,381,381,3,381,381,381,381,381,381,381,3,
  1,2,253,3,3,423,
79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,79,254,259,
55,255,260,
186,53,240,
151,151,151,52,58,261,56,
80,258,30,
21,32,33,39,40,41,42,35,151,151,171,22,19,181,181,177,171,151,259,136,137,
  58,34,139,38,80,120,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,181,56,31,30,29,23,
68,260,262,
186,51,240,
263,263,263,263,263,263,263,263,263,263,263,263,37,263,263,97,263,263,263,
  263,263,263,263,263,263,263,263,263,263,263,263,263,263,263,262,263,98,
  264,
265,265,265,265,265,265,265,265,265,265,265,265,11,265,265,265,265,265,265,
  265,265,265,265,265,265,265,265,265,265,265,265,265,265,263,265,
233,264,56,
21,32,33,39,40,41,42,64,65,66,67,68,82,35,63,69,70,73,76,77,78,80,151,151,
  171,22,19,181,181,177,171,151,57,136,137,58,34,139,12,12,12,93,12,19,20,
  91,12,90,12,89,88,87,86,85,84,83,12,19,20,81,81,38,92,79,75,74,90,72,71,
  90,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,181,56,31,30,29,23,

};


static const unsigned short ag_sbt[] = {
     0,  13,  52,  92,  98, 115, 151, 153, 190, 192, 220, 260, 300, 309,
   351, 417, 522, 627, 631, 668, 717, 722, 728, 783, 817, 820, 822, 829,
   864, 900, 933, 966, 999,1035,1071,1075,1105,1112,1119,1186,1212,1238,
  1264,1290,1293,1343,1393,1443,1493,1500,1505,1510,1519,1524,1527,1530,
  1533,1591,1594,1597,1602,1635,1638,1643,1652,1663,1675,1701,1712,1754,
  1797,1840,1846,1854,1865,1927,1933,1960,1969,1978,2013,2057,2060,2069,
  2072,2078,2083,2088,2153,2156,2159,2164,2230,2233,2338,2342,2405,2510,
  2521,2528,2563,2619,2625,2628,2635,2638,2641,2648,2650,2656,2664,2672,
  2677,2682,2691,2694,2703,2706,2711,2737,2787,2796,2822,2872,2898,2948,
  2974,3024,3075,3126,3152,3204,3230,3282,3308,3361,3387,3440,3466,3519,
  3545,3598,3624,3678,3704,3758,3784,3839,3865,3921,3947,4004,4030,4050,
  4115,4141,4161,4187,4207,4218,4224,4250,4313,4339,4401,4427,4489,4515,
  4577,4603,4665,4691,4753,4779,4841,4867,4929,4955,5017,5043,5105,5131,
  5193,5219,5281,5307,5369,5395,5457,5522,5525,5532,5541,5648,5713,5716,
  5778,5784,5787,5790,5793,5856,5860,5864,5869,5874,5980,5985,5990,6030,
  6070,6077,6084,6089,6094,6099,6104,6109,6114,6123,6132,6137,6140,6143,
  6201,6204,6207,6266,6329,6334,6341,6346,6349,6356,6399,6402,6465,6531,
  6536,6576,6680,6686,6691,6696,6699,6761,6764,6769,6818,6823,6830,6835,
  6840,6843,6869,6889,6892,6895,6902,6905,6966,6969,6972,7010,7045,7048,
  7153
};


static const unsigned short ag_sbe[] = {
     9,  49,  89,  96, 103, 150, 152, 189, 191, 216, 256, 296, 305, 347,
   371, 449, 554, 628, 665, 715, 720, 725, 774, 813, 819, 821, 827, 863,
   898, 929, 962, 995,1031,1067,1072,1101,1108,1115,1139,1208,1234,1260,
  1286,1291,1311,1361,1411,1461,1496,1502,1507,1514,1521,1525,1528,1531,
  1587,1592,1595,1599,1618,1636,1640,1648,1659,1671,1697,1708,1750,1793,
  1836,1843,1850,1861,1883,1930,1956,1965,1974,2011,2053,2058,2065,2070,
  2075,2080,2085,2107,2154,2157,2161,2183,2231,2265,2339,2360,2437,2517,
  2524,2561,2610,2622,2627,2633,2637,2640,2646,2649,2653,2661,2669,2674,
  2679,2687,2692,2699,2704,2708,2733,2755,2791,2818,2840,2894,2916,2970,
  2992,3042,3093,3148,3170,3226,3248,3304,3326,3383,3405,3462,3484,3541,
  3563,3620,3642,3700,3722,3780,3802,3861,3883,3943,3965,4026,4048,4069,
  4137,4159,4183,4205,4214,4221,4246,4268,4335,4357,4423,4445,4511,4533,
  4599,4621,4687,4709,4775,4797,4863,4885,4951,4973,5039,5061,5127,5149,
  5215,5237,5303,5325,5391,5413,5518,5523,5528,5536,5574,5667,5714,5734,
  5781,5785,5788,5791,5811,5857,5861,5866,5871,5907,5983,5988,6004,6044,
  6073,6080,6086,6091,6096,6101,6106,6111,6118,6127,6134,6138,6141,6161,
  6202,6205,6225,6284,6331,6337,6343,6347,6352,6395,6400,6420,6484,6533,
  6572,6608,6683,6689,6694,6697,6717,6762,6766,6814,6820,6826,6832,6837,
  6841,6865,6887,6890,6893,6898,6903,6923,6967,6970,7006,7043,7046,7080,7153
};


static const unsigned char ag_fl[] = {
  2,4,3,4,2,3,1,1,1,1,1,0,2,1,1,2,4,4,1,1,1,5,2,1,1,2,2,3,2,2,6,2,1,1,1,
  1,1,0,3,3,4,0,1,1,4,1,3,2,3,2,3,6,5,5,3,0,9,2,1,3,0,1,1,3,1,3,3,3,3,3,
  3,3,3,3,3,3,3,1,0,0,7,1,0,4,1,0,4,1,3,1,3,1,3,1,3,3,1,3,3,3,3,1,3,3,1,
  3,3,1,3,3,3,1,2,2,2,2,1,3,3,1,1,2,2,2,2,1,4,4,1,4,3,4,0,1,1,3,1,1,1,1,
  1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,
  2,2,1,2,1,1,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned short ag_ptt[] = {
    0, 12, 12, 12, 12, 12,203,204,205,206,207, 14, 14, 17, 17, 26, 26, 27,
   27, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 31, 32, 51,
   18, 13, 13, 16, 33, 56, 56, 34, 28, 29, 38, 42, 42, 43, 43, 47, 47, 47,
   47, 67, 48, 68, 54, 54, 19, 19, 37, 37, 64, 64, 64, 64, 64, 64, 64, 64,
   64, 64, 64, 64, 64, 69, 85, 87, 69, 83, 90, 83, 88, 93, 88, 91, 91, 94,
   94, 96, 96, 98, 98, 98,100,100,100,100,100,103,103,103,108,108,108,111,
  111,111,111,114,114,114,114,114,118,118,121,121,121,121,121,121,121,121,
  121,121, 70, 70, 70,126,130,130,131,131,123,123,123,123,  1,136,136,137,
  137,  1,141,141,142,142,  1,215,215,272,272,272,146,146,146,149,149,149,
  154,154,148,148,150,150,271,271,271,151,151,155,155,152,152,153,153,165,
  165,157,157,166,166,132,132,274,169,169,170,170,172,172,172,173,173,173,
  173,173,173,173,173,173,173,173,174,174,174,187,188,189,175,175,192,192,
  273,194,194,156, 20, 20,133,133,135,135,135,135,135,135,135,135,135,135,
  135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
  135,135,135,135,135,135,135,135,140,140,140,140,140,140,140,140,140,140,
  140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
  140,140,140,140,140,140,140,144,144,144,145,145,145,145,145,145,158,158,
  159,159,159,162,162,167,167,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,195,195,195,195,195,195,195,195,195,195,195,195,195,195,
  195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,
  195,201,201,202,202,  7,  8,  9, 10, 11, 36, 35, 39, 40, 41, 46, 45,  4,
   44, 49, 50, 52, 53, 55, 57, 58, 60, 59, 61, 62, 63, 65, 66, 71, 72, 73,
   74, 75, 76, 77, 78, 79, 80, 81, 82, 86, 84, 89, 92, 95, 97, 99,101,102,
  104,105,106,107,109,110,112,113,115,116,117,119,120,122,124,125,127,128,
  129,  2,  3,  6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: VNO AG_WRAP_5(ag_rp_5(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER); break;
    case 12: ag_rp_15(PCB_POINTER); break;
    case 13: ag_rp_16(PCB_POINTER, V(1,(FileLocation *)), V(3,(FileLocation *))); break;
    case 14: VWD(0,AG_WRAP_5 *); break;
    case 15: ag_rp_17(PCB_POINTER); break;
    case 16: ag_rp_18(PCB_POINTER); break;
    case 17: ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 18: ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_5 *); break;
    case 19: ag_rp_21(PCB_POINTER); break;
    case 20: ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_7 *); break;
    case 21: ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 22: ag_rp_24(PCB_POINTER); break;
    case 23: V(0,(FileLocation *)) = ag_rp_25(PCB_POINTER); break;
    case 24: ag_rp_26(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 25: ag_rp_27(PCB_POINTER); break;
    case 26: ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 27: ag_rp_29(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 28: ag_rp_30(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 29: ag_rp_31(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 30: ag_rp_32(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(5, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(5, AG_WRAP_7 *); break;
    case 31: ag_rp_33(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); break;
    case 32: ag_rp_34(PCB_POINTER, VW(2, AG_WRAP_7 *), VW(4, AG_WRAP_7 *)); 
            VWD(2, AG_WRAP_7 *); VWD(4, AG_WRAP_7 *); break;
    case 33: ag_rp_35(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_7 *); break;
    case 34: ag_rp_36(PCB_POINTER, VW(-4, AG_WRAP_4 *), VW(-2, AG_WRAP_7 *)); break;
    case 35: ag_rp_37(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(7, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(7, AG_WRAP_4 *); break;
    case 36: VNO AG_WRAP_4(ag_rp_38(PCB_POINTER)); break;
    case 37: VRO(AG_WRAP_4 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 38: VRO(AG_WRAP_7 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 39: VNO AG_WRAP_5(ag_rp_41(PCB_POINTER)); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: VRO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 43: VRO(AG_WRAP_5 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 44: VRO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 45: VRO(AG_WRAP_5 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 46: VRO(AG_WRAP_5 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 47: VRO(AG_WRAP_5 *, ag_rp_49(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 48: VRO(AG_WRAP_5 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 49: VRO(AG_WRAP_5 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 50: VRO(AG_WRAP_5 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 51: VRO(AG_WRAP_5 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 52: VRO(AG_WRAP_5 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 53: ag_rp_55(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 54: ag_rp_56(PCB_POINTER, VW(-5, AG_WRAP_5 *), VW(-2, AG_WRAP_5 *)); break;
    case 55: VRO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 56: ag_rp_58(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 57: VRO(AG_WRAP_5 *, ag_rp_59(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 58: ag_rp_60(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 59: VRO(AG_WRAP_5 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 60: VRO(AG_WRAP_5 *, ag_rp_62(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 61: VRO(AG_WRAP_5 *, ag_rp_63(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 62: VRO(AG_WRAP_5 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 63: VRO(AG_WRAP_5 *, ag_rp_65(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 64: VRO(AG_WRAP_5 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 65: VRO(AG_WRAP_5 *, ag_rp_67(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 66: VRO(AG_WRAP_5 *, ag_rp_68(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 67: VRO(AG_WRAP_5 *, ag_rp_69(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 68: VRO(AG_WRAP_5 *, ag_rp_70(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 69: VRO(AG_WRAP_5 *, ag_rp_71(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 70: VRO(AG_WRAP_5 *, ag_rp_72(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 71: VRO(AG_WRAP_5 *, ag_rp_73(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 72: VRO(AG_WRAP_5 *, ag_rp_74(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 73: VRO(AG_WRAP_5 *, ag_rp_75(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 74: VRO(AG_WRAP_5 *, ag_rp_76(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 75: VRO(AG_WRAP_5 *, ag_rp_77(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 76: VNO AG_WRAP_5(ag_rp_78(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 77: VNO AG_WRAP_5(ag_rp_79(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 78: VNO AG_WRAP_5(ag_rp_80(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 79: VNO AG_WRAP_5(ag_rp_81(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 80: VRO(AG_WRAP_5 *, ag_rp_82(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 81: VNO AG_WRAP_5(ag_rp_83(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 82: VRO(AG_WRAP_5 *, ag_rp_84(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 83: VNO AG_WRAP_5(ag_rp_85(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 84: VNO AG_WRAP_5(ag_rp_86(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 85: VRO(AG_WRAP_5 *, ag_rp_87(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 86: VRO(AG_WRAP_5 *, ag_rp_88(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 87: VNO AG_WRAP_5(ag_rp_89(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 88: VNO AG_WRAP_5(ag_rp_90(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 89: VRO(AG_WRAP_4 *, ag_rp_91(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 90: VRO(AG_WRAP_5 *, ag_rp_92(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 91: VRO(AG_WRAP_5 *, ag_rp_93(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 92: VRO(AG_WRAP_4 *, ag_rp_94(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 93: VNO AG_WRAP_6(ag_rp_95(PCB_POINTER)); break;
    case 94: VRO(AG_WRAP_5 *, ag_rp_96(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 95: VRO(AG_WRAP_6 *, ag_rp_97(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 96: VNO AG_WRAP_5(ag_rp_98(PCB_POINTER, V(0,(long *)))); break;
    case 97: VNO AG_WRAP_5(ag_rp_99(PCB_POINTER, V(0,(double *)))); break;
    case 98: VNO AG_WRAP_5(ag_rp_100(PCB_POINTER, V(0,(int *)))); break;
    case 99: VNO AG_WRAP_4(ag_rp_101(PCB_POINTER, V(0,(int *)))); break;
    case 100: VRO(AG_WRAP_4 *, ag_rp_102(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 101: V(0,(double *)) = ag_rp_103(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 102: V(0,(double *)) = ag_rp_104(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 103: V(0,(double *)) = ag_rp_105(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 104: V(0,(double *)) = ag_rp_106(PCB_POINTER, V(0,(double *))); break;
    case 105: V(0,(double *)) = ag_rp_107(PCB_POINTER, V(1,(double *))); break;
    case 106: V(0,(double *)) = ag_rp_108(PCB_POINTER, V(0,(long *))); break;
    case 107: V(0,(double *)) = ag_rp_109(PCB_POINTER, V(0,(long *))); break;
    case 108: V(0,(double *)) = ag_rp_110(PCB_POINTER, V(0,(long *))); break;
    case 109: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(1,(long *))); break;
    case 110: V(0,(long *)) = ag_rp_112(PCB_POINTER, V(1,(long *))); break;
    case 111: V(0,(double *)) = ag_rp_113(PCB_POINTER, V(0,(int *))); break;
    case 112: V(0,(double *)) = ag_rp_114(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 113: V(0,(long *)) = ag_rp_115(PCB_POINTER, V(0,(int *))); break;
    case 114: V(0,(long *)) = ag_rp_116(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 115: V(0,(long *)) = ag_rp_117(PCB_POINTER, V(0,(int *))); break;
    case 116: V(0,(long *)) = ag_rp_118(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 117: V(0,(long *)) = ag_rp_119(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 118: V(0,(long *)) = ag_rp_120(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 119: V(0,(long *)) = ag_rp_121(PCB_POINTER); break;
    case 120: V(0,(long *)) = ag_rp_122(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 121: V(0,(long *)) = ag_rp_123(PCB_POINTER); break;
    case 122: V(0,(long *)) = ag_rp_124(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 123: V(0,(int *)) = ag_rp_125(PCB_POINTER, V(0,(int *))); break;
    case 124: V(0,(int *)) = ag_rp_126(PCB_POINTER, V(0,(int *))); break;
    case 125: VRO(AG_WRAP_5 *, ag_rp_127(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 126: VNO AG_WRAP_5(ag_rp_128(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 127: VNO AG_WRAP_4(ag_rp_129(PCB_POINTER)); break;
    case 128: VRO(AG_WRAP_4 *, ag_rp_130(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 129: V(0,(int *)) = ag_rp_131(PCB_POINTER); break;
    case 130: V(0,(int *)) = ag_rp_132(PCB_POINTER); break;
    case 131: V(0,(int *)) = ag_rp_133(PCB_POINTER); break;
    case 132: V(0,(int *)) = ag_rp_134(PCB_POINTER); break;
    case 133: V(0,(int *)) = ag_rp_135(PCB_POINTER); break;
    case 134: V(0,(int *)) = ag_rp_136(PCB_POINTER); break;
    case 135: V(0,(int *)) = ag_rp_137(PCB_POINTER); break;
    case 136: V(0,(int *)) = ag_rp_138(PCB_POINTER); break;
    case 137: V(0,(int *)) = ag_rp_139(PCB_POINTER); break;
    case 138: V(0,(int *)) = ag_rp_140(PCB_POINTER); break;
    case 139: V(0,(int *)) = ag_rp_141(PCB_POINTER); break;
    case 140: V(0,(int *)) = ag_rp_142(PCB_POINTER, V(1,(int *))); break;
    case 141: V(0,(int *)) = ag_rp_143(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 142: V(0,(int *)) = ag_rp_144(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 143: V(0,(int *)) = ag_rp_145(PCB_POINTER, V(1,(int *))); break;
    case 144: V(0,(int *)) = ag_rp_146(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 145: V(0,(int *)) = ag_rp_147(PCB_POINTER, V(0,(int *))); break;
    case 146: V(0,(int *)) = ag_rp_148(PCB_POINTER, V(0,(int *))); break;
    case 147: V(0,(int *)) = ag_rp_149(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[275] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "syntax selection",
  "local statement",
  "statement list",
  "eof",
  "loop condition",
  "statement",
  "DO WHILE",
  "optional expression",
  "",
  "\"script\"",
  "\"whileLoop\"",
  "\"doLoop\"",
  "\"statement\"",
  "\"forField\"",
  "open statement",
  "closed statement",
  "if condition",
  "else",
  "simple statement",
  "DO",
  "WHILE",
  "for initialization",
  "for field",
  "';'",
  "')'",
  "expression",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "\"array\"",
  "'['",
  "']'",
  "structure declaration",
  "function definition",
  "\"do\"",
  "\"while\"",
  "FOR",
  "\"for\"",
  "\"local\"",
  "name list",
  "'('",
  "",
  "\"if\"",
  "\"else\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "\"struct\"",
  "\"function\"",
  "",
  "function body",
  "conditional expression",
  "lvalue",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "logical or expression",
  "'\\?'",
  "",
  "':'",
  "",
  "logical and expression",
  "\"||\"",
  "",
  "inclusive or expression",
  "\"&&\"",
  "",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "\"==\"",
  "\"!=\"",
  "shift expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "additive expression",
  "\"<<\"",
  "\">>\"",
  "multiplicative expression",
  "'+'",
  "'-'",
  "unary expression",
  "'*'",
  "'/'",
  "'%'",
  "factor",
  "'!'",
  "'~'",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "')'",
  "';'",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "']'",
  "'['",
  "name",
  "\"array\"",
  "\"do\"",
  "\"while\"",
  "\"for\"",
  "\"local\"",
  "'('",
  "\"if\"",
  "\"else\"",
  "'}'",
  "'{'",
  "\"dump\"",
  "','",
  "\"print\"",
  "\"struct\"",
  "\"function\"",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'+'",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned short)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned short)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


