/*
 dci.syn -- Direct Compilation of Bytecode
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.
 Modified to support local function definitions.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. Reduction procedures have been added to enable it to
 compile bytecode as it parses. Changes in the syntax have been minimal.

 dci.syn is compiled with the AnaGram parser generator, which creates dci.h
 and dci.cpp.

 To build a demonstration program for this parser, compile and link as a
 single project:
   dci.cpp                 // This parser, compiles bytecode
   demo.cpp                // Demo program
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include "bcidefs.h"                       // Bytecode interpreter definitions


/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DCI_H_1027348360
#include "dci.h"
#endif

#ifndef DCI_H_1027348360
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dci_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

CodeFragment dci_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 559, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

void dci_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::checkLoop() {
  if (loopDepth) return;
  PCONTEXT(*this) = FileLocation(pointer, line, column);
  reportError("No loop active");
}

// Constructor for parser control block
// Initializes dictionary, input pointer, and loopDepth

dci_pcb_struct::dci_pcb_struct(AgDictionary<AgString> &d, const char *text)
  : pointer((unsigned char *) text),
    dictionary(d),
    loopDepth(0)
{
  // Nothing else to do
}

int dci_pcb_struct::idName(const AgString &name) {
  return dictionary.intern(name);
}

CodeFragment dci_pcb_struct::code(Opcode op, const AgString &name) {
  return CodeFragment().append(op,dictionary.intern(name));
}

CodeFragment dci_pcb_struct::code(Opcode op, const Constant &x) {
  int n = constants.intern(x);
  return CodeFragment().append(op,n);
}

CodeFragment dci_pcb_struct::codeCall(
    const AgString &name,
    AgStack<CodeFragment> &argStack)
{
  int argCount = argStack.size();
  CodeFragment argCode;
  for (int i = 0; i < argCount; i++) argCode.concat(argStack[i]);
  int argCountId = constants.intern(Value(argCount));
  argCode.append(PUSHC, argCountId)
         .append(CALL, functionNames.intern(name));
  return argCode;
}

CodeFragment dci_pcb_struct::defineFunction(
    const AgString &n,
    const AgStack<AgString> &nameList,
    const AgString &text)
{
  LocalFunction f(nameList, text);
  int constantId = constants.intern(Value(f));
  int nameId = dictionary.intern(n);
  CodeFragment code;
  return code.append(LOCATE, nameId)
             .append(PUSHC, constantId)
             .append(SAP);
}

ScriptMethod::ScriptMethod(const char *text, AgDictionary<AgString> &d)
: dictionary(d),
  bytecode(),
  constantList(),
  localList(),
  functionNames()
{
  // Create a parser using the dictionary specified by the dataset
  dci_pcb_type pcb(dictionary, text);

  // run the parser
  try {
    dci(&pcb);
  }
  catch(ErrorMessage e) {
    pcb.reportError(e.message());
  }

  //Use dci_value() to retrieve the generated code
  bytecode = dci_value(&pcb).getBytecode();

  constantList = pcb.constants.contents();
  localList = pcb.localList.contents();
  functionNames = pcb.functionNames.contents();
}

// Apply script to a dataset
Value interpret(const char *text, Dataset &d) {
  ScriptMethod method(text, d.dictionary);

  // List the generated code
  //method.list(cout);

  // Execute the generated code
  return method.apply(d);
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
CodeFragment dci_value(PCB_DECL) {
  CodeFragment returnValue;
  returnValue = (*(AgObjectWrapper< CodeFragment > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline CodeFragment ag_rp_1(PCB_DECL, AgStack<AgString> &list, CodeFragment &x) {
/* Line 135, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.localList = list, x.append(HLT);
}

static inline CodeFragment ag_rp_2(PCB_DECL) {
/* Line 139, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_3(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 140, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y);
}

static inline CodeFragment ag_rp_4(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 158, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment::ifStatement(x, s);
}

static inline CodeFragment ag_rp_5(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 160, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_6(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 162, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.loopDepth--, CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_7(PCB_DECL, CodeFragment &init, CodeFragment &cond, CodeFragment &inc, CodeFragment &s) {
/* Line 165, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.loopDepth--,CodeFragment::forLoop(init, cond, inc, s);
}

static inline CodeFragment ag_rp_8(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 169, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_9(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 171, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.loopDepth--, CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_10(PCB_DECL, CodeFragment &init, CodeFragment &cond, CodeFragment &inc, CodeFragment &s) {
/* Line 174, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.loopDepth--, CodeFragment::forLoop(init, cond, inc, s);
}

static inline CodeFragment ag_rp_11(PCB_DECL, CodeFragment &x) {
/* Line 175, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_12(PCB_DECL, CodeFragment &s, CodeFragment &x) {
/* Line 188, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.loopDepth--, CodeFragment::doLoop(s,x);
}

static inline CodeFragment ag_rp_13(PCB_DECL, CodeFragment &x) {
/* Line 189, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(POP);
}

static inline CodeFragment ag_rp_14(PCB_DECL) {
/* Line 190, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_15(PCB_DECL) {
/* Line 192, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.checkLoop(), CodeFragment().appendBreak(BR);
}

static inline CodeFragment ag_rp_16(PCB_DECL) {
/* Line 193, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.checkLoop(), CodeFragment().appendContinue(BR);
}

static inline CodeFragment ag_rp_17(PCB_DECL) {
/* Line 194, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment().append(HLT);
}

static inline CodeFragment ag_rp_18(PCB_DECL, CodeFragment &x) {
/* Line 195, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(RETURN);
}

static inline AgStack<AgString> ag_rp_19(PCB_DECL) {
/* Line 201, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return AgStack<AgString>();
}

static inline AgStack<AgString> ag_rp_20(PCB_DECL, AgStack<AgString> &list) {
/* Line 202, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return list;
}

static inline void ag_rp_21(PCB_DECL) {
/* Line 205, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_22(PCB_DECL) {
/* Line 208, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_23(PCB_DECL) {
/* Line 211, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  PCB.loopDepth++;
}

static inline CodeFragment ag_rp_24(PCB_DECL, CodeFragment &x) {
/* Line 214, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_25(PCB_DECL, CodeFragment &s) {
/* Line 217, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return s;
}

static inline CodeFragment ag_rp_26(PCB_DECL, AgString &n) {
/* Line 221, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.code(DUMP,n);
}

static inline CodeFragment ag_rp_27(PCB_DECL, CodeFragment &code, AgString &n) {
/* Line 222, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return code.append(DUMP, PCB.idName(n));
}

static inline CodeFragment ag_rp_28(PCB_DECL, CodeFragment &code) {
/* Line 226, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return code.append(PRINT);
}

static inline CodeFragment ag_rp_29(PCB_DECL, CodeFragment &code, CodeFragment &x) {
/* Line 228, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return code.concat(x).append(PRINT);
}

static inline CodeFragment ag_rp_30(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 233, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.defineFunction(n, list, text);
}

static AgString ag_rp_31(PCB_DECL) {
/* Line 236, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_32(PCB_DECL, AgString &n) {
/* Line 241, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_33(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 242, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return stack.push(n);
}

static inline CodeFragment ag_rp_34(PCB_DECL) {
/* Line 245, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_35(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 252, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(POP).concat(y);
}

static inline CodeFragment ag_rp_36(PCB_DECL, CodeFragment &lv, Opcode op, CodeFragment &x) {
/* Line 258, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return lv.concat(x).append(op);
}

static inline CodeFragment ag_rp_37(PCB_DECL, CodeFragment &c, CodeFragment &x, CodeFragment &y) {
/* Line 264, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return CodeFragment::ifElse(c, x, y);
}

static inline CodeFragment ag_rp_38(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 274, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(LOR, y.size()).concat(y);
}

static inline CodeFragment ag_rp_39(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 279, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(LAND, y.size()).concat(y);
}

static inline CodeFragment ag_rp_40(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 285, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(IOR);
}

static inline CodeFragment ag_rp_41(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 290, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(XOR);
}

static inline CodeFragment ag_rp_42(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 295, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(AND);
}

static inline CodeFragment ag_rp_43(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 300, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_44(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 305, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_45(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 310, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_46(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 315, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_47(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 320, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_48(PCB_DECL, CodeFragment &x) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_49(PCB_DECL, Opcode op, CodeFragment &x) {
/* Line 325, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(op);
}

static inline CodeFragment ag_rp_50(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 333, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.concat(y).append(POW);
}

static inline CodeFragment ag_rp_51(PCB_DECL, CodeFragment &x) {
/* Line 341, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_52(PCB_DECL, Constant &x) {
/* Line 342, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.code(PUSHC, x);
}

static inline CodeFragment ag_rp_53(PCB_DECL, CodeFragment &x) {
/* Line 343, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(FETCH);
}

static inline CodeFragment ag_rp_54(PCB_DECL, CodeFragment &x) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(I_FETCH);
}

static inline CodeFragment ag_rp_55(PCB_DECL, CodeFragment &x) {
/* Line 345, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(D_FETCH);
}

static inline CodeFragment ag_rp_56(PCB_DECL, CodeFragment &x) {
/* Line 346, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(FETCH_I);
}

static inline CodeFragment ag_rp_57(PCB_DECL, CodeFragment &x) {
/* Line 347, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(FETCH_D);
}

static inline CodeFragment ag_rp_58(PCB_DECL, CodeFragment &x) {
/* Line 349, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(CAST_LONG);
}

static inline CodeFragment ag_rp_59(PCB_DECL, CodeFragment &x) {
/* Line 350, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x.append(CAST_DOUBLE);
}

static inline CodeFragment ag_rp_60(PCB_DECL, AgString &n) {
/* Line 353, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.code(LOCATE, n);
}

static inline CodeFragment ag_rp_61(PCB_DECL, AgString &n, AgStack<CodeFragment> &args) {
/* Line 356, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return PCB.codeCall(n, args);
}

static inline AgStack<CodeFragment> ag_rp_62(PCB_DECL) {
/* Line 359, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return AgStack<CodeFragment>();
}

static inline AgStack<CodeFragment> ag_rp_63(PCB_DECL, CodeFragment &code) {
/* Line 363, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return AgStack<CodeFragment>().push(code);
}

static inline AgStack<CodeFragment> ag_rp_64(PCB_DECL, AgStack<CodeFragment> &stack, CodeFragment &x) {
/* Line 364, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return stack.push(x);
}

static inline Constant ag_rp_65(PCB_DECL, long x) {
/* Line 367, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_66(PCB_DECL, double x) {
/* Line 368, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_67(PCB_DECL, int x) {
/* Line 370, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return Constant(x);
}

static inline Opcode ag_rp_68(PCB_DECL) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return STORE;
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return ADDM;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 376, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return SUBM;
}

static inline Opcode ag_rp_71(PCB_DECL) {
/* Line 377, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return MULM;
}

static inline Opcode ag_rp_72(PCB_DECL) {
/* Line 378, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return DIVM;
}

static inline Opcode ag_rp_73(PCB_DECL) {
/* Line 379, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return MODM;
}

static inline Opcode ag_rp_74(PCB_DECL) {
/* Line 380, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return IORM;
}

static inline Opcode ag_rp_75(PCB_DECL) {
/* Line 381, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return ANDM;
}

static inline Opcode ag_rp_76(PCB_DECL) {
/* Line 382, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return XORM;
}

static inline Opcode ag_rp_77(PCB_DECL) {
/* Line 383, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return LSM;
}

static inline Opcode ag_rp_78(PCB_DECL) {
/* Line 384, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return RSM;
}

static inline Opcode ag_rp_79(PCB_DECL) {
/* Line 385, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return POWM;
}

static inline Opcode ag_rp_80(PCB_DECL) {
/* Line 388, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return EQ;
}

static inline Opcode ag_rp_81(PCB_DECL) {
/* Line 389, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return NE;
}

static inline Opcode ag_rp_82(PCB_DECL) {
/* Line 392, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return LT;
}

static inline Opcode ag_rp_83(PCB_DECL) {
/* Line 393, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return LE;
}

static inline Opcode ag_rp_84(PCB_DECL) {
/* Line 394, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return GT;
}

static inline Opcode ag_rp_85(PCB_DECL) {
/* Line 395, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return GE;
}

static inline Opcode ag_rp_86(PCB_DECL) {
/* Line 398, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return LS;
}

static inline Opcode ag_rp_87(PCB_DECL) {
/* Line 399, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return RS;
}

static inline Opcode ag_rp_88(PCB_DECL) {
/* Line 402, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return ADD;
}

static inline Opcode ag_rp_89(PCB_DECL) {
/* Line 403, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return SUB;
}

static inline Opcode ag_rp_90(PCB_DECL) {
/* Line 406, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return MUL;
}

static inline Opcode ag_rp_91(PCB_DECL) {
/* Line 407, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return DIV;
}

static inline Opcode ag_rp_92(PCB_DECL) {
/* Line 408, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return MOD;
}

static inline Opcode ag_rp_93(PCB_DECL) {
/* Line 411, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return NEG;
}

static inline Opcode ag_rp_94(PCB_DECL) {
/* Line 412, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return NOT;
}

static inline Opcode ag_rp_95(PCB_DECL) {
/* Line 413, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return COM;
}

static inline AgString ag_rp_96(PCB_DECL, int c) {
/* Line 431, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_97(PCB_DECL, AgString &ns, int c) {
/* Line 432, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return ns.append(c);
}

static inline double ag_rp_98(PCB_DECL, double x, long e) {
/* Line 438, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_99(PCB_DECL, double x, long e) {
/* Line 439, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_100(PCB_DECL, double i, double f) {
/* Line 442, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return i+f;
}

static inline double ag_rp_101(PCB_DECL, double i) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return i;
}

static inline double ag_rp_102(PCB_DECL, double f) {
/* Line 444, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return f;
}

static inline double ag_rp_103(PCB_DECL, long x) {
/* Line 447, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x;
}

static inline double ag_rp_104(PCB_DECL, long x) {
/* Line 448, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x;
}

static inline double ag_rp_105(PCB_DECL, long x) {
/* Line 449, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return makeDecimal(x);
}

static inline long ag_rp_106(PCB_DECL, long x) {
/* Line 452, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return (long)x;
}

static inline long ag_rp_107(PCB_DECL, long x) {
/* Line 453, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return -(long)x;
}

static inline double ag_rp_108(PCB_DECL, int d) {
/* Line 456, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_109(PCB_DECL, int d, double f) {
/* Line 457, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_110(PCB_DECL, int d) {
/* Line 465, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return d-'0';
}

static inline long ag_rp_111(PCB_DECL, long x, int d) {
/* Line 466, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_112(PCB_DECL, int d) {
/* Line 469, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return d-'0';
}

static inline long ag_rp_113(PCB_DECL, long x, int d) {
/* Line 470, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_114(PCB_DECL, long x, int d) {
/* Line 473, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_115(PCB_DECL, long x, int d) {
/* Line 474, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_116(PCB_DECL) {
/* Line 477, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 0;
}

static inline long ag_rp_117(PCB_DECL, long n, int d) {
/* Line 478, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_118(PCB_DECL) {
/* Line 481, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 0;
}

static inline long ag_rp_119(PCB_DECL, long n, int d) {
/* Line 482, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_120(PCB_DECL, int x) {
/* Line 485, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return x - '0';
}

static inline int ag_rp_121(PCB_DECL, int d) {
/* Line 486, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return (d & 7) + 9;
}

static inline Constant ag_rp_122(PCB_DECL, Value &s) {
/* Line 491, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return Constant(s);
}

static inline Constant ag_rp_123(PCB_DECL, Constant &s, Value &e) {
/* Line 492, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return Constant(s+=e);
}

static inline Value ag_rp_124(PCB_DECL, AgString &b) {
/* Line 495, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return Value(b);
}

static inline AgString ag_rp_125(PCB_DECL) {
/* Line 499, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return AgString();
}

static inline AgString ag_rp_126(PCB_DECL, AgString &s, int c) {
/* Line 500, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return s.append(c);
}

static inline int ag_rp_127(PCB_DECL) {
/* Line 512, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\'';
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 513, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '"';
}

static inline int ag_rp_129(PCB_DECL) {
/* Line 514, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\?';
}

static inline int ag_rp_130(PCB_DECL) {
/* Line 515, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\\';
}

static inline int ag_rp_131(PCB_DECL) {
/* Line 516, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\a';
}

static inline int ag_rp_132(PCB_DECL) {
/* Line 517, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\b';
}

static inline int ag_rp_133(PCB_DECL) {
/* Line 518, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\f';
}

static inline int ag_rp_134(PCB_DECL) {
/* Line 519, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\n';
}

static inline int ag_rp_135(PCB_DECL) {
/* Line 520, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\r';
}

static inline int ag_rp_136(PCB_DECL) {
/* Line 521, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\t';
}

static inline int ag_rp_137(PCB_DECL) {
/* Line 522, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return '\v';
}

static inline int ag_rp_138(PCB_DECL, int d) {
/* Line 528, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return d-'0';
}

static inline int ag_rp_139(PCB_DECL, int n, int d) {
/* Line 531, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_140(PCB_DECL, int n, int d) {
/* Line 534, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_141(PCB_DECL, int d) {
/* Line 537, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return d;
}

static inline int ag_rp_142(PCB_DECL, int n, int d) {
/* Line 538, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_143(PCB_DECL, int d) {
/* Line 541, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return d-'0';
}

static inline int ag_rp_144(PCB_DECL, int d) {
/* Line 542, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_145(PCB_DECL, int c) {
/* Line 551, W:/agdev/agex/packages/xidek/cll/function/dci.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 4, 9, 6, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 6, 6, 6,
  6, 6, 6, 6, 6, 6, 4, 6, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 6, 6, 0, 0, 0, 6, 0, 0, 0, 0, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 6, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 4, 6, 0, 0, 6, 0, 6, 0, 0, 0, 6, 0, 0, 0, 0, 0, 6, 6, 6,
  6, 6, 6, 6, 7, 7, 6, 0, 0, 0, 6, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0, 9, 0,
  0, 0, 0, 6, 6, 0, 0, 0, 9, 4, 6, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<CodeFragment >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<CodeFragment> >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<Constant >
#undef AG_WRAP_9
#define AG_WRAP_9 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      case 9: ag_delete_object((AG_WRAP_9 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dci_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,  0,
   15, 16, 17, 18,  0,  0,  0, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
   30, 31, 32, 33, 34,  0,  0, 35,  0, 36,  0, 37,  0, 38,  0, 39,  0, 40,
    0, 41,  0, 42,  0, 43,  0, 44,  0, 45,  0, 46,  0, 47,  0, 48, 49,  0,
   50, 51, 52, 53, 54, 55, 56, 57,  0, 58, 59, 60, 61, 62,  0, 63, 64, 65,
   66,  0, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82,
   83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0, 96, 97,  0, 98, 99,100,101,102,103,104,105,  0,
    0,106,107,108,109,  0,  0,  0,110,111,112,113,114,115,116,117,  0,  0,
  118,119,120,121,122,123,124,125,126,  0,  0,  0,  0,  0,127,128,129,130,
  131,132,133,134,135,136,137,  0,  0,  0,138,139,140,141,142,143,144,145
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,183,  0,187,  0,188,  0,189,  0,190,  0,191,  0,192,  0,193,
  0,194,  0,198,  0,200,  0,201,  0,213,  0,214,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 88,120,255,111,117,255,111,117,255, 43, 45, 47, 48, 98,
   99,100,102,105,108,112,114,119,255, 42,255, 38, 61,255, 61,255, 42, 61,
  255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 88,120,255, 61,255, 60, 61,
  255, 61,255, 61, 62,255,117,255,111,117,255,111,117,255, 99,110,255,111,
  255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 48, 60, 61, 62, 94, 98, 99,
  100,101,102,105,108,112,114,119,124,255, 88,120,255,111,117,255,111,117,
  255, 43, 45, 48, 98, 99,100,102,105,108,112,114,119,255, 42, 47,255, 47,
  255, 88,120,255,111,117,255,111,117,255, 43, 45, 48, 98, 99,100,102,105,
  112,114,119,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,255,
   42, 47, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33,
   37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255, 42, 47,255, 88,120,255,
   43, 45, 47, 48,255, 42, 47,255, 88,120,255,111,117,255,111,117,255, 43,
   45, 47, 48, 98, 99,100,101,102,105,112,114,119,255, 34, 39, 63, 92, 97,
   98,102,110,114,116,118,120,255, 92,255, 42, 47,255, 60, 61,255, 61, 62,
  255, 33, 38, 42, 47, 60, 61, 62,124,255, 60, 61,255, 61, 62,255, 33, 38,
   42, 60, 61, 62,124,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,100,108,
  255, 88,120,255, 43, 45, 48,100,108,255, 88,120,255, 43, 45, 48,255, 60,
   61,255, 61, 62,255, 33, 38, 60, 61, 62,124,255, 33, 38, 60, 61, 62,124,
  255, 33, 38, 61,124,255, 38,124,255,124,255, 38, 61,255, 61,255, 42, 61,
  255, 43, 61,255, 45, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,
  124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255, 42, 47,255,
   88,120,255,111,117,255,111,117,255, 43, 45, 47, 48, 98, 99,100,102,105,
  112,114,119,255, 60, 61,255, 61, 62,255, 33, 38, 42, 43, 45, 60, 61, 62,
  124,255,119,255, 88,120,255,111,117,255,111,117,255, 43, 45, 48, 98, 99,
  100,101,102,105,112,114,119,255, 42, 47,255, 88,120,255, 60, 61,255, 61,
   62,255,111,117,255,111,117,255, 33, 38, 42, 43, 45, 47, 48, 60, 61, 62,
   98, 99,100,102,105,112,114,119,124,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,0,0,4,5,7,4,7,7,4,3,3,2,2,7,7,2,2,7,7,7,7,7,4,3,4,0,0,4,0,4,1,
  0,4,0,0,4,0,0,4,0,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,7,4,6,7,4,7,7,4,7,7,
  4,2,4,0,0,4,3,3,2,2,2,2,2,2,2,3,2,3,7,7,2,7,2,7,2,7,7,7,2,4,0,0,4,5,7,
  4,7,7,4,3,3,2,7,7,2,2,7,7,7,7,7,4,0,0,4,2,4,0,0,4,5,7,4,7,7,4,3,3,2,7,
  7,2,2,7,7,7,7,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,4,1,0,4,0,4,0,1,
  4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,0,0,4,0,0,4,3,3,2,2,4,0,0,4,0,0,4,5,
  7,4,7,7,4,3,3,2,2,7,7,2,7,2,7,7,7,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,
  0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,0,4,0,0,4,3,3,3,2,3,2,3,4,0,0,4,0,
  0,4,3,3,2,2,7,7,4,0,0,4,3,3,2,7,7,4,0,0,4,3,3,2,4,0,0,4,0,0,4,3,3,2,3,
  2,3,4,3,3,3,3,3,3,4,3,3,3,3,4,3,3,4,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,
  4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,3,2,4,0,0,4,0,0,4,5,7,4,7,
  7,4,3,3,2,2,7,7,2,2,7,7,7,7,4,0,0,4,0,0,4,3,3,3,3,3,2,3,2,3,4,7,4,0,0,
  4,5,7,4,7,7,4,3,3,2,7,7,2,7,2,7,7,7,7,4,0,0,4,0,0,4,0,0,4,0,0,4,5,7,4,
  7,7,4,3,3,3,3,3,2,2,2,3,2,7,7,2,2,7,7,7,7,3,4
};

static const unsigned char ag_key_parm[] = {
    0,113,118,  0,144,143,  0, 12, 18,  0, 14, 22,  0,211,212,  0,  0,  4,
    6,  0,  0, 16, 10, 20,  8,  2,  0,117,  0,205,225,  0,229,  0,210,221,
    0,211,219,  0,212,220,  0,113,118,222,  0,144,143,  0,227,  0,236,233,
    0,228,  0,235,237,  0, 26,  0, 12, 18,  0, 14, 22,  0, 10, 24,  0,  0,
    0,224,204,  0,231,223,  0,  0,  0,  0,  0,  0,  0,230,  0,226,  4,  6,
    0,  0,  0, 16,  0, 20,  8,  2,  0,  0,144,143,  0, 12, 18,  0, 14, 22,
    0,211,212,  0,  4,  6,  0,  0, 16, 10, 20,  8,  2,  0,113,118,  0,  0,
    0,144,143,  0, 12, 18,  0, 14, 22,  0,211,212,  0,  4,  6,  0,  0, 16,
   20,  8,  2,  0,205,225,  0,229,  0,210,221,  0,211,219,  0,212,220,  0,
  113,118,222,  0,227,  0,236,233,  0,228,  0,235,237,  0,224,204,  0,231,
  223,  0,  0,  0,  0,  0,  0,230,  0,226,  0,  0,113,118,  0,144,143,  0,
  211,212,  0,  0,  0,113,118,  0,144,143,  0, 12, 18,  0, 14, 22,  0,211,
  212,  0,  0,  4,  6,  0,  0,  0, 16, 20,  8,  2,  0,157,156,158,159,160,
  161,162,163,164,165,166,171,  0,  0,  0,113,118,  0,236,233,  0,235,237,
    0,231,205,210,  0,  0,230,  0,204,  0,236,233,  0,235,237,  0,231,205,
  210,  0,230,  0,204,  0,113,118,  0,144,143,  0,211,212,  0,  0, 26, 24,
    0,144,143,  0,211,212,  0, 26, 24,  0,144,143,  0,211,212,  0,  0,236,
  233,  0,235,237,  0,231,205,  0,230,  0,204,  0,231,205,233,230,235,204,
    0,231,205,230,204,  0,205,204,  0,204,  0,205,225,  0,229,  0,210,221,
    0,211,219,  0,212,220,  0,227,  0,236,233,  0,228,  0,235,237,  0,224,
  204,  0,231,223,  0,  0,  0,  0,222,  0,230,  0,226,  0,  0,113,118,  0,
  144,143,  0, 12, 18,  0, 14, 22,  0,211,212,  0,  0,  4,  6,  0,  0, 16,
   20,  8,  2,  0,236,233,  0,235,237,  0,231,205,210,211,212,  0,230,  0,
  204,  0,  2,  0,144,143,  0, 12, 18,  0, 14, 22,  0,211,212,  0,  4,  6,
    0,  0,  0, 16, 20,  8,  2,  0,113,118,  0,144,143,  0,236,233,  0,235,
  237,  0, 12, 18,  0, 14, 22,  0,231,205,210,211,212,  0,  0,  0,230,  0,
    4,  6,  0,  0, 16, 20,  8,  2,204,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0,  0,  0,  0,  0, 17,  0, 20, 22,  0,  0,  2,  1,  4,  4,
    9,  7, 10, 29, 31, 36, 41, 47,  0, 52,  0,  0,  0,  0,  0,  0, 32,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 50,  0,
    0,  0,  0,  0, 55,  0, 75,  0, 60, 79,  0, 86, 88,  0, 97,100,  0, 68,
    0,  0,  0,  0, 54, 56, 29, 34, 37, 40, 43, 47, 52, 58, 57, 60, 62, 67,
   62, 82, 65, 95, 71,102,107,113, 73,  0,  0,  0,  0,  0,135,  0,138,140,
    0,118,120,100,122,127,103,106,147,149,154,159,165,  0,  0,  0,  0,122,
    0,  0,  0,  0,  0,187,  0,190,192,  0,170,172,127,174,179,130,133,199,
  201,206,212,  0,  0,  0,  0,  0,  0,151,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,166,  0,  0,  0,  0,  0,171,  0,  0,  0,  0,217,
  219,148,153,156,159,162,168,221,173,223,176,  0,  0,  0,  0,  0,  0,  0,
  225,227,192,195,  0,  0,  0,  0,  0,  0,  0,  0,246,  0,253,255,  0,229,
  231,203,206,233,238,209,249,212,262,264,269,275,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,229,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,280,282,284,244,247,286,250,288,  0,  0,  0,  0,  0,  0,  0,290,292,
  294,262,296,265,298,  0,  0,  0,  0,  0,  0,  0,300,302,276,279,304,310,
    0,  0,  0,  0,314,316,289,318,324,  0,  0,  0,  0,328,330,298,  0,  0,
    0,  0,  0,  0,  0,332,334,305,336,308,338,  0,340,342,344,346,348,350,
    0,352,354,356,358,  0,360,362,  0,364,  0,  0,  0,  0,  0,  0,338,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,349,  0,  0,  0,  0,  0,354,  0,  0,
    0,  0,366,368,335,340,343,346,370,351,372,356,374,359,  0,  0,  0,  0,
    0,  0,  0,  0,393,  0,396,398,  0,376,378,375,378,380,385,381,384,405,
  407,412,418,  0,  0,  0,  0,  0,  0,  0,423,425,427,429,431,400,433,403,
  435,  0,437,  0,  0,  0,  0,  0,459,  0,466,468,  0,442,444,418,446,451,
  421,462,424,475,477,482,488,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,518,  0,521,523,  0,493,495,497,499,501,440,443,446,503,449,
  505,510,452,455,530,532,537,543,548,  0
};

static const unsigned short ag_key_index[] = {
   13,  0, 27, 76,109,  0,  0, 27, 27,125,  0,136,179,  0,136,198,  0,215,
  242,  0,  0,242,253,  0,253,253,253,253,253,253,253,253,253,268,282,198,
  198,198,  0,  0,292,198,268,301,301,311,311,311,318,325,330,330,330,333,
  362,  0,125,198,125, 13,362,387,  0,301,  0,198,125,125,136,125,125,125,
    0,  0,301,  0,  0,  0,136,  0,  0,  0,136,242,253,  0,  0,242,242,242,
    0,253,  0,  0,125,  0,125,  0,  0,198,301,406,198,198,198,301,301,198,
  198,301,198,198,198,198,301,198,198,301,198,301,198,301,198,301,198,301,
  198,301,301,198,301,198,198,198,198,198,198,198,198,198,198,198,198,301,
    0,136,301,  0,  0,301,416,301,301,301,427,  0,  0,458,301,301,311,311,
  311,318,325,330,330,  0,  0,  0,  0,215,  0,  0,  0,  0,  0,387,136,253,
  253,301,198,301,  0,301,301,136,  0,  0,  0,109,  0,301,136,  0,  0,136,
  136
};

static const unsigned char ag_key_ends[] = {
43,0, 45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 114,0, 110,99,116,105,111,110,0, 102,0, 111,99,97,108,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 47,0, 
61,0, 61,0, 61,0, 61,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 98,108,101,0, 109,112,0, 
108,115,101,0, 114,0, 110,99,116,105,111,110,0, 102,0, 97,108,0, 
103,0, 114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 
43,0, 45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 114,0, 110,99,116,105,111,110,0, 102,0, 111,99,97,108,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 43,0, 
45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
114,0, 110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 61,0, 61,0, 61,0, 
61,0, 43,0, 45,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 61,0, 38,0, 42,0, 
61,0, 124,0, 61,0, 38,0, 42,0, 61,0, 124,0, 43,0, 45,0, 
111,117,98,108,101,0, 111,110,103,0, 43,0, 45,0, 
111,117,98,108,101,0, 111,110,103,0, 43,0, 45,0, 61,0, 38,0, 
61,0, 124,0, 61,0, 38,0, 61,0, 61,0, 61,0, 124,0, 61,0, 
38,0, 61,0, 124,0, 38,0, 124,0, 124,0, 61,0, 61,0, 61,0, 
61,0, 61,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 61,0, 38,0, 42,0, 
43,0, 45,0, 61,0, 124,0, 104,105,108,101,0, 43,0, 45,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 114,0, 110,99,116,105,111,110,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 61,0, 
38,0, 42,0, 43,0, 45,0, 61,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 124,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   10,177,177,177,177,177,177,177,177,176,122,176,176,176,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,176,242,148,177,
  177,241,208,173,185,184,239,209,199,238,129,240,141,178,178,178,178,178,
  178,178,140,140,202,186,232,218,234,203,177,179,179,179,179,126,179,180,
  180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,
  180,177,170,177,207,180,177,179,179,179,179,126,179,180,180,180,180,180,
  180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,196,206,195,
  243,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dci_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
243,242,238,212,211,209,201,200,198,196,194,193,192,191,190,189,188,187,186,
  185,180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,10,0,1,
  181,182,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,173,170,148,141,140,129,126,122,0,120,121,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,173,170,148,141,140,129,126,122,117,0,115,
  116,
176,122,118,113,0,1,
243,242,238,212,211,209,201,200,198,196,194,193,192,191,190,189,188,187,186,
  185,180,179,178,173,148,144,143,141,140,129,126,10,0,7,8,33,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,173,170,148,141,140,129,126,0,
122,0,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,173,170,148,141,140,129,126,122,0,
117,0,
180,179,176,126,122,118,113,0,1,181,182,
180,179,126,0,4,34,197,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,10,0,9,
241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,
  222,221,220,219,218,212,211,210,209,208,207,206,205,204,203,202,199,186,
  185,184,180,179,178,176,141,140,126,122,118,113,0,1,181,182,
199,186,0,22,41,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,10,0,2,3,4,5,6,11,12,13,14,
  16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,
  47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,
  111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
180,179,126,0,4,197,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,190,189,188,187,186,
  185,184,183,180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,
  10,0,1,181,182,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,173,171,170,166,165,164,163,162,161,160,159,
  158,157,156,148,141,140,129,126,0,149,
178,141,140,0,136,
178,141,140,0,130,136,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,171,170,166,165,164,163,162,161,160,159,158,
  157,156,148,141,140,129,126,0,152,153,154,155,167,168,169,174,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,176,148,122,118,113,0,1,181,182,
129,126,0,
126,0,
179,178,141,140,126,0,146,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,178,176,141,140,129,126,122,118,113,0,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,178,176,141,140,129,126,122,118,113,0,136,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,176,122,118,113,0,1,181,182,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,176,122,118,113,0,1,181,182,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,176,122,118,113,0,1,181,182,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,180,179,176,126,122,118,113,0,1,181,182,
241,240,239,238,237,236,235,234,233,232,231,230,210,209,208,207,206,205,204,
  203,202,199,186,184,180,179,176,126,122,118,113,0,1,181,182,
148,0,5,244,
243,242,238,214,213,212,211,209,186,185,184,180,179,178,176,173,148,144,143,
  141,140,129,126,122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
180,179,126,0,4,47,197,
180,179,126,0,4,47,197,
243,242,238,214,213,212,211,209,185,180,179,178,173,148,144,143,141,140,129,
  126,0,2,3,4,5,6,17,18,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,
  74,75,77,78,79,80,81,82,85,106,110,111,125,128,131,132,133,137,145,197,
  215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
210,0,76,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,70,72,73,74,75,77,78,79,80,85,106,110,111,125,128,131,132,
  133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,70,72,73,74,75,77,78,79,80,85,106,110,111,125,128,131,132,
  133,137,145,197,215,216,217,244,
241,240,239,0,71,107,108,109,
238,209,0,69,73,106,
237,236,0,67,104,105,
235,234,233,232,0,65,100,101,102,103,
231,230,0,63,98,99,
208,0,61,
207,0,59,
206,205,0,55,57,
204,0,53,
185,0,17,
203,0,50,
180,179,176,126,122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
180,179,176,126,122,118,113,0,1,181,182,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,191,190,189,188,187,
  186,185,180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,0,1,
  181,182,
229,228,227,226,225,224,223,222,221,220,219,218,212,211,0,48,78,79,86,87,88,
  89,90,91,92,93,94,95,96,97,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,0,1,181,182,
180,179,126,0,4,197,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,
  79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
180,179,126,0,4,197,
243,242,238,212,211,209,186,185,180,179,178,176,173,148,144,143,141,140,129,
  126,122,118,113,0,1,181,182,
186,176,122,118,113,0,1,181,182,
186,176,122,118,113,0,1,181,182,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,190,189,188,187,186,
  185,180,179,178,173,148,144,143,141,140,129,126,0,9,
185,176,122,118,113,0,1,181,182,
185,176,122,118,113,0,1,181,182,
185,176,122,118,113,0,1,181,182,
199,186,0,22,41,
199,186,0,22,41,
243,242,238,212,211,209,186,185,180,179,178,173,148,144,143,141,140,129,126,
  0,2,3,4,5,6,17,18,22,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,
  75,77,78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,
  217,244,
186,0,22,
186,0,22,
199,186,0,22,41,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,11,12,13,14,16,
  17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,47,
  49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,
185,0,17,
185,0,17,
185,0,17,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,11,12,13,14,16,
  17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,47,
  49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,
243,242,241,240,239,238,234,232,218,209,208,207,206,203,202,199,196,195,186,
  185,184,180,179,178,177,176,173,171,170,166,165,164,163,162,161,160,159,
  158,157,156,148,141,140,129,126,0,150,152,153,154,155,167,168,169,
178,141,140,0,130,136,
178,141,0,
179,178,141,140,126,0,172,
178,141,0,
178,141,0,
179,178,141,140,126,0,172,
173,0,
178,141,140,0,130,136,
238,209,178,141,140,0,127,134,
238,209,178,141,140,0,127,134,
184,176,122,118,113,0,1,181,182,
184,0,19,
184,176,122,118,113,0,1,181,182,
184,0,19,
199,184,0,19,41,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,70,72,73,74,75,77,78,79,80,85,106,110,111,125,128,131,132,
  133,137,145,197,215,216,217,244,
212,211,0,78,79,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,70,72,73,74,75,77,78,79,80,85,106,110,111,125,128,131,132,
  133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,68,70,72,73,74,75,77,78,79,80,85,106,110,111,125,128,131,
  132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,125,128,
  131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,125,
  128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,125,
  128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,
  111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,
  110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,
  106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,
  85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,184,180,179,178,173,148,144,143,141,140,129,126,
  0,2,3,4,5,6,17,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,
  78,79,80,83,84,85,106,110,111,125,128,131,132,133,137,145,197,215,216,
  217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,18,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,
  78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,
  79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
185,0,17,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,190,189,188,187,186,
  185,180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,11,12,13,14,
  16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,39,40,42,43,44,
  46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,
  110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,
  79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
180,179,126,0,4,197,
199,186,0,22,41,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,
  79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
187,0,25,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,18,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,
  78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,186,185,180,179,178,173,148,144,143,141,140,129,126,
  0,2,3,4,5,6,17,18,21,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,
  75,77,78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,
  217,244,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,18,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,
  78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
183,0,15,
178,141,140,0,135,
178,141,140,0,135,
243,242,241,240,239,238,237,236,235,234,233,232,231,230,212,211,210,209,208,
  207,206,205,204,203,202,201,200,199,198,196,194,193,192,190,189,188,187,
  186,185,184,180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,
  0,1,181,182,
212,211,185,180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,17,47,
  75,77,78,79,80,85,125,128,131,132,133,137,145,197,215,216,217,244,
212,211,185,180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,17,47,
  75,77,78,79,80,85,125,128,131,132,133,137,145,197,215,216,217,244,
241,240,239,0,71,107,108,109,
238,209,0,69,73,106,
237,236,0,67,104,105,
235,234,233,232,0,65,100,101,102,103,
231,230,0,63,98,99,
208,0,61,
207,0,59,
199,0,41,
184,0,19,
202,199,0,41,51,
180,179,126,0,4,34,197,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,190,189,188,187,186,
  185,183,180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,10,
  0,1,181,182,
185,0,17,
199,184,0,19,41,
199,0,41,
186,0,22,
199,184,0,19,41,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,176,173,148,144,143,141,140,129,126,122,118,113,0,1,181,182,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,12,13,14,16,17,
  18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,47,49,
  52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,
178,141,140,0,136,
178,141,140,0,136,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,
  79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,185,180,179,178,176,173,148,144,143,141,140,129,126,
  122,118,113,0,1,181,182,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,
  80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
199,184,0,19,41,
243,242,238,212,211,209,185,180,179,178,173,148,144,143,141,140,129,126,0,2,
  3,4,5,6,17,18,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,
  78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,186,185,180,179,178,173,148,144,143,141,140,129,126,
  0,2,3,4,5,6,17,18,21,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,
  75,77,78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,
  217,244,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,12,13,14,16,17,
  18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,47,49,
  52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,
196,0,38,
199,184,0,19,41,
186,0,22,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,191,190,189,188,187,
  186,185,180,179,178,173,148,144,143,141,140,129,126,0,8,33,45,
186,0,22,
243,242,238,212,211,209,185,184,180,179,178,173,148,144,143,141,140,129,126,
  0,2,3,4,5,6,17,18,21,43,46,47,49,52,54,56,58,60,62,64,66,68,70,72,73,74,
  75,77,78,79,80,85,106,110,111,125,128,131,132,133,137,145,197,215,216,
  217,244,
243,242,238,212,211,209,201,200,198,196,195,194,193,192,190,189,188,187,186,
  185,180,179,178,173,148,144,143,141,140,129,126,0,9,
195,0,39,
184,0,19,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,11,12,13,14,16,
  17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,47,
  49,52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,
243,242,238,212,211,209,201,200,198,196,194,193,192,190,189,188,187,186,185,
  180,179,178,173,148,144,143,141,140,129,126,0,2,3,4,5,6,12,13,14,16,17,
  18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,37,38,40,42,43,44,46,47,49,
  52,54,56,58,60,62,64,66,68,70,72,73,74,75,77,78,79,80,85,106,110,111,
  125,128,131,132,133,137,145,197,215,216,217,244,

};


static unsigned const char ag_astt[5237] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,8,8,8,8,8,8,8,1,1,1,8,7,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,
  7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,1,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,7,0,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,5,3,7,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,
  7,1,1,3,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  1,2,2,7,2,2,1,2,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,
  5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,
  4,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,
  5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,5,2,1,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,2,1,2,2,2,7,2,2,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,
  2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,2,2,2,1,1,5,1,2,2,1,1,5,1,2,2,1,1,
  1,1,5,1,2,2,2,2,1,1,5,1,2,2,1,5,1,1,5,1,1,1,5,1,1,1,5,1,1,4,1,1,5,1,5,5,1,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,2,2,2,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,
  2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,1,1,
  1,7,1,1,3,5,1,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,7,1,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,
  1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,
  1,1,1,7,2,1,7,2,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,
  1,1,1,7,1,1,7,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,
  1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,
  7,2,1,1,1,8,8,8,7,2,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,
  7,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,
  2,1,1,1,1,1,1,1,4,2,2,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,
  1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,2,
  2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,
  2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,1,7,2,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,2,2,7,
  1,2,2,2,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
  4,1,2,2,2,1,1,4,1,2,2,1,1,4,1,2,2,1,1,1,1,4,1,2,2,2,2,1,1,4,1,2,2,1,4,1,1,
  4,1,1,5,1,1,7,2,1,1,7,1,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,1,1,7,2,1,1,5,1,1,7,
  1,1,1,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,
  7,2,2,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,4,
  2,10,10,10,4,2,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,
  1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,7,1,1,1,7,1,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,7,2,1,1,
  1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,7,2,1,7,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,4,2,2,1,2,2,2,2,2,1,1,1,1,1,2,
  2,1,2,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,2,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,
  2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,4,4,4,4,4,4,4,3,1,2,4,0,3,
  3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
351,351,1,2,353,351,
25,25,25,25,25,25,25,25,25,25,25,25,25,9,25,25,25,25,25,25,25,25,25,25,25,
  25,25,25,25,25,25,25,4,0,11,10,
128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
  128,128,128,128,128,128,128,128,128,128,128,128,128,128,130,
131,6,
123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,
  123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,125,
126,8,
352,352,3,352,3,1,2,9,3,3,362,
132,132,132,10,38,13,12,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,11,14,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,133,133,133,3,133,133,133,3,1,2,12,3,3,368,
15,17,13,26,16,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,1,14,89,90,54,166,92,3,3,3,82,81,40,77,80,16,13,
  78,28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,50,
  49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,
  25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,15,3,3,370,
132,132,132,16,39,12,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,1,2,352,17,
  3,3,357,
169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
  169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,169,
  169,169,169,169,169,169,169,169,18,83,
157,157,157,141,157,
84,84,84,20,139,84,
90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,
  90,86,85,186,185,184,183,182,181,180,179,178,177,176,90,90,90,90,90,21,
  90,90,90,89,88,87,90,90,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,3,352,3,1,2,22,3,3,415,
91,92,23,
93,134,
165,164,164,164,165,151,163,
150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,159,150,159,156,142,142,150,150,150,26,
149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
  149,149,149,149,149,153,149,153,153,140,140,149,149,149,27,153,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,3,3,1,2,28,3,3,388,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,3,3,1,2,29,3,3,387,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,3,3,1,2,30,3,3,386,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,352,352,3,352,3,1,2,31,3,3,383,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,352,352,3,352,3,1,2,32,3,3,382,
18,91,167,22,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,
  352,352,352,352,3,1,2,34,3,3,356,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,35,3,3,414,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,36,3,3,413,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,37,3,3,409,
132,132,132,38,83,77,12,
132,132,132,39,83,76,12,
35,36,37,94,96,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,40,89,
  90,54,166,92,40,98,98,98,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,
  42,74,39,38,42,97,95,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,
  22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,41,3,3,380,
99,71,100,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,43,89,90,54,
  166,92,40,101,70,70,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,
  25,162,12,30,29,28,22,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,44,89,90,54,
  166,92,40,101,69,69,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,
  25,162,12,30,29,28,22,
102,103,104,64,105,115,116,117,
37,41,62,106,113,114,
107,108,60,109,111,112,
110,111,112,113,58,114,107,108,109,110,
115,116,56,117,105,106,
118,54,119,
120,52,121,
122,124,50,125,123,
126,48,127,
34,83,128,
129,46,130,
352,352,3,352,3,1,2,56,3,3,372,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,57,3,3,371,
352,352,3,352,3,1,2,58,3,3,369,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,1,2,59,3,3,367,
131,132,133,134,135,136,137,138,139,140,141,142,31,32,75,143,78,79,93,94,95,
  96,97,98,99,100,101,102,103,104,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,3,352,352,352,352,352,352,352,352,3,1,2,61,3,3,363,
132,132,132,62,144,12,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,63,89,90,54,
  166,92,40,34,34,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,
  38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
132,132,132,64,32,12,
352,352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,
  352,3,1,2,65,3,3,361,
352,3,3,1,2,66,3,3,360,
352,3,3,1,2,67,3,3,359,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,68,145,
352,3,3,1,2,69,3,3,364,
352,3,3,1,2,70,3,3,358,
352,3,3,1,2,71,3,3,365,
15,17,72,23,146,
15,17,73,22,147,
35,36,37,31,32,41,17,34,132,132,152,21,18,162,162,158,152,20,132,74,89,90,
  54,166,92,40,148,20,148,148,60,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
17,75,19,
17,76,18,
15,17,77,15,149,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,78,89,90,54,166,92,150,150,150,82,81,40,77,80,16,
  13,78,28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,
  50,49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,
  26,25,162,12,30,29,28,22,
34,79,151,
34,80,152,
34,81,153,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,82,89,90,54,166,92,6,6,154,82,81,40,77,80,16,13,
  78,28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,50,
  49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,
  25,162,12,30,29,28,22,
170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
  170,170,170,170,170,170,170,170,86,85,186,185,184,183,182,181,180,179,
  178,177,176,168,170,170,170,170,83,170,170,170,170,89,88,87,170,
84,84,84,147,148,84,
190,190,85,
196,195,195,195,196,86,193,
192,192,188,
191,191,187,
196,195,195,195,196,175,194,
197,90,
84,84,84,138,137,84,
155,156,156,156,156,92,136,156,
155,156,156,156,156,93,135,156,
352,3,3,1,2,94,3,3,385,
157,95,158,
352,3,3,1,2,96,3,3,384,
157,97,159,
15,157,98,73,149,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,99,3,3,381,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,100,89,90,54,
  166,92,40,101,72,72,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,
  25,162,12,30,29,28,22,
31,32,75,78,79,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,102,3,3,412,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,103,3,3,411,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,104,3,3,410,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,105,89,90,54,
  166,92,40,101,67,67,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,
  25,162,12,30,29,28,22,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,106,89,90,54,
  166,92,40,101,160,160,160,44,43,42,74,39,38,42,33,118,119,120,24,23,27,
  19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,107,3,3,408,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,108,3,3,407,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,109,89,90,54,
  166,92,40,101,161,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,
  19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,110,3,3,406,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,111,3,3,405,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,112,3,3,404,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,113,3,3,403,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,114,89,90,54,
  166,92,40,101,162,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,
  27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,115,3,3,402,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,116,3,3,401,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,117,89,90,54,
  166,92,40,101,163,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,
  23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,118,3,3,379,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,119,89,90,54,
  166,92,40,101,164,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,
  24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,120,3,3,378,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,121,89,90,54,
  166,92,40,101,165,49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,
  120,24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,122,3,3,377,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,123,89,90,54,
  166,92,40,101,166,50,49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,
  119,120,24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,124,3,3,376,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,125,89,90,54,
  166,92,40,101,51,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,
  118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,126,3,3,375,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,127,89,90,54,
  166,92,40,101,49,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,38,42,
  33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
35,36,37,31,32,41,34,85,132,132,152,21,18,162,162,158,152,20,132,128,89,90,
  54,166,92,40,87,87,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,
  39,38,42,168,167,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,129,3,3,374,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,130,89,90,54,
  166,92,40,169,169,169,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,
  74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,131,3,3,400,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,132,3,3,399,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,133,3,3,398,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,134,3,3,397,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,135,3,3,396,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,136,3,3,395,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,137,3,3,394,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,138,3,3,393,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,139,3,3,392,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,140,3,3,391,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,141,3,3,390,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,142,3,3,389,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,143,89,90,54,
  166,92,40,45,45,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,
  38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
34,144,170,
35,36,37,31,32,41,56,57,58,59,171,71,69,61,65,66,67,70,17,34,132,132,152,21,
  18,162,162,158,152,20,132,145,89,90,54,166,92,3,3,3,82,81,40,77,80,16,
  13,78,28,13,76,75,74,73,72,13,27,29,79,68,31,64,63,77,62,77,60,55,53,52,
  51,50,49,48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,
  19,26,25,162,12,30,29,28,22,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,146,89,90,54,
  166,92,40,35,35,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,
  38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
132,132,132,147,33,12,
15,17,148,21,149,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,149,89,90,54,
  166,92,40,43,43,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,
  38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
70,150,172,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,151,89,90,54,
  166,92,40,173,173,173,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,
  74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
35,36,37,31,32,41,40,34,132,132,152,21,18,162,162,158,152,20,132,152,89,90,
  54,166,92,40,174,175,174,174,60,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,153,89,90,54,
  166,92,40,176,176,176,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,
  74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
177,5,178,
154,154,154,155,179,
154,154,154,156,180,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,1,2,157,3,3,
  355,
31,32,34,132,132,152,21,18,162,162,158,152,20,132,158,89,90,54,166,92,40,
  101,82,74,39,38,82,33,24,23,27,19,26,25,162,12,30,29,28,22,
31,32,34,132,132,152,21,18,162,162,158,152,20,132,159,89,90,54,166,92,40,
  101,81,74,39,38,81,33,24,23,27,19,26,25,162,12,30,29,28,22,
102,103,104,65,105,115,116,117,
37,41,63,106,113,114,
107,108,61,109,111,112,
110,111,112,113,59,114,107,108,109,110,
115,116,57,117,105,106,
118,55,119,
120,53,121,
15,86,181,
157,168,84,
182,15,169,149,183,
132,132,132,170,38,184,12,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,1,2,352,171,3,3,
  366,
34,172,185,
15,157,173,30,149,
15,41,149,
17,175,186,
15,157,176,187,149,
352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
  352,352,352,3,352,352,352,352,352,352,352,352,3,1,2,177,3,3,354,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,178,89,90,54,166,92,7,10,82,81,40,77,80,16,13,78,
  28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,50,49,
  48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,
  162,12,30,29,28,22,
155,155,155,146,155,
155,155,155,145,155,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,181,89,90,54,
  166,92,40,88,88,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,
  38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
352,352,352,352,352,352,352,352,352,352,3,352,352,352,352,352,352,352,352,3,
  1,2,182,3,3,373,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,183,89,90,54,
  166,92,40,47,101,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,74,39,38,
  42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
15,157,184,188,16,
35,36,37,31,32,41,34,132,132,152,21,18,162,162,158,152,20,132,185,89,90,54,
  166,92,40,189,189,189,60,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,
  74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
35,36,37,31,32,41,40,34,132,132,152,21,18,162,162,158,152,20,132,186,89,90,
  54,166,92,40,174,190,174,174,60,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,187,89,90,54,166,92,8,11,82,81,40,77,80,16,13,78,
  28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,50,49,
  48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,
  162,12,30,29,28,22,
59,188,191,
15,157,189,192,149,
17,190,193,
25,25,25,25,25,25,25,25,25,25,25,25,25,25,9,25,25,25,25,25,25,25,25,25,25,
  25,25,25,25,25,25,25,191,194,10,195,
17,192,14,
35,36,37,31,32,41,34,40,132,132,152,21,18,162,162,158,152,20,132,193,89,90,
  54,166,92,40,174,196,174,174,60,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,162,12,30,29,28,22,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,194,197,
171,195,36,
157,196,198,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,37,89,90,54,166,92,3,3,3,82,81,40,77,80,16,13,78,
  28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,50,49,
  48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,
  162,12,30,29,28,22,
35,36,37,31,32,41,56,57,58,59,71,69,61,65,66,67,70,17,34,132,132,152,21,18,
  162,162,158,152,20,132,198,89,90,54,166,92,9,12,82,81,40,77,80,16,13,78,
  28,13,76,75,74,73,72,13,27,29,79,68,64,63,77,62,77,60,55,53,52,51,50,49,
  48,47,46,45,45,45,44,43,42,74,39,38,42,33,118,119,120,24,23,27,19,26,25,
  162,12,30,29,28,22,

};


static const unsigned short ag_sbt[] = {
     0,  40,  77, 115, 121, 157, 191, 193, 228, 230, 241, 248, 281, 334,
   339, 440, 466, 472, 514, 561, 566, 572, 625, 658, 661, 663, 670, 704,
   739, 771, 803, 835, 870, 905, 909, 939, 965, 991,1017,1024,1031,1099,
  1125,1128,1179,1230,1238,1244,1250,1260,1266,1269,1272,1277,1280,1283,
  1286,1297,1323,1334,1374,1404,1442,1448,1511,1517,1544,1553,1562,1595,
  1604,1613,1622,1627,1632,1698,1701,1704,1709,1809,1812,1815,1818,1918,
  1972,1978,1981,1988,1991,1994,2001,2003,2009,2017,2025,2034,2037,2046,
  2049,2054,2080,2131,2136,2162,2188,2214,2265,2317,2343,2369,2422,2448,
  2474,2500,2526,2580,2606,2632,2687,2713,2769,2795,2852,2878,2936,2962,
  3022,3048,3109,3175,3201,3265,3291,3317,3343,3369,3395,3421,3447,3473,
  3499,3525,3551,3577,3640,3643,3745,3808,3814,3819,3882,3885,3949,4015,
  4079,4082,4087,4092,4151,4191,4231,4239,4245,4251,4261,4267,4270,4273,
  4276,4279,4284,4291,4332,4335,4340,4343,4346,4351,4389,4488,4493,4498,
  4561,4587,4649,4654,4718,4784,4883,4886,4891,4894,4930,4933,4999,5032,
  5035,5038,5138,5237
};


static const unsigned short ag_sbe[] = {
    36,  74, 112, 119, 153, 190, 192, 227, 229, 237, 244, 279, 330, 336,
   370, 462, 469, 510, 559, 564, 569, 616, 654, 660, 662, 668, 703, 737,
   767, 799, 831, 866, 901, 906, 935, 961, 987,1013,1020,1027,1051,1121,
  1126,1146,1197,1233,1240,1246,1254,1262,1267,1270,1274,1278,1281,1284,
  1293,1319,1330,1370,1388,1438,1445,1466,1514,1540,1549,1558,1593,1600,
  1609,1618,1624,1629,1651,1699,1702,1706,1739,1810,1813,1816,1848,1963,
  1975,1980,1986,1990,1993,1999,2002,2006,2014,2022,2030,2035,2042,2047,
  2051,2076,2098,2133,2158,2184,2210,2232,2283,2339,2365,2387,2444,2470,
  2496,2522,2544,2602,2628,2650,2709,2731,2791,2813,2874,2896,2958,2980,
  3044,3066,3128,3197,3219,3287,3313,3339,3365,3391,3417,3443,3469,3495,
  3521,3547,3573,3595,3641,3674,3763,3811,3816,3837,3883,3903,3968,4033,
  4080,4085,4090,4147,4165,4205,4234,4241,4247,4255,4263,4268,4271,4274,
  4277,4281,4287,4328,4333,4337,4341,4344,4348,4385,4419,4491,4496,4516,
  4583,4605,4651,4672,4737,4814,4884,4888,4892,4926,4931,4952,5030,5033,
  5036,5068,5168,5237
};


static const unsigned char ag_fl[] = {
  2,3,0,2,1,1,2,4,5,9,4,5,9,1,7,2,1,1,2,2,2,3,2,2,1,0,3,1,1,1,4,3,2,3,2,
  3,8,2,1,3,0,1,1,3,1,3,1,5,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,2,
  2,1,3,3,1,1,2,2,2,2,1,4,4,1,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,
  1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,3,0,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  9,  9, 11, 11, 12, 12, 12, 12, 13, 13, 13, 13, 23, 23, 23, 23,
   23, 23, 23, 23, 23, 23, 23,  8,  8, 24, 16, 20, 14, 26, 30, 30, 31, 31,
   32, 45, 34, 34, 21, 21, 18, 18, 43, 43, 46, 46, 49, 49, 52, 52, 54, 54,
   56, 56, 58, 58, 60, 60, 62, 62, 64, 64, 66, 66, 68, 68, 70, 70, 70, 72,
   72, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 47, 80, 83, 83, 84, 84, 77,
   77, 77, 77, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 63, 63, 65,
   65, 65, 65, 67, 67, 69, 69, 71, 71, 71, 74, 74, 74,  1,115,115,116,116,
    1,120,120,121,121,  1,197,197,216,216,216,125,125,125,128,128,128,134,
  134,127,127,130,130,215,215,215,131,131,135,135,132,132,133,133,145,145,
  137,137,146,146, 85, 85,244,149,149,150,150,152,152,152,153,153,153,153,
  153,153,153,153,153,153,153,154,154,154,167,168,169,155,155,172,172,217,
  174,174,136,112,112,114,114,114,114,114,114,114,114,114,114,114,114,114,
  114,114,114,114,114,114,114,114,114,114,114,114,114,114,114,114,114,114,
  114,114,114,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,
  119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,
  123,123,123,124,124,124,124,124,124,138,138,139,139,139,142,142,147,147,
  151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,151,
  151,151,151,151,151,151,151,151,151,151,151,151,151,175,175,175,175,175,
  175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,175,
  175,175,175,175,175,175,175,175,181,181,182,182, 15, 19, 17, 22, 25, 27,
   28, 29, 33, 35, 36, 37, 39, 38,  4, 40, 41, 42, 44, 51, 50, 53, 55, 57,
   59, 61, 73, 76, 78, 79, 81, 82,  2,  3,  6, 86, 87, 88, 89, 90, 91, 92,
   93, 94, 95, 96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,
  111,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VRO(AG_WRAP_9 *, ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_9 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 4: VRO(AG_WRAP_6 *, ag_rp_4(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 5: VRO(AG_WRAP_6 *, ag_rp_5(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 6: VNO AG_WRAP_6(ag_rp_6(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 7: VNO AG_WRAP_6(ag_rp_7(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 8: VRO(AG_WRAP_6 *, ag_rp_8(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 9: VNO AG_WRAP_6(ag_rp_9(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 10: VNO AG_WRAP_6(ag_rp_10(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 11: VRO(AG_WRAP_6 *, ag_rp_11(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 12: VNO AG_WRAP_6(ag_rp_12(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 13: VRO(AG_WRAP_6 *, ag_rp_13(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 14: VNO AG_WRAP_6(ag_rp_14(PCB_POINTER)); break;
    case 15: VNO AG_WRAP_6(ag_rp_15(PCB_POINTER)); break;
    case 16: VNO AG_WRAP_6(ag_rp_16(PCB_POINTER)); break;
    case 17: VNO AG_WRAP_6(ag_rp_17(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_6(ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 19: VNO AG_WRAP_9(ag_rp_19(PCB_POINTER)); break;
    case 20: VNO AG_WRAP_9(ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_9 *); break;
    case 21: ag_rp_21(PCB_POINTER); break;
    case 22: ag_rp_22(PCB_POINTER); break;
    case 23: ag_rp_23(PCB_POINTER); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 25: VNO AG_WRAP_6(ag_rp_25(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 26: VNO AG_WRAP_6(ag_rp_26(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 27: VRO(AG_WRAP_6 *, ag_rp_27(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 28: VNO AG_WRAP_6(ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 29: VRO(AG_WRAP_6 *, ag_rp_29(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 30: VNO AG_WRAP_6(ag_rp_30(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_9 *), VW(6, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_9 *); VWD(6, AG_WRAP_4 *); break;
    case 31: VRO(AG_WRAP_9 *, ag_rp_31(PCB_POINTER)); break;
    case 32: VRO(AG_WRAP_4 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 33: VRO(AG_WRAP_9 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_9 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 34: VNO AG_WRAP_6(ag_rp_34(PCB_POINTER)); break;
    case 35: VRO(AG_WRAP_6 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 36: VRO(AG_WRAP_6 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 37: VRO(AG_WRAP_6 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 38: VRO(AG_WRAP_6 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 39: VRO(AG_WRAP_6 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 40: VRO(AG_WRAP_6 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 41: VRO(AG_WRAP_6 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 42: VRO(AG_WRAP_6 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 43: VRO(AG_WRAP_6 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 44: VRO(AG_WRAP_6 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 45: VRO(AG_WRAP_6 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 46: VRO(AG_WRAP_6 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 47: VRO(AG_WRAP_6 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 48: VNO AG_WRAP_6(ag_rp_48(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 49: VNO AG_WRAP_6(ag_rp_49(PCB_POINTER, V(0,(Opcode *)), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 50: VRO(AG_WRAP_6 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 51: VNO AG_WRAP_6(ag_rp_51(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 52: VRO(AG_WRAP_8 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_8 *))); break;
    case 53: VRO(AG_WRAP_6 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 54: VNO AG_WRAP_6(ag_rp_54(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 55: VNO AG_WRAP_6(ag_rp_55(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 56: VRO(AG_WRAP_6 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 57: VRO(AG_WRAP_6 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 58: VNO AG_WRAP_6(ag_rp_58(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 59: VNO AG_WRAP_6(ag_rp_59(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 60: VRO(AG_WRAP_4 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 61: VRO(AG_WRAP_4 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_7 *))); 
            VWD(2, AG_WRAP_7 *); break;
    case 62: VNO AG_WRAP_7(ag_rp_62(PCB_POINTER)); break;
    case 63: VRO(AG_WRAP_6 *, ag_rp_63(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 64: VRO(AG_WRAP_7 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 65: VNO AG_WRAP_8(ag_rp_65(PCB_POINTER, V(0,(long *)))); break;
    case 66: VNO AG_WRAP_8(ag_rp_66(PCB_POINTER, V(0,(double *)))); break;
    case 67: VNO AG_WRAP_8(ag_rp_67(PCB_POINTER, V(0,(int *)))); break;
    case 68: V(0,(Opcode *)) = ag_rp_68(PCB_POINTER); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: V(0,(Opcode *)) = ag_rp_71(PCB_POINTER); break;
    case 72: V(0,(Opcode *)) = ag_rp_72(PCB_POINTER); break;
    case 73: V(0,(Opcode *)) = ag_rp_73(PCB_POINTER); break;
    case 74: V(0,(Opcode *)) = ag_rp_74(PCB_POINTER); break;
    case 75: V(0,(Opcode *)) = ag_rp_75(PCB_POINTER); break;
    case 76: V(0,(Opcode *)) = ag_rp_76(PCB_POINTER); break;
    case 77: V(0,(Opcode *)) = ag_rp_77(PCB_POINTER); break;
    case 78: V(0,(Opcode *)) = ag_rp_78(PCB_POINTER); break;
    case 79: V(0,(Opcode *)) = ag_rp_79(PCB_POINTER); break;
    case 80: V(0,(Opcode *)) = ag_rp_80(PCB_POINTER); break;
    case 81: V(0,(Opcode *)) = ag_rp_81(PCB_POINTER); break;
    case 82: V(0,(Opcode *)) = ag_rp_82(PCB_POINTER); break;
    case 83: V(0,(Opcode *)) = ag_rp_83(PCB_POINTER); break;
    case 84: V(0,(Opcode *)) = ag_rp_84(PCB_POINTER); break;
    case 85: V(0,(Opcode *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(Opcode *)) = ag_rp_86(PCB_POINTER); break;
    case 87: V(0,(Opcode *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(Opcode *)) = ag_rp_88(PCB_POINTER); break;
    case 89: V(0,(Opcode *)) = ag_rp_89(PCB_POINTER); break;
    case 90: V(0,(Opcode *)) = ag_rp_90(PCB_POINTER); break;
    case 91: V(0,(Opcode *)) = ag_rp_91(PCB_POINTER); break;
    case 92: V(0,(Opcode *)) = ag_rp_92(PCB_POINTER); break;
    case 93: V(0,(Opcode *)) = ag_rp_93(PCB_POINTER); break;
    case 94: V(0,(Opcode *)) = ag_rp_94(PCB_POINTER); break;
    case 95: V(0,(Opcode *)) = ag_rp_95(PCB_POINTER); break;
    case 96: VNO AG_WRAP_4(ag_rp_96(PCB_POINTER, V(0,(int *)))); break;
    case 97: VRO(AG_WRAP_4 *, ag_rp_97(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 98: V(0,(double *)) = ag_rp_98(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 99: V(0,(double *)) = ag_rp_99(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 100: V(0,(double *)) = ag_rp_100(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 101: V(0,(double *)) = ag_rp_101(PCB_POINTER, V(0,(double *))); break;
    case 102: V(0,(double *)) = ag_rp_102(PCB_POINTER, V(1,(double *))); break;
    case 103: V(0,(double *)) = ag_rp_103(PCB_POINTER, V(0,(long *))); break;
    case 104: V(0,(double *)) = ag_rp_104(PCB_POINTER, V(0,(long *))); break;
    case 105: V(0,(double *)) = ag_rp_105(PCB_POINTER, V(0,(long *))); break;
    case 106: V(0,(long *)) = ag_rp_106(PCB_POINTER, V(1,(long *))); break;
    case 107: V(0,(long *)) = ag_rp_107(PCB_POINTER, V(1,(long *))); break;
    case 108: V(0,(double *)) = ag_rp_108(PCB_POINTER, V(0,(int *))); break;
    case 109: V(0,(double *)) = ag_rp_109(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 110: V(0,(long *)) = ag_rp_110(PCB_POINTER, V(0,(int *))); break;
    case 111: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 112: V(0,(long *)) = ag_rp_112(PCB_POINTER, V(0,(int *))); break;
    case 113: V(0,(long *)) = ag_rp_113(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 114: V(0,(long *)) = ag_rp_114(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 115: V(0,(long *)) = ag_rp_115(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 116: V(0,(long *)) = ag_rp_116(PCB_POINTER); break;
    case 117: V(0,(long *)) = ag_rp_117(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 118: V(0,(long *)) = ag_rp_118(PCB_POINTER); break;
    case 119: V(0,(long *)) = ag_rp_119(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 120: V(0,(int *)) = ag_rp_120(PCB_POINTER, V(0,(int *))); break;
    case 121: V(0,(int *)) = ag_rp_121(PCB_POINTER, V(0,(int *))); break;
    case 122: VRO(AG_WRAP_5 *, ag_rp_122(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 123: VRO(AG_WRAP_8 *, ag_rp_123(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 124: VNO AG_WRAP_5(ag_rp_124(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 125: VNO AG_WRAP_4(ag_rp_125(PCB_POINTER)); break;
    case 126: VRO(AG_WRAP_4 *, ag_rp_126(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 127: V(0,(int *)) = ag_rp_127(PCB_POINTER); break;
    case 128: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 129: V(0,(int *)) = ag_rp_129(PCB_POINTER); break;
    case 130: V(0,(int *)) = ag_rp_130(PCB_POINTER); break;
    case 131: V(0,(int *)) = ag_rp_131(PCB_POINTER); break;
    case 132: V(0,(int *)) = ag_rp_132(PCB_POINTER); break;
    case 133: V(0,(int *)) = ag_rp_133(PCB_POINTER); break;
    case 134: V(0,(int *)) = ag_rp_134(PCB_POINTER); break;
    case 135: V(0,(int *)) = ag_rp_135(PCB_POINTER); break;
    case 136: V(0,(int *)) = ag_rp_136(PCB_POINTER); break;
    case 137: V(0,(int *)) = ag_rp_137(PCB_POINTER); break;
    case 138: V(0,(int *)) = ag_rp_138(PCB_POINTER, V(1,(int *))); break;
    case 139: V(0,(int *)) = ag_rp_139(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 140: V(0,(int *)) = ag_rp_140(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 141: V(0,(int *)) = ag_rp_141(PCB_POINTER, V(1,(int *))); break;
    case 142: V(0,(int *)) = ag_rp_142(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 143: V(0,(int *)) = ag_rp_143(PCB_POINTER, V(0,(int *))); break;
    case 144: V(0,(int *)) = ag_rp_144(PCB_POINTER, V(0,(int *))); break;
    case 145: V(0,(int *)) = ag_rp_145(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dci_token_names
const char *const dci_token_names[245] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "local statement",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"else\"",
  "WHILE",
  "'('",
  "expression",
  "')'",
  "FOR",
  "optional expression",
  "';'",
  "simple statement",
  "DO",
  "\"while\"",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "function definition",
  "\"local\"",
  "name list",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "\"function\"",
  "function body",
  "conditional expression",
  "lvalue",
  "assignment op",
  "logical or expression",
  "'\\?'",
  "':'",
  "logical and expression",
  "\"||\"",
  "inclusive or expression",
  "\"&&\"",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "equality op",
  "shift expression",
  "relational op",
  "additive expression",
  "shift op",
  "multiplicative expression",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"else\"",
  "')'",
  "'('",
  "';'",
  "\"while\"",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "\"local\"",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'}'",
  "'{'",
  "name",
  "\"dump\"",
  "','",
  "\"print\"",
  "\"function\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "'+'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dci(dci_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dci(dci_pcb_type *PCB_POINTER) {
  init_dci(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


